#pragma once
//  DXACLRule
//
//  The DXACLRule object contains the rules for a single ACL Entry.
//
//  Message Identifier:      DXR
//  Message Range:           7301 - 7399
//  Trace area assignment:   TRACE_ACLRULESET (22)
//  Version:                 3.11.4   Build: 102
//
//  Version History:
//
//  3.5.0		15/08/2009		Initial version
//  3.9.1		03/09/2010		XML specification changed
//  3.9.2		22/09/2010		Various fixes to ACL processing
//  3.11.1		22/10/2011		VS 2011 Updates
//  3.11.3		12/11/2011		Fixes to Explorer and ACL Processor
//  3.11.4		27/11/2011		Fixed an issue with wildcard matching
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2009 - 2011
//

//  Application includes
#include	"../ExecEnvironment.h"					//  DX Run Time

//  Trace Messages
#define		MSG_DXR7301T				"DXR7301T: A new DXACLRule instance has been created @%p."
#define		MSG_DXR7302T				"DXR7302T: A DXACLRule instance @%p has been destroyed."
#define		MSG_DXR7304T				"DXR7304T: Matched entry '%s'(%i) to %s '%s'(%i)."
#define		MSG_DXR7305T				"DXR7305T: Failed to match entry '%s'(%i) to %s '%s'(%i)."
#define		MSG_DXR7306T				"DXR7306T: Failed to match entry '%s'(%i) to %s '%s'(%i)."
#define		MSG_DXR7307T				"DXR7307T: Access level should be updated from %i to %i."
#define		MSG_DXR7308T				"DXR7308T: Access flags should be updated from %i to %i (On: %i Off: %i)."
#define		MSG_DXR7309T				"DXR7309T: Roles should be updated to '%s'."
#define		MSG_DXR7310T				"DXR7310T: Privelege array changes from %s' to %s'."
#define		MSG_DXR7311T				"DXR7311T: Comparing privelege arrays entry: %s' to computed: %s'."
#define		MSG_DXR7312T				"DXR7312T: ACL Entry Type should be updated from %i to %i."

//  Error Messages
#define		MSG_DXR7303E				"DXR7303E: Unable to abbreviate '%s', returning a mismatch."

//  Miscellaneous Definitions
#define		ROLE_BFR_SIZE				512			//  Size of the Role Names Buffer
#define		MAX_PSTRING					50			//  Size of privilege string
#define		TYPE_FLAG_BITS				(ACL_FLAG_SERVER | ACL_FLAG_PERSON | ACL_FLAG_GROUP)

//  Processing states
#define		PS_ACTION_DELETE			1			//  Delete this entry
#define		PS_ACTION_MOD_LEVEL			2			//  Modify this entry (Access Level)
#define		PS_ACTION_MOD_PRIVS			4			//  Modify this entry (Roles)
#define		PS_ACTION_MOD_FLAGS			8			//  Modify this entry (Flags)
#define		PS_ENTRY_EXISTS				16			//  Entry exists in the ACL
#define		PS_ACTION_RENAME			32			//  Rename this entry
#define		PS_ACTION_MOD_TYPE			64			//  Modify this entry (Type)

//  Class Definitions
class DXACLRule
{
public:
	//  Constructor
	DXACLRule(ExecEnvironment *xeParent, int iThreadID);

	//  Destructor
	~DXACLRule(void);

	//  Public Members
	DXACLRule				*arNext;						//  The Next ACL Rule in the chain
	WORD					AccessLevel;					//  The Access Level for this rule
	WORD					AccessFlagsOn;					//  Access Flags that must be ON
	WORD					AccessFlagsOff;					//  Access Flags that must be set OFF
	WORD					AccessFlagsUpdate;				//  Updated Access Flags
	WORD					EntryType;						//  The type of ACL Entry
	WORD					ProcState;						//  Processing state for this entry
	BOOL					NameIsPattern;					//  Flag indications that the name element is a pattern
	ACL_PRIVILEGES			apSetOn;						//  Privileges to be set on
	ACL_PRIVILEGES			apSetOff;						//  Privileges to be set off
	ACL_PRIVILEGES			apUpdate;						//  Updated privileges
	char					szEntryName[MAXUSERNAME + 1];	//  User Name
	char					szNewName[MAXUSERNAME + 1];		//  New user name
	char					cRoleSetBuffer[ROLE_BFR_SIZE + 1];	//  Roles to be Set/Unset Buffer

	//  Public Functions
	int	checkSettings(WORD wAccessLevel, ACL_PRIVILEGES *apEntry, WORD wAccessFlags, int iThreadID);  // Check settings against rule
	BOOL matchEntry(char *szEntry, WORD wFlags, int iThreadID);

private:
	//  Private Members
	ExecEnvironment			*xeLocal;						//  DX Run Time
	int						ParentThread;					//  Parent Thread

	//  Private Functions
};
