#pragma once
//  DXACLRuleSet
//
//  The DXACLRuleSet object describes a set of rules that can be applied to the ACL of a database in
//  order to achieve the desired content of the ACL. The object contains the entry points that are
//  used to coerce the ACL to the desired state.
//
//  Message Identifier:      DXR
//  Message Range:           7201 - 7299
//  Trace area assignment:   TRACE_ACLRULESET (22)  - for execution
//							 TRACE_ACLRSPARSER (21) - for validation
//  Version:                 3.14.0   Build: 108
//
//  Version History:
//
//  3.5.0		15/08/2009		Initial version
//  3.9.1		03/09/2010		XML specification changed
//  3.9.2		22/09/2010		Various fixes to ACL processing
//  3.10.0		01/10/2010		Wildcard & multi-thread execution added to DX processing
//								DXNoteScanner incorporated in kernel
//  3.10.7		17/05/2011		Fixed an issue with ACL processing
//  3.11.1		22/10/2011		VS 2011 Updates
//  3.11.3		12/11/2011		Fixes to Explorer and ACL Processor
//	3.14.0		21/01/2015		x64 compatability fixes
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2009 - 2015
//

//  Application includes
#include	"../ExecEnvironment.h"					//  DX Run Time
#include	"DXACLRule.h"							//  An Individual ACL Rule

//  Local Definitions
//  Coersion Process Phases
#define		CPHASE_FORBIDDEN		1
#define		CPHASE_ALLOWED			2
#define		CPHASE_COMPULSORY		3
#define		CPHASE_ADMIN			4

//  Miscellaneous definitions
#define		DEFAULT_ADMIN_FLAGS		(ACL_FLAG_NODELETE | ACL_FLAG_NOREPLICATE)
#define		DEFAULT_ACL_ENTRYNAME	"-Default-"

//  Error Messages
#define		MSG_DXR7203E			"DXR7203E: The Admin Server entry contains conflicting flag settings."
#define		MSG_DXR7204E			"DXR7204E: The compulsory entry for '%s' contains conflicting flag settings."
#define		MSG_DXR7205E			"DXR7205E: There are duplicate entries for '%s' in the compulsory rules."
#define		MSG_DXR7206E			"DXR7206E: The forbidden entry for '%s' contains conflicting flag settings."
#define		MSG_DXR7207E			"DXR7207E: There are duplicate entries for '%s' in the forbidden rules."
#define		MSG_DXR7208E			"DXR7208E: Entry '%s' in the forbidden rules also matches an entry in the compulsory rules."
#define		MSG_DXR7210E			"DXR7210E: The permitted entry for '%s' contains conflicting flag settings."
#define		MSG_DXR7211E			"DXR7211E: There are duplicate entries for '%s' in the permitted rules."
#define		MSG_DXR7222E			"DXR7222E: Unable to enumerate the ACL."
#define		MSG_DXR7226E			"DXR7226E: Unable to remove entry '%s' from the ACL."
#define		MSG_DXR7231E			"DXR7231E: The ACL could not be bound to the current rule set."
#define		MSG_DXR7232E			"DXR7232E: Unable to get a list of all roles in the current ACL."
#define		MSG_DXR7233E			"DXR7233E: Failed to bind the ACL to rule #%i on the %s chain."
#define		MSG_DXR7236E			"DXR7236E: Failed to update the ACL entry for '%s'."
#define		MSG_DXR7237E			"DXR7237E: Unable to canonicalize '%s'."
#define		MSG_DXR7238E			"DXR7238E: Unable to add the ACL entry for '%s'."
#define		MSG_DXR7243E			"DXR7243E: The AdminServer rule conflicts with one or more forbidden entries."
#define		MSG_DXR7244E			"DXR7244E: The AdminServer rule conflicts with one or more compulsory entries."
#define		MSG_DXR7245E			"DXR7245E: Invalid attempt to coerce ACL without supplying a valid ACL Handle."
#define		MSG_DXR7246E			"DXR7246E: Failed to rename the ACL entry for '%s'."

//  Warning Messages
#define		MSG_DXR7209W			"DXR7209W: Entry '%s' in the forbidden rules also matches an entry in the permitted rules."

//  Information Messages
#define		MSG_DXR7225I			"DXR7225I: ACL entry '%s' is being purged as it is forbidden."
#define		MSG_DXR7229I			"DXR7229I: ACL entry '%s' has been permitted."
#define		MSG_DXR7239I			"DXR7239I: ACL entry '%s' has been added."
#define		MSG_DXR7240I			"DXR7240I: The ACL has had %i updates applied and should be saved."
#define		MSG_DXR7247I			"DXR7247I: ACL entry '%s' has been updated."

//  Trace Messages
#define		MSG_DXR7201T			"DXR7201T: A new DXACLRuleSet instance has been created @%p."
#define		MSG_DXR7202T			"DXR7202T: A DXACLRuleSet instance @%p has been destroyed."
#define		MSG_DXR7212T			"DXR7212T: Validating Admin Server entry."
#define		MSG_DXR7213T			"DXR7213T: Validating compulsory ACL entries."
#define		MSG_DXR7214T			"DXR7214T: Validating forbidden ACL entries."
#define		MSG_DXR7215T			"DXR7215T: Validating permitted ACL entries."
#define		MSG_DXR7216T			"DXR7216T: Validating Admin Server entry for '%s'."
#define		MSG_DXR7217T			"DXR7217T: Validating compulsory ACL entry for '%s'."
#define		MSG_DXR7218T			"DXR7218T: Validating forbidden ACL entry for '%s'."
#define		MSG_DXR7219T			"DXR7219T: Validating permitted ACL entry for '%s'."
#define		MSG_DXR7220T			"DXR7220T: Starting to coerce ACL."
#define		MSG_DXR7221T			"DXR7221T: Starting enumeration of ACL for %s rules processing."
#define		MSG_DXR7223T			"DXR7223T: Start of processing for ACL entry '%s' for processing phase %i."
#define		MSG_DXR7224T			"DXR7224T: ACL entry '%s' has been identified as forbidden by rule %i."
#define		MSG_DXR7227T			"DXR7227T: Starting enumeration of ACL for Permitted rules processing."
#define		MSG_DXR7228T			"DXR7228T: ACL entry '%s' has been identified as not permitted."
#define		MSG_DXR7230T			"DXR7230T: ACL entry '%s' needs updating (action: %i) to match rule %i."
#define		MSG_DXR7234T			"DXR7234T: Rule chain %s has bound the ACL to %i rules."
#define		MSG_DXR7235T			"DXR7235T: Rule for '%s' has bound %i roles on and %i roles off, %i roles were not found."
#define		MSG_DXR7241T			"DXR7241T: Attempting to bind '%s'."
#define		MSG_DXR7242T			"DXR7242T: Bound Privs ON: %s' and OFF: %s'."

//  Class Definition
class DXACLRuleSet
{
public:
	//  Constructor
	DXACLRuleSet(ExecEnvironment *xeParent, int iThreadID);

	//  Destructor
	~DXACLRuleSet(void);

	//  Public Members
	DXACLRule		*FirstAdminServerRule;				//  First Admin Server Rule
	DXACLRule		*FirstCompulsoryRule;				//  First Compulsory Rule
	DXACLRule		*FirstForbiddenRule;				//  First Forbidden Rule
	DXACLRule		*FirstPermittedRule;				//  First Permitted Rule
	DXACLRule		*FirstActionRule;					//  First dummy rule to be actioned
	ACL_PRIVILEGES	apAllOn;							//  Mask of all available privileges

	//  Public Functions
	BOOL validate(int iThreadID);						//  Validate the Rule Set
	BOOL coerce(DHANDLE hACL, int iThreadID);			//  Coerce the ACL to the Rules
	void clone(DXACLRuleSet *arsClone, int iThreadID);	//  Create a clone of this Rule Set
	static void LNCALLBACK WrapperACLEnum(void *pObj, char *szName, WORD wAccessLevel, ACL_PRIVILEGES *apEntry, WORD wAccessFlags);

private:
	//  Private Members
	ExecEnvironment	*xeLocal;							//  DX Run Time
	int				ParentThread;						//  Parent Thread ID
	WORD			ProcPhase;							//  Processing Phase
	int				UpdateCount;						//  Count of updates applied

	//  Private Functions
	BOOL validateAdminServer(int iThreadID);			//  Validate the Admin Server entry
	BOOL validateCompulsoryEntries(int iThreadID);		//  Validate the Compulsory entries
	BOOL validateForbiddenEntries(int iThreadID);		//  Validate the Forbidden entries
	BOOL validatePermittedEntries(int iThreadID);		//  Validate the Permitted entries
	int	countEntries(DXACLRule *arStart, char *szEntry, WORD wType, int iThreadID);  //  Count entries
	void clearRuleChainState(DXACLRule *arStart);		//  Clear the processing state on a rule chain
	void actionACLEnumForbidden(char *szName, WORD wAccessLevel, ACL_PRIVILEGES *apEntry, WORD wAccessFlags, int iThreadID);
	void actionACLEnumAllowed(char *szName, WORD wAccessLevel, ACL_PRIVILEGES *apEntry, WORD wAccessFlags, int iThreadID);
	void actionACLEnumAdmin(char *szName, WORD wAccessLevel, ACL_PRIVILEGES *apEntry, WORD wAccessFlags, int iThreadID);
	void vectorACLEnum(char *szName, WORD wAccessLevel, ACL_PRIVILEGES *apEntry, WORD wAccessFlags, int iThreadID);
	void purgeForbiddenEntries(DHANDLE hACL, int iThreadID);			//  Purge forbidden entries
	BOOL bindACL(DHANDLE hACL, int iThreadID);			//  Bind the ACL to the RuleSet
	BOOL bindRuleChain(DXACLRule *arStart, char *szRoles, char *szChain, int iThreadID);	//  Bind the ACL roles to a Rule Chain
	BOOL bindRule(DXACLRule *arCurrent, char *szRoles, int iThreadID);	//  Bind the ACL roles to an individual Rule
	int getPrivNum(char *sCurrentRole, char *szAllRoles);   //  Return the Privilege number for a given role
	void updatePermittedEntries(DHANDLE hACL, int iThreadID);	//  Update the permitted entries
	void addMissingEntries(DHANDLE hACL, int iThreadID);			//  Add any missing compulsory entries
	void addAdminServer(DHANDLE hACL, int iThreadID);			//  Add missing Admin Server
	DXACLRule * cloneRuleChain(DXACLRule *arStart, int iThreadID);		//  Clone an ACL rule chain
	DXACLRule * cloneRule(DXACLRule *arCurrent, int iThreadID);		//  Clone a single ACL Rule
};

//  The following class defines objects that are passed into ACL Enumerators
class DXACLEnumInfo
{
public:
	//  Constructor
	DXACLEnumInfo(DXACLRuleSet *rsParent, int iThreadID);

	//  Destructor
	~DXACLEnumInfo(void);

	//  Public Members
	DXACLRuleSet		*rsLocal;				//  The parent Rule Set
	int					iThread;				//  ID of the invoking thread
};
