#pragma once
//  DbMover
//
//  This class provides the Database Move functions used in Domono eXplorer (DX) utilities
//
//  Message Identifier:      DXR
//  Message Range:           6001 - 6499
//  Trace area assignment:   TRACE_DBM (25)
//  DX Version:				 3.14.0
//  Version:                 3.14.0  Build: 108
//
//  Version History:
//
//  3.6.2		14/10/2009		Prepared for QMove 1.0
//  3.6.3		19/12/2009		Updates to support additional ops for QMove 1.1.0
//  3.6.4		23/12/2009		Numerous cosmetic fixes for QMove 1.1.1
//  3.7.0		25/12/2009		UNIX compatability fixes in QMove
//  3.7.3		06/02/2010		Handle Db & Dir Links
//  3.10.2		18/11/2010		MTIO Support added to DbMover
//  3.11.1		22/10/2011		VS 2011 Updates
//	3.14.0		19/01/2015		x64 compatability fixes
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2009 - 2015
//

//  Runtime Includes
#include "../MTX/MTExecutive.h"
#include "../Threads/Runnable.h"

//  Application Includes
#include "MoveRequest.h"
#include "PartMoveRequest.h"

#ifdef UNIX
#include <fcntl.h>										//  Include POSIX File I/O
#endif

//  Trace Messages
#define			MSG_DXR6001T		"DXR6001T: A new DbMover object has been created @%p."
#define			MSG_DXR6002T		"DXR6002T: A DbMover object @%p has been destroyed."
#define			MSG_DXR6116T		"DXR6116T: Mover dispatched with RQ: @%p, Function: %i, PRQ: @%p."
#define			MSG_DXR6117T		"DXR6117T: DbMover has been entered for %i processing."
#define			MSG_DXR6124T		"DXR6124T: A short block read (%i bytes) was encountered on arena %i, EOF assumed."
#define			MSG_DXR6125T		"DXR6125T: Purging all Copy Arena requests due to loss of the Run Permit."

//  Information Messages
#define			MSG_DXR6064I		"DXR6064I: Applying change quantum %i, size = %i."
#define			MSG_DXR6069I		"DXR6069I: Database '%s' has been moved to '%s', database is online."
#define			MSG_DXR6085I		"DXR6069I: Database '%s' will be moved to '%s'."
#define			MSG_DXR6086I		"DXR6086I: Database '%s' has been taken offline and deleted after %i attempts."
#define			MSG_DXR6087I		"DXR6087I: Database change info volume is %i:%i."
#define			MSG_DXR6089I		"DXR6089I: Database copy '%s' has been refreshed and will be brought back online."
#define			MSG_DXR6092I		"DXR6092I: Database '%s' will be backed up to '%s'."
#define			MSG_DXR6093I		"DXR6093I: Database '%s' will be deleted."
#define			MSG_DXR6094I		"DXR6094I: Database '%s' will be retired to '%s'."
#define			MSG_DXR6095I		"DXR6095I: Database '%s' has been deleted."
#define			MSG_DXR6096I		"DXR6096I: Database '%s' has been backed up to '%s'."
#define			MSG_DXR6097I		"DXR6097I: Database '%s' has been retired to '%s'."
#define			MSG_DXR6098I		"DXR6098I: Database Log ID: %.8lX:%.8lX-%.8lX:%.8lX, DBIID: %.8lX:%.8lX-%.8lX:%.8lX. First Log extent %i."
#define			MSG_DXR6101I		"DXR6101I: The source database has been re-directed from '%s'."
#define			MSG_DXR6102I		"DXR6102I: The source database has been re-directed to '%s'."
#define			MSG_DXR6103I		"DXR6103I: The link file '%s' could not be opened, assuming directory re-direction."
#define			MSG_DXR6110I		"DXR6110I: Database copy '%s' has been refreshed, backup is up to date."
#define			MSG_DXR6111I		"DXR6111I: Initiating copy of %i%s %s using %i Kb. transfer buffer."
#define			MSG_DXR6112I		"DXR6112I: Copy of %i%s %s completed using %i I/O operations in %i seconds and took %i checkpoints."
#define			MSG_DXR6118I		"DXR6118I: The copy will be performed using the Larga File (asynchronous) copier and will copy %i x %i Mb arenas."
#define			MSG_DXR6122I		"DXR6122I: Copy of %i x %i Mb Arenas completed in %i seconds and took %i checkpoints."
#define			MSG_DXR6127I		"DXR6127I: Database link '%s' has been deleted."

//  Warning Messages
#define			MSG_DXR6057W		"DXR6057W: Database '%s' is not logged - Skipping move."
#define			MSG_DXR6108W		"DXR6108W: The specified database is already re-directed via a directory link, update the link when all databases are moved."
#define			MSG_DXR6113W		"DXR6113W: Recovery manager has signalled that transaction log space is becoming critical."
#define			MSG_DXR6114W		"DXR6114W: An error was detected in the recovery manager refresh processing, a fixup will be needed to clear the error."

//  Error Messages
#define			MSG_DXR6054E		"DXR6054E: The source database '%s' could not be opened, move abandoned."
#define			MSG_DXR6055E		"DXR6055E: Unable to start recording changes on '%s', move abandoned."
#define			MSG_DXR6056E		"DXR6056E: Unable to get Logging Information for '%s', move abandoned."
#define			MSG_DXR6058E		"DXR6058E: Unable to copy file '%s', move abandoned."
#define			MSG_DXR6059E		"DXR6059E: Unable to stop recording changes on '%s', move abandoned."
#define			MSG_DXR6060E		"DXR6060E: Unable to get change info size for '%s', move abandoned."
#define			MSG_DXR6061E		"DXR6061E: Unable to create a change context for '%s', move abandoned."
#define			MSG_DXR6062E		"DXR6062E: Unable to allocate a %i byte buffer for applying changes, move abandoned."
#define			MSG_DXR6063E		"DXR6063E: Unable to get change quantum for '%s', move abandoned."
#define			MSG_DXR6065E		"DXR6065E: Unable to apply change quantum to '%s', move abandoned."
#define			MSG_DXR6066E		"DXR6066E: Unable to take '%s' offline and delete after %i attempts, move abandoned."
#define			MSG_DXR6067E		"DXR6067E: Unable to create the DbLink '%s', move incomplete."
#define			MSG_DXR6068E		"DXR6068E: Unable to bring database '%s' back online, move incomplete."
#define			MSG_DXR6070E		"DXR6070E: Failed to open '%s' for copying from."
#define			MSG_DXR6071E		"DXR6071E: Failed to create '%s' for copying to."
#define			MSG_DXR6072E		"DXR6072E: Unable to allocate a %i Kb buffer for copying database."
#define			MSG_DXR6073E		"DXR6073E: Unable to set the position for the source file."
#define			MSG_DXR6074E		"DXR6074E: An error was encountered while reading %i bytes from the source database."
#define			MSG_DXR6075E		"DXR6075E: Attempting to read %i bytes from the source database returned only %i bytes."
#define			MSG_DXR6076E		"DXR6076E: Unable to notify the backup high watermark."
#define			MSG_DXR6077E		"DXR6077E: An error was encountered while writing %i bytes to the target database."
#define			MSG_DXR6078E		"DXR6078E: Attempting to write %i bytes to the target database wrote only %i bytes."
#define			MSG_DXR6079E		"DXR6079E: Unable to close the target file after copying."
#define			MSG_DXR6080E		"DXR6080E: Unable to close the source file after copying."
#define			MSG_DXR6081E		"DXR6081E: Failed to create '%s' for writing as a link."
#define			MSG_DXR6082E		"DXR6082E: An error was encountered while writing %i bytes to the database link file."
#define			MSG_DXR6083E		"DXR6083E: Attempting to write %i bytes to the database link file wrote only %i bytes."
#define			MSG_DXR6084E		"DXR6084E: Unable to close the database link file after writing."
#define			MSG_DXR6088E		"DXR6088E: An error was encountered while refreshing '%s'."
#define			MSG_DXR6090E		"DXR6090E: Target directory '%s' does not exist and could not be created, move abandoned."
#define			MSG_DXR6091E		"DXR6091E: Create failed for directory '%s'"
#define			MSG_DXR6099E		"DXR6099E: The move operation does not appear logical, move abandoned."
#define			MSG_DXR6100E		"DXR6100E: Unable to determine the path of the source database."
#define			MSG_DXR6104E		"DXR6104E: The database link file is empty."
#define			MSG_DXR6105E		"DXR6105E: The database link file appears to be a valid NSF database file."
#define			MSG_DXR6106E		"DXR6106E: The expected name of the link file has changed from: '%s'."
#define			MSG_DXR6107E		"DXR6107E: The expected name of the link file has changed to: '%s'."
#define			MSG_DXR6109E		"DXR6109E: The requested move is attempting to copy the source file to itself, move abandoned."
#define			MSG_DXR6115E		"DXR6115E: Please run a fixup -J on database '%s'."
#define			MSG_DXR6119E		"DXR6119E: Unable to set the position for the target file."
#define			MSG_DXR6120E		"DXR6120E: Unable to extend the EOF on the target file."
#define			MSG_DXR6121E		"DXR6121E: Unable to post a request to perform an arena copy action."
#define			MSG_DXR6123E		"DXR6123E: Unable to obtain a Local File Handle to copy arena: %i."
#define			MSG_DXR6126E		"DXR6126E: Global run permit has been revoked, abandoning transaction."
#define			MSG_DXR6128E		"DXR6128E: Unable to delete database link '%s'."

//  Local Definitons
#define			BYTES_PER_KB			1024			//  Divisor bytes per kilobyte
#define			KB_PER_MB				1024			//  Divisor kilobytes per megabyte
#define			MB_PER_GB				1024			//  Divisor megabytes per gigabyte
#define			DB_REDIRECT_NONE		1				//  Source D/B is not re-directed
#define			DB_REDIRECT_DB			2				//  Source D/B is re-directed at the database level
#define			DB_REDIRECT_DIR			3				//  Source D/B is re-directed at the directory level
#define			CHG_BUFFER_SIZE			32*1024			//  32k change buffer
#define			READ_CKPT_INTERVAL		10				//  Checkpoint every 10 reads
#define			DB_OFFLINE_WAIT			5000			//  5 secods wait for database to come offline
#define			MAX_TAKEOFF_ATTEMPTS	100				//  100 attempts to get the database offline
#define			DISP_SIZE_GB			"Gb."			//  Gigabytes
#define			DISP_SIZE_MB			"Mb."			//  Megabytes
#define			DISP_SIZE_KB			"Kb."			//  Kilobytes
#define			INLINE_COPY_LIMMB		50				//  50 Mb limit for inline copies
#define			ARENA_TABLE_SIZE		4096			//  32k bit array
#define			REJOIN_WAIT				20				//  20 Ms wait in rejoin loop

class DbMover :
	Runnable
{
public:
	//  Default Constructor
	DbMover(void);

	//  Constructor
	DbMover(MTExecutive *xeParent, int iThreadID);

	//  Destructor
	~DbMover(void);

	//  Public Members
	int			OwningThread;											//  Creating Thread ID

	//  Public Functions
	void ExecuteThisRequest(void *pReqObject, int iThreadID);			//  Multi thread request entry point
	BOOL MoveThisDb(MoveRequest *mrRQ, int iThreadID);					//  Entry point to initiate moving

private:
	//  Private Members

	//  Private Functions
	void PerformMoveAction(PartMoveRequest *pmrqRQ, int iThreadID);		//  Perform a move action
	void PerformMoveArena(PartMoveRequest *pmrqRQ, int iThreadID);			//  Perform a move arena action
	BOOL checkCopy(MoveRequest *mrRQ, char *szIn, char *szOut, char *szLink, DBHANDLE hdbSrc, int *Config, int iThreadID);
	BOOL copyOSFile(MoveRequest *mrUOW, char *szIn, char *szOut, DBHANDLE hdbSrc, DHANDLE hBUContext, DWORD SizeHi, DWORD SizeLo, int iThreadID);
	BOOL copyLargeOSFile(MoveRequest *mrUOW, char *szIn, char *szOut, DBHANDLE hdbSrc, DHANDLE hBUContext, DWORD SizeHi, DWORD SizeLo, int iThreadID);
	BOOL createDbLink(MoveRequest *mrRQ, char *szDb, char *szLink, int iThreadID);
	BOOL createDirectory(MoveRequest *mrRQ, char *szDir, int iThreadID);
	void normalizePath(char *szIn);
	DWORD markArenaCompleted(DWORD Arena, BYTE *pTable, int iTableSize);	// Mark arena completed
	void DrainRequests(MoveRequest *mrRQ, int iThreadID);					//  Drain all requests		

protected:
	//  Protected Members
	MTExecutive			*xeLocal;											//  Multi-Threading Runtime

	//  Protected Functions
	BOOL virtual ProxyLogMessage(MoveRequest *mrRQ, char *szMsg, int iThreadID);			//  Proxy Log interface
};
