#pragma once
//  MoveRequest
//
//  This class contains the transaction data for a DbMover execution
//
//  Message Identifier:      DXR
//  Message Range:           6000 - 6499
//  Trace area assignment:   None
//  DX Version:				 3.14.0
//  Version:                 3.14.0  Build: 108
//
//  Version History:
//
//  3.6.2		14/11/2009		Re-based on DX version 3.6.2
//  3.6.3		19/12/2009		Updates to support additional ops for QMove 1.1.0
//  3.10.2		15/11/2010		Suppport for MTIO copying added
//  3.14.0		21/01/2015		x64 compatability fixes
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2009 - 2015
//

//  Application Includes
#include  "../ExecEnvironment.h"

//  Local Definitions
#define			DEFAULT_COPYBFR_SIZE			32				//  Default copy buffer size 32k

class MoveRequest
{
public:
	//  Constructor
	MoveRequest(void);

	//  Destructor
	~MoveRequest(void);

	//  Public Members
	int			ReturnCode;												//  Completion code from the move
	UINT		CopyBfrKb;												//  Copy Buffer size Kb
	UINT		ArenaSize;												//  Arena Size Mb
	BOOL		PermitRun;												//  Run Permit Flag
	BOOL		PerformCopy;											//  Copy action is required
	BOOL		PerformDelete;											//  Delete action is required
	BOOL		PerformLink;											//  Link action is required
	BOOL		UseLFC;													//  Enable the Large File Copier
	
	//  Retry controls
	BOOL		AllowRetry;												//  Show if request can be retried
	int			RetryCount;												//  Retry iteration count
	int			RetryLimit;												//  Retry iteration limit
	int			RestartActions;											//  Restart Recovery actions
	int			FaultTolerance;											//  Fault Tolerance

	//  Statistics
	//  Section 1 -- stats from the move phase
	DWORD		dwDataDocsCopied;										//  Data documents moved

	//  Section 2 -- summary statistics
	DWORD		dwTotalElapsed;											//  Total elapsed time
	DWORD		dwSizeKB;												//  Database Size in KB

	char		szTransID[MAX_DATABASE + 1];							//  Request Transaction ID
	char		szSourceDatabase[MAX_DATABASE + 1];						//  Source Database
	char		szTargetDir[MAX_DATABASE + 1];							//  Target directory

};
