//  PartCopyRequest
//
//  This class is used to communicate partial database copy requests between the threads in the
//  database copier class.
//
//  Message Identifier:      DXR
//  Message Range:           5001 - 5499
//  Trace area assignment:   TRACE_DBC (23)
//  DX Version:				 3.13.0
//  Version:                 3.13.0  Build: 109
//
//  Version History:
//
//  1.0.0		10/02/2001		Initial version
//  3.5.3		11/09/2209		Re-based ready for 4.0.0
//  3.10.1		15/10/2010		Asynch Transaction Handler added
//  3.10.9		07/06/2011		One time replication added
//								Attitional error recovery
//	3.13.0		20/12/2013		Advanced Target Database Properties
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2001 - 2014
//

//  Application Includes
#include "PartCopyRequest.h"

PartCopyRequest::PartCopyRequest(CopyRequest *crOriginator)
: CopyAction(0)
, CompletionStatus(0)
, CopyNoteCount(0)
, dwNotesCopied(0)
, dwStubsCopied(0)
, dwCopyRetries(0)
, dwCopyAccessErrors(0)
, dwCopyOtherErrors(0)
, pcrqCopyDesign(NULL)
, pcrqCopyData(NULL)
, dbcCopier(NULL)
, iProcThread(0)
{
	//  Constructor
	Parent = crOriginator;
}

PartCopyRequest::~PartCopyRequest(void)
{
	//  Destructor
}

//  AccumulateTotals
//
//  This function will accumulate run stas from a child request
//
//  Parameters
//
//  1  -  PartCopyRequest *		-  Pointer to the child request
//
//  Returns
//
//  API Use Notes:
//
//
void PartCopyRequest::AccumulateTotals(PartCopyRequest *pcrqChild)
{
	dwNotesCopied += pcrqChild->dwNotesCopied;
	dwStubsCopied += pcrqChild->dwStubsCopied;
	dwCopyRetries += pcrqChild->dwCopyRetries;
	dwCopyAccessErrors += pcrqChild->dwCopyAccessErrors;
	dwCopyOtherErrors += pcrqChild->dwCopyOtherErrors;
}
