#pragma once
//  PartCopyRequest
//
//  This class is used to communicate partial database copy requests between the threads in the
//  database copier class.
//
//  Message Identifier:      DXR
//  Message Range:           5001 - 5499
//  Trace area assignment:   TRACE_DBC (23)
//  DX Version:				 3.13.0
//  Version:                 3.13.0  Build: 109
//
//  Version History:
//
//  1.0.0		10/02/2001		Initial version
//  3.5.3		11/09/2209		Re-based ready for 4.0.0
//  3.7.1		06/01/2010		Prevent a race condition in Rejoin processing
//  3.7.5		30/03/2010		Improved Error Handling in DbCopier
//  3.10.1		15/10/2010		Asynch Transaction Handler added
//  3.10.9		07/06/2011		One time replication added
//								Attitional error recovery
//	3.13.0		20/12/2013		Advanced Target Database Properties
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2001 - 2014
//

//  Application Includes
#include	"CopyRequest.h"						//  Parent CopyRequest

//  Local Constants
#define		COPY_NOTE_MULTIPLEX		100			//  Batch size for note copy operations
#define		COPY_MIN_MULTIPLEX		5			//  Minimum setting of the multiplexor

//  Copy Action Codes
#define		COPY_DESIGN				1			//  Copy the complete design
#define		COPY_DESIGN_NOTES		2			//  Copy some design notes
#define		COPY_DATA				3			//  Copy all data notes
#define		COPY_DATA_NOTES			4			//  Copy some data notes
#define		COPY_MAINTAIN_FOLDERS	5			//  Update the contents of all folders
#define		COPY_MAINTAIN_FOLDER	6			//  Update contents of a single folder
#define		COPY_FTI				7			//  Copy the Full Text Index definition
#define		UPGRADE_DESIGN			8			//  Upgrade the design of the database
#define		COPY_BUILD_INDEXES		9			//  Build all of the view indexes in the database
#define		COPY_BUILD_INDEX		10			//  Build a single view index in the database
#define		RUN_AGENT				11			//  Run an agent in the database
#define		CHECK_RECOMPRESS		12			//  Check notes for recompression
#define		RECOMPRESS_NOTE			13			//  Recompress attachments in note
#define		COPY_DATABASE			100			//  Request is passing a top-level copy request
#define		COPY_FREED				-1			//  Mark the parm object as freed

//  Cmpletion Status Codes
#define		ACTION_COMPLETED		0			//  Action was completed ok
#define		ACTION_INPROGRESS		1			//  Action is in progress
#define		ACTION_WARN				4			//  Action was completed but with warnings
#define		ACTION_FAILED			8			//  Action failed
#define		ACTION_RECOVER			12			//  Action failed - but may be recovered
#define		ACTION_ABORT			16			//  Abort any following actions (and quench current ones)

class PartCopyRequest
{
public:
	//  Constructor
	PartCopyRequest(CopyRequest *crOriginator);

	//  Destructor
	~PartCopyRequest(void);

	//  Public Members
	CopyRequest *Parent;							//  Parent Copy Request
	PartCopyRequest *pcrqCopyDesign;				//  Design Copy Part Request
	PartCopyRequest *pcrqCopyData;					//  Data Copy Part Request
	void *dbcCopier;								//  Pointer to the current D/B Copier
	int CopyAction;									//  Requested Copy Action
	int CompletionStatus;							//  Completeion Status of the request
	int CopyNoteCount;								//  Number of notes to copy in this request
	int	iProcThread;								//  Thread processing this request
	NOTEID nidToCopy[COPY_NOTE_MULTIPLEX];			//  Array of NoteIDs to copy

	//  Stats
	DWORD dwNotesCopied;							//  Count of notes copied
	DWORD dwStubsCopied;							//  Count of stubs copied
	DWORD dwCopyRetries;							//  Count of note copy operations retried
	DWORD dwCopyAccessErrors;						//  Count of note copy operations failed - access errors
	DWORD dwCopyOtherErrors;						//  Count of note copy operations failed - other errors

	//  Public Functions
	void AccumulateTotals(PartCopyRequest *pcrqChild);   //  Accumulate totals from a child request
};
