#pragma once
//  PartMoveRequest
//
//  This class is used to communicate partial database copy requests between the threads in the
//  database mover class.
//
//  Message Identifier:      DXR
//  Message Range:           6001 - 6499
//  Trace area assignment:   None
//  DX Version:				 3.10.2
//  Version:                 3.10.2  Build: 82
//
//  Version History:
//
//  3.10.2		15/11/2010		Initial Version
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2010
//

//  Application Includes
#include	"MoveRequest.h"						//  Db Move Request

//  Local Constants

//  Move Action Codes
#define		MOVE_ARENA				1			//  Request to Copy/Move and Arena
#define		MOVE_DATABASE			100			//  Request is passing a top-level move request
#define		MOVE_FREED				-1			//  Mark the parm object as freed

//  Cmpletion Status Codes
#define		ACTION_COMPLETED		0			//  Action was completed ok
#define		ACTION_INPROGRESS		1			//  Action is in progress
#define		ACTION_WARN				4			//  Action was completed but with warnings
#define		ACTION_FAILED			8			//  Action failed
#define		ACTION_ABORT			16			//  Abort any following actions (and quench current ones)

class PartMoveRequest
{
public:
	//  Constructor
	PartMoveRequest(MoveRequest *mrOriginator);

	//  Destructor
	~PartMoveRequest(void);

	//  Public Members

	//  Request Definition Elements
	MoveRequest *Parent;							//  Parent Move Request
	void *dbmMover;									//  Pointer to the current D/B Mover
	int MoveAction;									//  Requested Move Action
	int CompletionStatus;							//  Completeion Status of the request
	
	//  Move Arena Elements
	char	*szIn;									//  Pointer to source file name
	char	*szOut;									//  Pointer to target file name
#ifdef WIN32
	HANDLE hfNativeSource;							//  Native Handle of the source file
	HANDLE hfNativeTarget;							//  Native Handle of the target file
	DWORD dwSourceAccess;							//  Access Flags for source
	DWORD dwSourceSharing;							//  Sharing Flags for source
	DWORD dwTargetAccess;							//  Access Flags for target
	DWORD dwTargetSharing;							//  Sharing Flags for target
#endif
#ifdef UNIX
	int fNativeSource;								//  Native Handle of the source file
	int fNativeTarget;								//  Native Handle of the target file
	int iSourceFlags;								//  Open flags for source
	int iTargetFlags;								//  Open flags for target
#endif
	DWORD	ArenaNo;								//  Arena Identifier
	DWORD	ArenaSizeKb;							//  Copy Arena Size (in Kb)
	DWORD	ArenaOffsetHi;							//  Arena Offset (High part)
	DWORD	ArenaOffsetLo;							//  Arena Offset (Low part)
};
