#pragma once
//  DbModifier
//
//  This class provides the Database Modifier functions used in Domono eXplorer (DX) utilities
//
//  Message Identifier:      DXR
//  Message Range:           7501 - 7699
//  Trace area assignment:   TRACE_DBP (29)
//  DX Version:				 3.11.1
//  Version:                 3.11.1  Build: 99
//
//  Version History:
//
//  3.10.3		13/12/2010		Initial version
//  3.11.1		22/10/2011		VS 2011 Updates
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2010 - 2011
//

//  Runtime Includes
#include "../MTX/MTExecutive.h"
#include "../Threads/Runnable.h"

//  Application Includes
#include "ModRequest.h"

//  Local Definitions
#define			DBOPT_SET_MASK		DBOPTION_UNIFORM_ACCESS | DBOPTION_HAS_CALENDAR
#define			MAX_OPTS			30
#define			OPT_TX_OFF			"-t"
#define			OPT_TX_ON			"-T"
#define			TXT_TX_OFF			"disabled"
#define			TXT_TX_ON			"enabled"
#ifdef WIN32
#define			COMPACT_APP			"ncompact.exe"
#endif
#ifdef UNIX
#define			COMPACT_APP			"compact"
#endif

//  Trace Messages
#define			MSG_DXR7501T		"DXR7501T: A new DbModifier object has been created @%p."
#define			MSG_DXR7502T		"DXR7502T: A DbModifier object @%p has been destroyed."
#define			MSG_DXR7526T		"DXR7526T: A Db Modify request is starting with mask: 0x'%X' and settings: 0x'%X'."
#define			MSG_DXR7527T		"DXR7527T: Database options have been changed from: 0x'%X' to: 0x'%X'."
#define			MSG_DXR7528T		"DXR7528T: Database options remain unchanged from: 0x'%X'."
#define			MSG_DXR7529T		"DXR7529T: Database replication flags have been changed from: 0x'%X' to: 0x'%X'."
#define			MSG_DXR7530T		"DXR7530T: Database replication flags remain unchanged from: 0x'%X'."

//  Information Messages
#define			MSG_DXR7505I		"DXR7505I: Opened the target database '%s' on '%s' for modifying."
#define			MSG_DXR7506I		"DXR7506I: One or more changes were made to the database settings by this transaction."
#define			MSG_DXR7520I		"DXR7520I: Template name was updated from '%s' to '%s', design has been refreshed."
#define			MSG_DXR7524I		"DXR7524I: Database '%s' has been compacted to set transaction logging to %s."
#define			MSG_DXR7525I		"DXR7525I: Database '%s' has not been changed, transaction logging remains %s."
#define			MSG_DXR7527I		"DXR7527I: Database options have been changed from: 0x'%X' to: 0x'%X'."
#define			MSG_DXR7528I		"DXR7528I: Database options remain unchanged from: 0x'%X'."
#define			MSG_DXR7529I		"DXR7529I: Database replication flags have been changed from: 0x'%X' to: 0x'%X'."
#define			MSG_DXR7530I		"DXR7530I: Database replication flags remain unchanged from: 0x'%X'."
#define			MSG_DXR7531I		"DXR7531I: Max Internet Access level has been changed from: %i to: %i."
#define			MSG_DXR7532I		"DXR7532I: Max Internet Access level remains unchanged from: %i ."

//  Warning Messages
#define			MSG_DXR7503W		"DXR7503W: Transaction '%s' contains no actions to perform."
#define			MSG_DXR7519W		"DXR7519W: Templplate name has been set to '%s' but the design refresh failed."

//  Error Messages
#define			MSG_DXR7504E		"DXR7504E: Unable to open the target database '%s' on '%s'."
#define			MSG_DXR7507E		"DXR7507E: Unable to read the DbOptions from the target database." 
#define			MSG_DXR7508E		"DXR7508E: Unable to set the DbOptions in the target database." 
#define			MSG_DXR7509E		"DXR7509E: Unable to read the replication settings from the target database." 
#define			MSG_DXR7510E		"DXR7510E: Unable to set the replication settings in the target database." 
#define			MSG_DXR7511E		"DXR7511E: Unable to read the Maximum Internet Access Level from the target database." 
#define			MSG_DXR7512E		"DXR7512E: Unable to set the Maximum Internet Access Level in the target database." 
#define			MSG_DXR7513E		"DXR7513E: Unable to read the Quota Settings from the target database." 
#define			MSG_DXR7514E		"DXR7514E: Unable to canonicalize '%s' to read Quota Settings."
#define			MSG_DXR7515E		"DXR7515E: Unable to construct a net path for '%s' on '%s' to read Quota Settings."
#define			MSG_DXR7516E		"DXR7516E: Unable to set the Quota Settings in the target database." 
#define			MSG_DXR7517E		"DXR7517E: Unable to read the Database Info Buffer from the target database."
#define			MSG_DXR7518E		"DXR7518E: Unable to set the Database Info Buffer in the target database."
#define			MSG_DXR7521E		"DXR7521E: Unable to determine if the target database is transaction logged or not."
#define			MSG_DXR7522E		"DXR7522E: Unable to run the Compact task in order to change the transaction logging settings."
#define			MSG_DXR7523E		"DXR7523E: The Compact task returned %i (0x%.4x), the transaction logging settings may not have been changed."

//  Local Definitons

//  Class Definition
class DbModifier :
	public Runnable
{
public:
	//  Default Constructor
	DbModifier(void);

	//  Constructor
	DbModifier(MTExecutive *xeParent, int iThreadID);

	//  Destructor
	~DbModifier(void);

	//  Public Members
	int			OwningThread;											//  Creating Thread ID

	//  Public Functions
	BOOL ModifyThisDb(ModRequest *mrRQ, int iThreadID);					//  Entry point to initiate modifying
	void ExecuteThisRequest(void *pReqObject, int iThreadID);			//  Multi thread request entry point

private:
	//  Private Members

	//  Private Functions
	BOOL ModifyDbOptions(ModRequest *mrRQ, DBHANDLE hdbSrc, int iThreadID);
	BOOL ModifyRepSettings(ModRequest *mrRQ, DBHANDLE hdbSrc, int iThreadID);
	BOOL ModifyMaxInternetAccess(ModRequest *mrRQ, DBHANDLE hdbSrc, int iThreadID);
	BOOL ModifyDbQuota(ModRequest *mrRQ, DBHANDLE hdbSrc, int iThreadID);
	BOOL ModifyTemplate(ModRequest *mrRQ, DBHANDLE hdbSrc, int iThreadID);
	BOOL ModifyTXLogging(ModRequest *mrRQ, DBHANDLE hdbSrc, int iThreadID);

protected:
	//  Protected Members
	MTExecutive			*xeLocal;									//  Multi-Threading Runtime

	//  Protected Functions
	BOOL virtual ProxyLogMessage(ModRequest *mrRQ, char *szMsg, int iThreadID);			//  Proxry Log interface
};
