#pragma once
//  ModRequest
//
//  This class contains the transaction data for a DbModifier execution
//
//  Message Identifier:      N/A
//  Message Range:           0000 - 0000
//  Trace area assignment:   None
//  DX Version:				 3.10.3
//  Version:                 3.10.3  Build: 83
//
//  Version History:
//
//  3.10.3		13/12/2010		Initial version
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2010
//

//  Application Includes
#include  "../ExecEnvironment.h"

//  Local Definitions

//  Modify Actions
#define			MOD_PROP_FTI			0x80000000						//  Modify FTI Property
#define			MOD_PROP_BGA			0x40000000						//  Modify Background Agents Property
#define			MOD_PROP_CAL			0x20000000						//  Modify Calendar Property
#define			MOD_PROP_REPT			0x10000000						//  Modify Replication Property
#define			MOD_PROP_REPP			0x08000000						//  Modify Replication Property
#define			MOD_PROP_BROWSE			0x04000000						//  Modify Browsable Property
#define			MOD_PROP_CAT			0x02000000						//  Modify Catalog Property
#define			MOD_PROP_TXL			0x01000000						//  Modify Transaction Logged Property
#define			MOD_PROP_CACL			0x00800000						//  Modify Consistent ACL Property
#define			MOD_PROP_MIA			0x00400000						//  Modify Max Internet Access Property
#define			MOD_PROP_TPLT			0x00200000						//  Modify Template Name Property
#define			MOD_PROP_QUOTA			0x00100000						//  Modify Database Quota settings

//  Function Masks for modify actions
#define			MOD_MASK_OPTS			(MOD_PROP_CACL | MOD_PROP_CAL | MOD_PROP_FTI)
#define			MOD_MASK_REPF			(MOD_PROP_REPT | MOD_PROP_BGA | MOD_PROP_REPP | MOD_PROP_CAT | MOD_PROP_BROWSE)

class ModRequest
{
public:
	//  Constructor
	ModRequest(void);

	//  Destructor
	~ModRequest(void);

	//  Public Members
	//  Control variables
	BOOL		PermitRun;												//  Run Permit Flag
	DBHANDLE	hDBSrc;													//  Database Handle

	//  Configuration variables
	int			ReturnCode;												//  Completion code from the copy
	int			PrestartActions;										//  Prestart actions
	int			RetryCount;												//  Retry iteration count
	int			RetryLimit;												//  Retry iteration limit
	int			RestartActions;											//  Restart Recovery actions
	int			FaultTolerance;											//  Fault Tolerance
	BOOL		AllowRetry;												//  Show if request is can be retried

	//  Property Settings
	DWORD		dwPropsToMod;											//  Mask of properties to be modified
	DWORD		dwPropSetting;											//  Property setting to be applied
	WORD		MaxInternetAccess;										//  Max internet access level
	DWORD		QuotaWarn;												//  Db Quota Warning Threshold
	DWORD		QuotaLimit;												//  Db Quota Size Limit

	char		szTransID[MAX_DATABASE + 1];							//  Request Transaction ID
	char		szSourceServer[MAX_SERVER + 1];							//  Source Server
	char		szSourceDatabase[MAX_DATABASE + 1];						//  Source Database or directory
	char		szTemplateName[MAX_TEMPLATE + 1];						//  Template name to apply to the target database

};
