#pragma once
//  DesignManager for DX based applications
//
//  The DesignManifestLine class provides the parsing capability for a single line from a DesignManifest XML
//
//  Message Identifier:      DXR
//  Message Range:           5500 - 5999
//  Trace area assignment:   TRACE_DM (24)
//  DX Version:				 3.11.1
//  Version:                 2.1.1  Build: 118
//
//  Version History:
//
//  20/06/2011		-  2.1.0		-  Tidy up for unix
//  22/10/2011		-  2.1.1		-  VS 2011 Updates
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2001 - 2011
//

//  Platform Includes
#include	"../Platform/PlatBase.h"
#include	"../Platform/NotesBase.h"

//  Application Includes

//  Local Definitions
#define			MAX_DM_NAME				60					//  Max Name size
#define			MAX_DM_VALUE			256					//  Max Value size
#define			MAX_DM_VERSION			30					//  Max Version info size
#define			MAX_DM_UNID				32					//  Max UNID size

//  XML Element Names
#define			DM_NODE_DM				"DESIGNMANIFEST"	//  Design Manifest
#define			DM_NODE_DS				"DESIGNSET"			//  Design Set
#define			DM_NODE_DE				"DESIGNELEMENT"		//  Design Element

//  XML Attribute Names
#define			DM_ATTR_NAME			"NAME"				//  Name=
#define			DM_ATTR_VERSION			"VERSION"			//  Version=
#define			DM_ATTR_TYPE			"TYPE"				//  Type=
#define			DM_ATTR_SIGN			"SIGN"				//  Sign=
#define			DM_ATTR_UNID			"UNID"				//  UNID=
#define			DM_ATTR_REQUNIDMATCH	"REQUNIDMATCH"		//  ReqUNIDMatch=

//  Element Types
#define			DM_TYPE_PAGE			"PAGE"				//  Page
#define			DM_TYPE_VIEW			"VIEW"				//  View
#define			DM_TYPE_FORM			"FORM"				//  Form
#define			DM_TYPE_SFORM			"SUBFORM"			//  Subform
#define			DM_TYPE_FSET			"FRAMESET"			//  Frameset
#define			DM_TYPE_AGENT			"AGENT"				//  Agent
#define			DM_TYPE_IMAGE			"IMAGERESOURCE"		//  Image Resource
#define			DM_TYPE_SFIELD			"SHAREDFIELD"		//  Shared Field
#define			DM_TYPE_SACTION			"SHAREDACTION"		//  Shared Action
#define			DM_TYPE_LIB				"LIBRARY"			//  Library
#define			DM_TYPE_FOLDER			"FOLDER"			//  Folder
#define			DM_TYPE_OUTLINE			"OUTLINE"			//  Outline
#define			DM_TYPE_DBICON			"DBICON"			//  DbIcon
#define			DM_TYPE_SHCOL			"SHAREDCOLUMN"		//  Shared Column
#define			DM_TYPE_HABOUT			"HELPABOUT"			//  Help About
#define			DM_TYPE_HUSING			"HELPUSING"			//  Help Using

//  Encoded line types
#define			DM_LINE_UNDEFINED		0					//  Line type is undefined
#define			DM_LINE_MANIFEST		1					//  Line is a DesignManifest
#define			DM_LINE_SET				2					//  Line is a DesignSet
#define			DM_LINE_ELEMENT			3					//  Line is a DesignElement
#define			DM_LINE_MANIFEST_END	4					//  Line is the end of a DesignManifest
#define			DM_LINE_SET_END			5					//  Line is the end of a DesignSet
#define			DM_LINE_ELEMENT_END		6					//  Line is the end of an element
#define			DM_LINE_VALUE			7					//  Line is a value
#define			DM_LINE_STRUCT			8					//  Line is XML structure
#define			DM_LINE_COMMENT			9					//  Line is Comment
#define			DM_LINE_INVALID			10					//  Line is Invalid

//  Encoded element types
#define			DM_ELEMENT_NONE			0					//  Element type is undefined
#define			DM_ELEMENT_PAGE			1					//  Element type is Page
#define			DM_ELEMENT_VIEW			2					//  Element type is View
#define			DM_ELEMENT_FORM			3					//  Element type is Form
#define			DM_ELEMENT_SFORM		4					//  Element type is Subform
#define			DM_ELEMENT_FSET			5					//  Element type is FrameSet
#define			DM_ELEMENT_AGENT		6					//  Element type is Agent
#define			DM_ELEMENT_IMAGE		7					//  Element type is ImageResource
#define			DM_ELEMENT_SFIELD		8					//  Element type is SharedField
#define			DM_ELEMENT_SACTION		9					//  Element type is SharedAction
#define			DM_ELEMENT_LIB			10					//  Element type is Library
#define			DM_ELEMENT_FOLDER		11					//  Element type is Folder
#define			DM_ELEMENT_OUTLINE		12					//  Element type is Outline
#define			DM_ELEMENT_DBICON		13					//  Element type is DbIcon
#define			DM_ELEMENT_SHCOL		14					//  Element is a Shared Column
#define			DM_ELEMENT_HABOUT		15					//  Element is Help About
#define			DM_ELEMENT_HUSING		16					//  Element is Help Using

//  Class Definition
class DesignManifestLine
{
public:
	//  Constructor
	DesignManifestLine(char *szLine);

	//  Destructor
	~DesignManifestLine(void);

	//  Public Members
	int LineType;										//  Encoded Line Type
	int ElementType;									//  Encoded Element Type
	char szName[MAX_DM_NAME + 1];						//  Element Name
	char szValue[MAX_DM_VALUE + 1];						//  Element Value (URL)
	char szVersion[MAX_DM_VERSION + 1];					//  Element Version
	char szUNID[MAX_DM_UNID + 1];						//  Element UNID
	BOOL bSign;											//  Flag to indicate if element should be signed after import
	BOOL bReqUNIDMatch;									//  Flag to indicate that existing UNID match is required

private:
	int EncodeElementType(char *szType);
};
