#pragma once
//  DXPartNoteScanReq
//
//  This class is used to communicate partial database scan requests between the threads in the
//  DXNoteScanner class.
//
//  Message Identifier:      DXR
//  Message Range:           1801 - 1899
//  Trace area assignment:   TRACE_DBSCAN (28)
//  DX Version:				 3.10.0
//  Version:                 3.10.0  Build: 80
//
//  Version History:
//
//  1.0.0		14/09/2004		Initial version
//  3.5.3		21/08/2009		Re-based ready for 4.0.0
//  3.9.2		22/09/2010		Avoided a recursion in DX processing
//  3.10.0		01/10/2010		Wildcard & multi-thread execution added to DX processing
//								DXNoteScanner incorporated in kernel
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2001 - 2010
//

//  Application Includes
#include	"../MTX/MTExecutive.h"					//  Run time

//  Local Constants
#define		MAX_SEARCH_STRING				256
#define		MAX_SEARCH_ENTRIES				10

//  Special Scan flags
#define		FLAG_NOTE_SCAN_OBJECT_FILE		0x0001				//  Scan for Documents with File Attachments
#define		FLAG_NOTE_SCAN_OBJECT_OTHER		0x0002				//  Scan for Documents with other objects
#define		FLAG_NOTE_SCAN_SIGNED			0x0004				//  Scan for Signed Documents
#define		FLAG_NOTE_SCAN_ENCRYPTED		0x0008				//  Scan for Encrypted Documents
#define		FLAG_NOTE_SCAN_DELSTUBS			0x0010				//  Scan for Deleteion Stubs
#define		FLAG_NOTE_SCAN_REPCONS			0x0020				//  Scan for Replication Conflicts
#define		FLAG_NOTE_SCAN_NOACCESS			0x0040				//  Scan for Documents that cannot be read
#define		FLAG_NOTE_SCAN_NOT_DELSTUB		0x0080				//  Scan for Documents that are NOT deleted

//  Data Structures
typedef struct {
	DHANDLE hitMatches;										//  Handle of IDTable containing matches for this search
	WORD	wNoteClass;										//  Note classes to scan for
	DWORD	dwSearchFlags;									//  Search Flags... see FLAG_NOTE_SCAN_xxx
	char	szSearchString[MAX_SEARCH_STRING];				//  Search string to be used for this search
	DWORD	dwCount;										//  Number of noteids added to IDTable
} DXNoteScanEntry;

typedef struct {
	DXNoteScanEntry		dxnsEntry[MAX_SEARCH_ENTRIES];		//  Array of DXNoteScanEntry objects
} DXNoteScanTable;

//  Class Definition
class DXNoteScanReq
{
public:
	//  Constructor
	DXNoteScanReq(DXNoteScanTable *nstToUse, int iThreadID);

	//  Destructor
	~DXNoteScanReq(void);

	//  Public Members
	int				OwningThread;						//  Creating Thread ID
	DWORD			dwMaxErrors;						//  Max number of note scan errors before exiting with error
	char			szServer[MAX_SERVER];				//  Server that hosts database to be scanned
	char			szDbPath[MAX_DATABASE];				//  Path of database to be scanned
	DBHANDLE		hdbToScan;							//  Handle of database to be scanned
	int				iReturnCode;						//  Return code from DXNoteScanner
	DWORD			dwNotesToScan;						//  Number of notes found by base search
	DWORD			dwNotesScanned;						//  Number of notes scanned during processing
	DWORD			dwErrors;							//  Number of errors during processing
	DWORD			dwElapsed;							//  Time in seconds it took to perform the scan
	DXNoteScanTable	*nsObject;							//  Pointer to DXNoteScanTable data structure

	//  Public Functions

private:
	//  Private Members

	//  Private Functions

protected:
	//  Protected Members

	//  Protected Functions

};
