#pragma once
//  DXNoteScanner
//
//  This class is the eXplorer class that handles the scanning of the notes within a database according
//  to the search criteria
//
//  Message Identifier:      DXR
//  Message Range:           1801 - 1899
//  Trace area assignment:   TRACE_EXPLORER (05)
//  DX Version:				 3.14.0
//  Version:                 3.14.0  Build: 108
//
//  Version History:
//
//  1.0.0		14/09/2004		Initial version
//  3.5.3		21/08/2009		Re-based ready for 4.0.0
//  3.9.2		22/09/2010		Avoided a recursion in DX processing
//  3.10.0		01/10/2010		Wildcard & multi-thread execution added to DX processing
//								DXNoteScanner incorporated in kernel
//  3.11.1		22/10/2011		VS 2011 Updates
//	3.14.0		21/01/2015		x64 compatability fixes
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2004 - 2015
//
//

//  Application Includes
#include "../ExecEnvironment.h"
#include "DXPartNoteScanReq.h"

//  Additional Domino includes
#include <nsfsearc.h>

//  Local Definitions
#define		MAX_SEARCH_FORMULA_LENGTH		2048				//  Max length of global NSFSearch string
#define		SCAN_ACC_MUTEX					1					//  Data accumulator mutes
#define		REJOIN_WAITS_MAX				1800				//  3 mins wait limit
#define		LARGE_MAXPATH					(MAXPATH * 2)		//  Long buffer for filename processing
#define		FORMULA_START					"(%s)"
#define		FORMULA_CONCAT					" | ("
#define		FORMULA_END						")"

//  Trace Messages
#define		MSG_DXR1801T		"DXR1801T: A new DXNoteScanner instance has been created @%p."
#define		MSG_DXR1802T		"DXR1802T: A DXNoteScanner instance @%p has been destroyed."
#define		MSG_DXR1821T		"DXR1821T: Performing search of database using NSFDbGetModifiedNoteTable."
#define		MSG_DXR1822T		"DXR1822T: Performing search of database using NSFSearch with formula '%s'."
#define		MSG_DXR1829T		"DXR1829T: DXNoteScanner PerformScanAction has been entered for %i processing."
#define		MSG_DXR1835T		"DXR1835T: ScanForObjectItems is processing note %1.4X."
#define		MSG_DXR1849T		"DXR1849T: SearchDb stage1 results - # SearchEntries %d, GlobalNoteClasses %d, GlobalScanFlags %d, SearchFound %d, AllFormulasMatch %d."
#define		MSG_DXR1850T		"DXR1850T: SearchDb stage1 checks starting."
#define		MSG_DXR1852T		"DXR1852T: SearchDb stage2 checks starting."
#define		MSG_DXR1853T		"DXR1853T: SearchDb stage2 results - # SearchStrings added to NSFSearch %d, NSFSearch '%s'."

#ifdef W64
#define		MSG_DXR1851T		"DXR1851T: IDTable %i, NoteClass %X, SearchFlags %X, SearchString '%s'."
#else
#define		MSG_DXR1851T		"DXR1851T: IDTable %p, NoteClass %X, SearchFlags %X, SearchString '%s'."
#endif

//  Information Messages
#define		MSG_DXR1804I		"DXR1804I: Performing DXNoteScan against database '%s' on '%s'."
#define		MSG_DXR1806I		"DXR1806I: DXNoteScanner has opened the source database."
#define		MSG_DXR1807I		"DXR1807I: DXNoteScanner using supplied opened source database."
#define		MSG_DXR1810I		"DXR1810I: DXNoteScanner is checking supplied DXNoteScanReq object."
#define		MSG_DXR1813I		"DXR1813I: Completed scan of database."
#define		MSG_DXR1814I		"DXR1814I: Closed database."
#define		MSG_DXR1815I		"DXR1815I: Scanning completed in %s%d seconds at a rate of %s%d documents per second."
#define		MSG_DXR1817I		"DXR1817I: The source database contains no notes matching the requested search(s)."
#define		MSG_DXR1824I		"DXR1824I: Database search found %d notes to scan."
#define		MSG_DXR1826I		"DXR1826I: Processed: %d/%d part db scan requests (scanned: %d, %d total)."
#define		MSG_DXR1827I		"DXR1827I: DXNoteScanner has finished posting PartNoteScanReq objects to the thread manager ."
#define		MSG_DXR1828I		"DXR1828I: All PartNoteScanReq objects have completed processing."

//  Warning Messages
#define		MSG_DXR1839W		"DXR1839W: Attachment filename length of %d is too long."
#define		MSG_DXR1843W		"DXR1843W: Search of database for notes matching all search criteria found 0 matches!"

//  Error Messages
#define		MSG_DXR1803E		"DXR1803E: Failed to scan database because source DXNoteScanReq pointer was NULL."
#define		MSG_DXR1805E		"DXR1805E: DXNoteScanner was unable to open the source database."
#define		MSG_DXR1808E		"DXR1808E: Formula in search entry %d failed to compile (%s)."
#define		MSG_DXR1809E		"DXR1809E: Combined length of all search entry formulas is longer than max allowed %d bytes."
#define		MSG_DXR1811E		"DXR1811E: Supplied DXNoteScanReq object does not contain an open database and source '%s' is blank!."
#define		MSG_DXR1812E		"DXR1812E: Supplied DXNoteScanReq object does not contain an IDTable in the first search entry!."
#define		MSG_DXR1816E		"DXR1816E: Failed to release HCOMPUTE pointer for search entry %d."
#define		MSG_DXR1818E		"DXR1818E: Unable to get a modified note ID Table."
#define		MSG_DXR1819E		"DXR1819E: Error at Line: %i, Col: %i, Offset: %i, Length: %i."
#define		MSG_DXR1820E		"DXR1820E: Global search formula failed to compile (%s)."
#define		MSG_DXR1823E		"DXR1823E: NSFSearch using global search formula failed."
#define		MSG_DXR1825E		"DXR1825E: Unable to post a request to scan notes, data scan terminated."
#define		MSG_DXR1830E		"DXR1830E: Failed to get local Db Handle."
#define		MSG_DXR1831E		"DXR1831E: Failed to insert NoteID %1.4X into IDTable of search entry %d."
#define		MSG_DXR1832E		"DXR1832E: Failed to open note %1.4X."
#define		MSG_DXR1833E		"DXR1833E: Failed to close note %1.4X."
#define		MSG_DXR1834E		"DXR1834E: ScanForObjectItems failed because supplied parameter '%s' is NULL!"
#define		MSG_DXR1836E		"DXR1836E: ScanForObjectItems failed to get info about item %d on note %1.4X."
#define		MSG_DXR1837E		"DXR1837E: GetObjectInfo failed because supplied BLOCKID is NULL."
#define		MSG_DXR1838E		"DXR1838E: GetObjectInfo failed to lock object block."
#define		MSG_DXR1839E		"DXR1839E: ScanForObjectItems failed because both HasFileAttach and HasOtherObject pointers are NULL."
#define		MSG_DXR1840E		"DXR1840E: Failed to get info about note %1X."
#define		MSG_DXR1841E		"DXR1841E: AccumulateTotals failed because source '%s' pointer was NULL."
#define		MSG_DXR1842E		"DXR1842E: Search of database for notes matching all search criteria failed."
#define		MSG_DXR1844E		"DXR1844E: Scan of notes matching all search criteria failed."
#define		MSG_DXR1845E		"DXR1845E: Processing error count %d exceeds the max allowed %d. Cleaning up and exiting."
#define		MSG_DXR1846E		"DXR1846E: Failed to evaluate formula in search entry %d against NoteID %1.4X."
#define		MSG_DXR1847E		"DXR1847E: Search entry %d is invalid - do not specify search string along with DelStubs | NoAccess Flags."
#define		MSG_DXR1848E		"DXR1848E: Failed to initialize formula in search entry %d."
#define		MSG_DXR1854E		"DXR1854E: Failed to create IDTable to store results of NSFSearch."
#define		MSG_DXR1855E		"DXR1855E: Failed to get thread level handle to compiled formula in search entry %d."
#define		MSG_DXR1856E		"DXR1856E: Failed to remove thread level handle to compiled formula in search entry %d."

//  Data Structures
typedef struct {
	char			szSearchString[MAX_SEARCH_STRING];				//  Search string to be used for this search
	FORMULAHANDLE	hfSearch;										//  Handle allocated to formula via NSFFormulaCompile
	char			*phfSearch;										//  Pointer used to lock handle when in use
	HCOMPUTE		hcSearch;										//  HCOMPUTE allocated by NSFComputeStart when being used
} DXNoteScanIntEntry;

typedef struct {
	int			iNIDArrayIndex;
	NOTEID		nidRetNID;
	NOTEHANDLE	hnToScan;
	WORD		wNoteclass;
	DWORD		dwNoteFlags;
} DXNoteScanNoteProps;

//  Class Definition
class DXNoteScanner
{
public:
	//  Default Constructor
	DXNoteScanner(void);

	//  Constructor
	DXNoteScanner(ExecEnvironment *xeParent, int iThreadID);

	//  Destructor
	~DXNoteScanner(void);

	//  Public Members
	int	OwningThread;													//  Creating Thread ID
	DHANDLE hitNotesToScan;												//  IDTable of notes to scan

	//  Public Functions
	BOOL ScanDb(DXNoteScanReq *nsrCurrent, int iThreadID);				//  Main entry point
	STATUS static LNCALLBACK WrapperProcessSearchResult(void *ObjPtr, SEARCH_MATCH *smCurrent, ITEM_TABLE *itCurrent);
	STATUS ProcessSearchResult(SEARCH_MATCH *smHit, ITEM_TABLE *itCurrent, DXNoteScanner *dxnsCurrent);
	BOOL GetObjectInfo(BLOCKID bidItemValue, DWORD *dwObjID, WORD *wObjType, char *szFileName, int iFileNameLen, char *szExt, int iExtLen, DWORD *dwFileSize, WORD *wHostType, WORD *wCompType, int iThreadID);

private:
	//  Private Members
	DWORD dwNotesInserted;													//  Number of notes inserted into IDTable
	BOOL bDbLocallyOpened;													//  Whether DXNoteScanner opened the Db or not
	BOOL bDbSearchPerformed;												//  Whether DXNoteScanner did an NSFSearch
	DWORD dwGlobalScanFlags;												//  Global value covering all search entry flags
	WORD wGlobalNoteClasses;												//  Global note classes covering all search entries
	DXNoteScanIntEntry	nsieTable[MAX_SEARCH_ENTRIES];						//  Array of DXNoteScanIntEntry

	//  Private Functions
	BOOL CheckSourceReq(DXNoteScanReq *nsrCurrent, int iThreadID);			//  Check the source req object
	BOOL SearchDb(DXNoteScanReq *nsrCurrent, int iThreadID);				//  Search the database as required
	BOOL CleanupIntTable(int iThreadID);									//  Cleanup the internal formula data structure table
	BOOL ProcessNotes(DXNoteScanReq *nsrCurrent, int iThreadID);			//  Multi threaded DbScanner
	void PerformScanAction(DXPartNoteScanReq *ppnsCurrent, int iThreadID);	//  Perform a scan action
	void PerformScanNotes(DXPartNoteScanReq *ppnsCurrent, int iThreadID);	//  Perform a scan of data notes
	BOOL CheckSearchEntriesForMatch2(DXPartNoteScanReq *ppnsCurrent, DXNoteScanNoteProps *nsnpCurrent, int iThreadID);	//  Check each search entry for specified flag
	BOOL ScanForObjectItems(DXPartNoteScanReq *ppnsCurrent, DBHANDLE hdbTLSource, NOTEHANDLE hnToScan, BOOL *bHasFileAttach, BOOL *bHasOtherObject, int iThreadID);	//  Scan note for object items

protected:
	//  Protected Members
	ExecEnvironment				*xeLocal;									//  Runtime

	//  Protected Functions
	void AccumulateTotals(DXNoteScanReq *nsrCurrent, DXPartNoteScanReq *ppnsCurrent, int iThreadID);
	
};
