//  DXReporter
//
//  This is the base class that is extended to build any Domino eXplorer functionality that operates at
//  the database level. The class defines the interface that used to invoke the functional code as well
//  as providing a number of helper functions.
//
//  Message Identifier:      DXR
//  Message Range:           1501 - 1599
//  Trace area assignment:   TRACE_EXPLORER (05)
//  DX Version:				 3.10.5
//  Version:                 3.10.5  Build: 86
//
//  Version History:
//
//  1.0.0		14/09/2004		Initial version
//  3.5.3		21/08/2209		Re-based ready for 4.0.0
//  3.10.0		01/10/2010		Wildcard & multi-thread execution added to DX processing
//								DXNoteScanner incorporated in kernel
//  3.10.5		18/02/2011		Report on D/Bs which cannot be opened
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2004 - 2011
//

#include "DXReporter.h"

//  Default Constructor
DXReporter::DXReporter(void)
: xeLocal(NULL)
{
}

//  Constructor that inherits the execution environment
DXReporter::DXReporter(ExecEnvironment *xeParent, int iThreadID)
: xeLocal(NULL)
{
}

//  Destructor
DXReporter::~DXReporter(void)
{
}

//  ReportOnThisDb
//
//  This interface defines the function signature for functions that execute against each D/B
//
//  Parameters
//
//  1  -  DXRequest *		-  Pointer to a DXRequest object that defines the eXplorer search scope and constraints
//  2  -  NOTEHANDLE		-  Handle to the current D/B that is to be reported on
//  3  -  int				-  ID of the invoking Thread
//
//  Returns
//
//  BOOL  -			TRUE if the d/b was processed successfully
//                  FALSE if the d/b was not processed successfully
//
//  API Use Notes:
//
//
BOOL DXReporter::ReportOnThisDb(DXRequest *dxrRQ, DBHANDLE hdbEntity, int iThreadID)
{
	return FALSE;
}

//  ReportNoAccess
//
//  This interface defines the function signature for functions that execute against each D/B that cannot be opened
//
//  Parameters
//
//  1  -  DXRequest *		-  Pointer to a DXRequest object that defines the eXplorer search scope and constraints
//  2  -  int				-  ID of the invoking Thread
//
//  Returns
//
//  BOOL  -			TRUE if the d/b was processed successfully
//                  FALSE if the d/b was not processed successfully
//
//  API Use Notes:
//
//
BOOL DXReporter::ReportNoAccess(DXRequest *drRQ, int iThreadID)
{
	return FALSE;
}

//  GetDescription
//
//  This interface defines a function that must be implemented to provide a description of the processor
//  that is being executed against Databases or Notes. The value is used in documentation.
//
//  Parameters
//
//  1  -  char *		- Pointer to a string buffer where the desription must be stored
//
//  Returns
//
//  API Use Notes:
//
//
void DXReporter::GetDescription(char *szDescription)
{
}
