#pragma once
//  DXServerScanner
//
//  This class is the eXplorer class that handles the scanning of the server level in the hierarchy according to
//  the scope and constraints from the search request.
//
//  Message Identifier:      DXR
//  Message Range:           1201 - 1299
//  Trace area assignment:   TRACE_EXPLORER (05)
//  DX Version:				 3.14.0
//  Version:                 3.14.0  Build: 108
//
//  Version History:
//
//  1.0.0		14/09/2004		Initial version
//  3.5.3		21/08/2209		Re-based ready for 4.0.0
//  3.9.2		22/09/2010		Avoided a recursion in DX processing
//  3.10.0		01/10/2010		Wildcard & multi-thread execution added to DX processing
//								DXNoteScanner incorporated in kernel
//  3.11.1		22/10/2011		VS 2011 Updates
//	3.14.0		21/01/2015		x64 compatability fixes
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2004 - 2015
//

//  Application Includes
#include "../ExecEnvironment.h"
#include "../DXException.h"
#include "DXRequest.h"
#include "DXFilter.h"
#include "DXDirScanner.h"
#include "DXSStash.h"

//  Trace Messages
#define		MSG_DXR1223T		"DXR1223T: Checking for availability of server '%s'."
#define		MSG_DXR1227T		"DXR1227T: A new DXServerScanner instance has been created @%p."
#define		MSG_DXR1228T		"DXR1228T: A DXServerScanner instance @%p has been destroyed."
#define		MSG_DXR1239T		"DXR1239T: Accepted a request for server: %s."

//  Information Messages
#define		MSG_DXR1234I		"DXR1234I: %i Servers were passed to the stash."
#define		MSG_DXR1238I		"DXR1238I: Server: %s in domain: %s has been added to the stash."
#define		MSG_DXR1240I		"DXR1240I: Server: %s has been dropped as it is unmatched or filtered."

//  Error Messages
#define		MSG_DXR1220E		"DXR1220E: The request could not be serviced because the requested server was unavailable."
#define		MSG_DXR1221E		"DXR1221E: An unspecified exception has been trapped."
#define		MSG_DXR1222E		"DXR1222E: Server '%s' is not available."
#define		MSG_DXR1224E		"DXR1224E: Unable to get the path to the Server NAB."
#define		MSG_DXR1225E		"DXR1224E: Unable to parse the server name from an extended database path."
#define		MSG_DXR1226E		"DXR1226E: Unable to abbreviate the returned server name."
#define		MSG_DXR1229E		"DXR1229E: Unable to locate the $Servers view in the NAB, stash was not updated."
#define		MSG_DXR1230E		"DXR1230E: Unable to open the $Servers view in the NAB, stash was not updated."
#define		MSG_DXR1231E		"DXR1231E: Unable to read the $Servers view in the NAB, stash was not updated."
#define		MSG_DXR1232E		"DXR1232E: Nothing returned from reading the $Servers view in the NAB, stash was not updated."
#define		MSG_DXR1233E		"DXR1233E: Unable to open a server document, stash was not updated."
#define		MSG_DXR1235E		"DXR1235E: A null server name was returned from a server document, stash was not updated."
#define		MSG_DXR1236E		"DXR1236E: Unable to abberviate a server name, stash was not updated."
#define		MSG_DXR1237E		"DXR1237E: A null domain name was returned from a server document, stash was not updated."

//  Miscellaneoud definitions
#define		DEFAULT_NAB_DB		"names.nsf"				//  D/B name for a NAB

class DXServerScanner
{
public:
	//  Constructor

	DXServerScanner(ExecEnvironment *xeParent, int iThreadID);

	//  Destructor
	~DXServerScanner(void);

	//  Public Members
	DXSStash *dxsStash;												//  Stash object
	int iParentThread;												//  ID of the owning thread
	
	//  Public Functions
	BOOL ProcessRequest(DXRequest *dxrRQ, int iThreadID);			//  Process an explorer request
	BOOL UpdateStash(DBHANDLE hdbNAB, int iThreadID);				//  Update the stash

private:
	//  Private Members
	ExecEnvironment *xeLocal;										//  Execution Environment
	DXDirScanner *dxdsLocal;										//  Directory Scanner

	//  Private Functions
	BOOL IsServerAvailable(DXRequest *dxrRQ, int iThreadID);		//  Check if server is available
	BOOL UpdateStashFromNote(NOTEHANDLE hnServer, int iThreadID);	//  Update the stash from a server document
	BOOL CoreProcessRequest(DXRequest *dxrRQ, int iThreadID);		//  Internal Request processor
};

