#pragma once
//  DXSpider
//
//  This class is the top level eXplorer class, it will orchestrate the search strategy that best matches
//  the scope and constraints from the search request.
//
//  Message Identifier:      DXR
//  Message Range:           1101 - 1199
//  Trace area assignment:   TRACE_EXPLORER (05)
//  DX Version:				 3.14.0
//  Version:                 3.14.0  Build: 108
//
//  Version History:
//
//  1.0.0		14/09/2004		Initial version
//  3.5.3		21/08/2209		Re-based ready for 4.0.0
//  3.9.2		22/09/2010		Avoided a recursion in DX processing
//  3.10.0		01/10/2010		Wildcard & multi-thread execution added to DX processing
//								DXNoteScanner incorporated in kernel
//  3.11.1		22/10/2011		VS 2011 Updates
//	3.14.0		21/01/2015		x64 compatability fixes
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2004 - 2015
//

//  Application Includes
#include "../ExecEnvironment.h"
#include "../DXException.h"
#include "DXRequest.h"
#include "DXFilter.h"
#include "DXServerScanner.h"

//  Trace Messages
#define		MSG_DXR1101T		"DXR1101T: Passing Request through to Server scan layer."
#define		MSG_DXR1103T		"DXR1103T: A new DXSpider instance has been created @%p."
#define		MSG_DXR1104T		"DXR1104T: A DXSpider instance @%p has been destroyed."
#define		MSG_DXR1108T		"DXR1108T: Initial population of the server stash is complete."
#define		MSG_DXR1110T		"DXR1110T: Attempting to locate initial server for spidering."
#define		MSG_DXR1112T		"DXR1112T: Querying for server list on port: '%s'."
#define		MSG_DXR1114T		"DXR1114T: Dispatching a Server scan for: %s in domain: %s."

//  Information Messages
#define		MSG_DXR1111I		"DXR1111I: Trying server '%s' for initial spider point."
#define		MSG_DXR1113I		"DXR1113I: Server: %s in domain: %s is not a member of domain: %s."
#define		MSG_DXR1116I		"DXR1116I: Server: %s in domain: %s has been dropped by the filter."

//  Error Messages
#define		MSG_DXR1100S		"DXR1100S: Universal scope requests are not supported by this explorer."
#define		MSG_DXR1102E		"DXR1102E: An unspecified exception has been trapped."
#define		MSG_DXR1105S		"DXR1105S: Unable to open the local Names and Address Book (NAB), unable to spider."
#define		MSG_DXR1106E		"DXR1106E: Unable to check the design of the local Names and Address Book (NAB), unable to spider."
#define		MSG_DXR1107E		"DXR1107E: Failed to initially populate the server stash, unable to spider."
#define		MSG_DXR1109E		"DXR1109E: Unable to find any server to start spidering."
#define		MSG_DXR1115E		"DXR1115E: Scan of server: %s completed with errors, continuing with next server."

//  Miscellaneous definitions
#define		DEFAULT_NAB_DB		"names.nsf"				//  Default D/B name for a NAB
#define		ENV_PORTS_ENTRY		"Ports"					//  Ports= entry in the environment

class DXSpider
{
public:
	//  Constructor
	DXSpider(ExecEnvironment *xeParent, int iThreadID);

	//  Default Destructor
	~DXSpider(void);

	//  Public Members
	int iParentThread;												//  ID of the owning thread

	//  Public Functions
	BOOL ProcessRequest(DXRequest *dxrRQ, int iThreadID);			//  Process the request
	
private:
	//  Private Members
	ExecEnvironment * xeLocal;										//  Inherited Execution Environment
	DXServerScanner *dxssInstance;									//  Domino Explorer Server Scanner
	DXSStash *dxsStash;												//  Stash object

	//  Private Functions
	BOOL InitiallyPopulateStash(int iThreadID);						//  Initially populate the stash
	DBHANDLE SwapNAB(DBHANDLE hdbPNAB, int iThreadID);				//  Switch to a server NAB
	BOOL ValidateDomain(DXRequest *dxrRQ, DXRequest *dxrSRQ, int iThreadID);		//  Check the Domain selection
	BOOL CoreProcessRequest(DXRequest *dxrRQ, int iThreadID);		//  Process a request	
};
