//  DXResource
//
//  This class is used to encapsulate all of the information needed by the ResourceLoader for the lifetime 
//  of a resource.
//
//  Message Identifier:      DXR
//  Message Range:           7001 - 7099
//  Trace area assignment:   20
//  Version:                 3.11.1
//
//  Version History:
//
//  1.0.0		12/08/2009		Initial version
//  3.10.1		15/10/2010		Asynch Transaction Handler added
//								Resource class added + On Demand loading of resources
//  3.11.1		22/10/2011		VS 2011 Updates
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2009 - 2011
//

#include "DXResource.h"

//  Constructor
DXResource::DXResource(ExecEnvironment *xeParent, int iThreadID)
{
	char		szMsg[MAX_MSG + 1];						//  Generic message buffer

	//  Set the DX Runtime
	xeLocal = xeParent;
	iOwningThread = iThreadID;
	pRLoader = NULL;

	//  Set the class and state of the resource
	iRMCLass = 0;
	iRMState = RMSTATE_NOTINIT;

	//  Set the HTTP Proxy Strategy
	HTTPPStrategy = HTTPP_FALLBACK;
	iCurrentConnectConfig = HTTPP_NONE;

	//  Initialise the stateful variables
#ifdef WIN32
	hWHConnection = NULLHANDLE;
	hWHRequest = NULLHANDLE;
#endif
	hRespBuffer = NULLHANDLE;
	hResBuffer = NULLHANDLE;
	dwRespBfrSize = 0;
	pFileDesc = NULL;
	bEOFReached = FALSE;
	dwMemSize = 0;
	dwMemOffset = 0;
	pEnumMemBuffer = NULL;
	szCurrentReadURL[0] = '\0';
	szCurrentFile[0] = '\0';

	//  Trace construction
	sprintf_s(szMsg, MAX_MSG, MSG_DXR7091T, this);
	xeLocal->LogTrace(TRACE_RESOURCELOADER, szMsg, iOwningThread);
}

//  Destructor
DXResource::~DXResource(void)
{
	char		szMsg[MAX_MSG + 1];						//  Generic message buffer

	//  Prevent resource leaks
#ifdef WIN32
	if (hWHRequest != NULLHANDLE)
	{
		WinHttpCloseHandle(hWHRequest);
		hWHRequest = NULLHANDLE;
	}
	if (hWHConnection != NULLHANDLE)
	{
		WinHttpCloseHandle(hWHConnection);
		hWHConnection = NULLHANDLE;
	}
#endif
	if (hRespBuffer != NULLHANDLE)
	{
		OSMemoryUnlock(hRespBuffer);
	}
	if (hResBuffer != NULLHANDLE)
	{
		OSMemoryUnlock(hResBuffer);
	}
	if (pFileDesc != NULL) fclose(pFileDesc);

	//  Trace destruction
	sprintf_s(szMsg, MAX_MSG, MSG_DXR7092T, this);
	xeLocal->LogTrace(TRACE_RESOURCELOADER, szMsg, iOwningThread);
}

