#pragma once
//  DXResource
//
//  This class is used to encapsulate all of the information needed by the ResourceLoader for the lifetime 
//  of a resource.
//
//  Message Identifier:      DXR
//  Message Range:           7001 - 7099
//  Trace area assignment:   20
//  Version:                 3.11.1
//
//  Version History:
//
//  1.0.0		12/08/2009		Initial version
//  3.10.1		15/10/2010		Asynch Transaction Handler added
//								Resource class added + On Demand loading of resources
//  3.11.1		22/10/2011		VS 2011 Updates
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2009 - 2011
//

//  Platform Includes
#include "Platform/PlatBase.h"

//  Runtime Includes
#include "ExecEnvironment.h"
#include "DXGlobals.h"

//  Additional Platform includes
#ifdef WIN32
#include	<winhttp.h>											//  WinHTTP API
#endif

//  Additional Notes API includes
#include	<editods.h>											//  CD Record definitions

//  Resource Source Types
#define		RSOURCE_ARSRC				1						//  Application Resource
#define		RSOURCE_OSFILE				2						//  Operating System File
#define		RSOURCE_URL					3						//  URL
#define		RSOURCE_NMLTXT				4						//  Notes Multi-Line Text Item
#define		RSOURCE_NRTXT				5						//  Notes Rich Text Item
#define		RSOURCE_INVALID				6						//  Invalid or Unrecognized

//  Resource Management Class
#define		RMCLASS_PRIMARY				1						//  This is a Primary Resource
#define		RMCLASS_CLONE				2						//  This is a Cloned Resource
#define		RMCLASS_HTTPLOD				4						//  This is a HTTP Load On Demand Resource
#define		RMCLASS_FILELOD				8						//  This is a File Load On Demand Resource
#define		RMCLASS_MEMLOD				16						//  This is a Memory Load On Demand Resource
#define		RMCLASS_TYPE				(RMCLASS_HTTPLOD | RMCLASS_FILELOD | RMCLASS_MEMLOD)

//  Resource State
#define		RMSTATE_NOTINIT				1						//  No stateful information
#define		RMSTATE_INIT				2						//  No stateful informatipon, but source is defined
#define		RMSTATE_INPROGRESS			4						//  Stateful information is valid
#define		RMSTATE_FINISHED			8						//  Completed On Demand
#define		RMSTATE_DISPOSED			16						//  Resource has been disposed

//  HTTP Proxy Strategies
#define		HTTPP_NONE					0						//  No Strategy has been set
#define		HTTPP_DIRECT				1						//  Always use direct (no Proxy) access
#define		HTTPP_WPROXY				2						//  Always use WinHTTP Proxy Settings
#define		HTTPP_IEPROXY				3						//  Always use IE Proxy Settings
#define		HTTPP_AUTOP					4						//  Use MS Autoproxy
#define		HTTPP_FALLBACK				5						//  Use direct with WinHTTP Proxy as fallback
#define		HTTPP_LEARN					6						//  Learn the proxy access strategy

//  Trace Messages
#define		MSG_DXR7091T				"DXR7091T: A new DXResource instance has been created @%p."
#define		MSG_DXR7092T				"DXR7092T: A DXResource instance @%p has been destroyed."

//  Class Definition
class DXResource
{
public:
	//  Constructor
	DXResource(ExecEnvironment *xeParent, int iThreadID);

	//  Destructor
	~DXResource(void);

	//  Public Members
	ExecEnvironment *xeLocal;									//  Runtime
	void *pRLoader;												//  Pointer to the resource loader
	int HTTPPStrategy;											//  HTTP Proxy Strategy
	int	iOwningThread;											//  Owning Thread
	int iRMCLass;												//  Resource Class
	int	iRMState;												//  Resource State
#ifdef WIN32
	HINTERNET hWHConnection;									//  Handle to the WinHTTP Connection
	HINTERNET hWHRequest;										//  Handle to the WinHTTP Request
#endif
	MEMHANDLE hRespBuffer;										//  Handle to the response buffer
	MEMHANDLE hResBuffer;										//  Handle to the resource buffer
	DWORD dwRespBfrSize;										//  Size of data in the response buffer
	DWORD dwEnumBufferUsed;										//  Size of the buffer used by enumerators
	int	iCurrentConnectConfig;									//  The current connection config for HTTP
	FILE	*pFileDesc;											//  Pointer to the File Descriptor
	BOOL	bEOFReached;										//  EOF reached on the file
	DWORD	dwMemSize;											//  Size of memory for Domino memory request
	DWORD	dwMemOffset;										//  Current Offset for On Demand reading/writing
	char *pEnumMemBuffer;										//  Pointer into the memory buffer used by enumerators
	char szCurrentReadURL[MAX_URL + 1];							//  Current URL to Read
	char szCurrentFile[MAXPATH];								//  Current File to read
};
