//  Helper
//
//  This class contains a number of helper routines and functions that are useful for debugging.
//  In debug (_DEBUG) builds of the application then
//
//  Message Identifier:      DXR
//  Message Range:           0701 - 0799
//  Trace area assignment:   N/A
//  DX Version:				 3.11.1
//  Version:                 3.11.1  Build: 99
//
//  Version History:
//
//  3.11.1		22/20/2011			VS 2011 Updates
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2010 - 2011
//
#ifdef _DEBUG
//  Runtime Includes
#include "Helper.h"

//  Default Constructor
Helper::Helper()
{
}

//  Normal Constructor
Helper::Helper(ExecEnvironment *xeParent, int iThreadID)
:dwMonitorCount(0)
{
	char			szMsg[MAX_MSG + 1];						//  Generic message buffer

	iOwningThread = iThreadID;
	xeLocal = xeParent;
	sprintf_s(szMsg, MAX_MSG, MSG_DXR0701T, this);
	xeLocal->LogTrace(TRACE_CORERT, szMsg, iOwningThread);

	//  Send all debug reports to STDOUT and the Debugger Output Window (if running under a debugger)
	//  NOTE: This alters the default behaviour of assertion failures, you will not get a dialog box
	//        with the assertion failure message. Instead the assertion failure will be written to the
	//        output (STDOUT and Debugger) and the application will continue.
   _CrtSetReportMode(_CRT_WARN, _CRTDBG_MODE_FILE | _CRTDBG_MODE_DEBUG);
   _CrtSetReportFile(_CRT_WARN, _CRTDBG_FILE_STDOUT);
   _CrtSetReportMode(_CRT_ERROR, _CRTDBG_MODE_FILE | _CRTDBG_MODE_DEBUG);
   _CrtSetReportFile(_CRT_ERROR, _CRTDBG_FILE_STDOUT);
   _CrtSetReportMode(_CRT_ASSERT, _CRTDBG_MODE_FILE | _CRTDBG_MODE_DEBUG);
   _CrtSetReportFile(_CRT_ASSERT, _CRTDBG_FILE_STDOUT);

   //  Gather the initial memory stats
	SnapMemoryStats(TRUE, iThreadID);
}

//  Destructor
Helper::~Helper()
{
	char			szMsg[MAX_MSG + 1];						//  Generic message buffer

	sprintf_s(szMsg, MAX_MSG, MSG_DXR0702T, this);
	xeLocal->LogTrace(TRACE_CORERT, szMsg, iOwningThread);
}

//  ReportMemoryUsage
//
//  This function reports on the current memory usage including the delta from the start and last report.
//  If the reset flag is provided then the current values are used for the initial and last values
//  AFTER the report is logged.
//
//  Parameters
//
//  1	-	BOOL			- Flag indicating that the values should be reset after reporting on them
//	2	-	int				- ID of the invoking thread
//
//  Returns
//
//  API Use Notes:
//
//
void Helper::ReportMemoryUsage(BOOL bReset, int iThreadID)
{
	ReportMemoryUsage(bReset, FALSE, iThreadID);
	return;
}

//  ReportMemoryUsage
//
//  This function reports on the current memory usage including the delta from the start and last report.
//  If the reset flag is provided then the current values are used for the initial and last values
//  AFTER the report is logged.
//
//  Parameters
//
//  1	-	BOOL			- Flag indicating that the values should be reset after reporting on them
//  2	-	BOOL			- Flag indicating if output should be achoed to the console
//	3	-	int				- ID of the invoking thread
//
//  Returns
//
//  API Use Notes:
//
//
void Helper::ReportMemoryUsage(BOOL bReset, BOOL bEcho, int iThreadID)
{
	//  Get a new snapshot
	SnapMemoryStats(FALSE, iThreadID);

	//  Report on each statistic
	ReportOnStat("Working Set size", "Kb", dwCWSSize, dwLWSSize, dwIWSSize, dwPWSSize, bEcho, iThreadID);
	ReportOnStat("Paged Pool use", "Kb", dwCPPool, dwLPPool, dwIPPool, dwPPPool, bEcho, iThreadID);
	ReportOnStat("Non-Paged Pool use", "Kb", dwCNPPool, dwLNPPool, dwINPPool, dwPNPPool, bEcho, iThreadID);
	ReportOnStat("Normal Objects on the Heap", "", dwCHeapNormalCount, dwLHeapNormalCount, dwIHeapNormalCount, dwPHeapNormalCount, bEcho, iThreadID);
	ReportOnStat("Normal Objects Allocation", "Kb", dwCHeapNormalSize, dwLHeapNormalSize, dwIHeapNormalSize, dwPHeapNormalSize, bEcho, iThreadID);
	ReportOnStat("Client Objects on the Heap", "", dwCHeapClientCount, dwLHeapClientCount, dwIHeapClientCount, dwPHeapClientCount, bEcho, iThreadID);
	ReportOnStat("Client Objects Allocation", "Kb", dwCHeapClientSize, dwLHeapClientSize, dwIHeapClientSize, dwPHeapClientSize, bEcho, iThreadID);

	//  If a reset is required - then shuffle the stats
	if (bReset)
	{
		dwLWSSize = dwCWSSize;
		dwLPPool = dwCPPool;
		dwLNPPool = dwCNPPool;
		dwLHeapNormalCount = dwCHeapNormalCount;
		dwLHeapNormalSize = dwCHeapNormalSize;
		dwLHeapClientCount = dwCHeapClientCount;
		dwLHeapClientSize = dwCHeapClientSize;
		dwIWSSize = dwCWSSize;
		dwIPPool = dwCPPool;
		dwINPPool = dwCNPPool;
		dwIHeapNormalCount = dwCHeapNormalCount;
		dwIHeapNormalSize = dwCHeapNormalSize;
		dwIHeapClientCount = dwCHeapClientCount;
		dwIHeapClientSize = dwCHeapClientSize;
		dwPWSSize = dwCWSSize;
		dwPPPool = dwCPPool;
		dwPNPPool = dwCNPPool;
		dwPHeapNormalCount = dwCHeapNormalCount;
		dwPHeapNormalSize = dwCHeapNormalSize;
		dwPHeapClientCount = dwCHeapClientCount;
		dwPHeapClientSize = dwCHeapClientSize;
		dwMonitorCount = 0;
	}
	return;
}

//  CreateMemoryDump
//
//  This function will create a core dump of the applcation for later inspection in a debugger.
//
//  Parameters
//
//	1	-	int			- ID of the invoking thread.
//
//  Returns
//
//  API Use Notes:
//
//
void Helper::CreateMemoryDump(int iThreadID)
{
	HANDLE			hfDump;									//  File handle for the dump file
	char			szDumpFile[MAXPATH + 1];				//  Name of the dump file
	char			szMsg[MAX_MSG + 1];						//  Generic message buffer
	
	//  Get the name of the dump file - this will be written to the IBM_TECHICAL_SUPPORT directory
	GetDumpFileName(szDumpFile, MAXPATH, iThreadID);
	if (szDumpFile[0] == '\0')
	{
		xeLocal->LogMessage(MSG_DXR0705E, iThreadID);
		return;
	}
	
	//  Open a file handle for the dump file
	hfDump = CreateFile(szDumpFile, GENERIC_READ|GENERIC_WRITE, FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
	if (hfDump == (HANDLE) HFILE_ERROR)
	{
		sprintf_s(szMsg, MAX_MSG, MSG_DXR0706E, szDumpFile);
		xeLocal->LogMessage(szMsg, iThreadID);
		return;
	}

	//  Write the memory dump to the file
	MiniDumpWriteDump(GetCurrentProcess(),
					  GetCurrentProcessId(),
					  hfDump,
					  MiniDumpWithFullMemory,
					  NULL,
					  NULL,
					  NULL);

	//  Close the dump file
	CloseHandle(hfDump);

	//  Report that a dump has been taken
	sprintf_s(szMsg, MAX_MSG, MSG_DXR0707I, szDumpFile);
	xeLocal->LogMessage(szMsg, iThreadID);

	return;
}

//  ReportOnStat
//
//  This function will Log a line showing the values for a single statistic.
//
//  Parameters
//
//  1	-	char *			-  Label for the statistic
//  2	-	char *			-  Unit Of Measure for the statistic
//  3	-	SIZE_T			-  Current value of the statistic
//	4	-	SIZE_T			-  Previous value of the statistic
//	5	-	SIZE_T			-  Initial value of the statistic
//  6	-	SIZE_T			-  Peak value of the statistic
//	7	-	BOOL			-  Echo output to the console
//	8	-	int				-  ID of the calling thread
//
//  Returns
//
//  API Use Notes:
//
//
void Helper::ReportOnStat(char *StatName, char *Measure, SIZE_T dwCurrent, SIZE_T dwLast, SIZE_T dwInitial, SIZE_T dwPeak, BOOL bEcho, int iThreadID)
{
	SIZE_T			dwDeltaLast = 0;						//  Clear the delta
	SIZE_T			dwDeltaInit = 0;						//  Clear the delta
	char			szDeltaInit[2];							//  Sign of delta from initial value
	char			szDeltaLast[2];							//  Sign of delta from last value
	char			szMsg[MAX_MSG + 1];						//  Generic message buffer
	
	szDeltaInit[0] = '\0';
	szDeltaInit[1] = '\0';
	szDeltaLast[0] = '\0';
	szDeltaLast[1] = '\0';

	//  Determine and set the sign for the initial and last delta
	if (dwCurrent != dwInitial)
	{
		if (dwCurrent > dwInitial) 
		{
			szDeltaInit[0] = '+';
			dwDeltaInit = dwCurrent - dwInitial;
		}
		else 
		{
			szDeltaInit[0] = '-';
			dwDeltaInit = dwInitial - dwCurrent;
		}
	}
	if (dwCurrent != dwLast)
	{
		if (dwCurrent > dwLast) 
		{
			szDeltaLast[0] = '+';
			dwDeltaLast = dwCurrent - dwLast;
		}
		else 
		{
			szDeltaLast[0] = '-';
			dwDeltaLast = dwLast - dwCurrent;
		}
	}

	//  Format the stats message line
	sprintf_s(szMsg, MAX_MSG, MSG_DXR0704I, dwMonitorCount,
								StatName,
								dwCurrent,
								Measure,
								szDeltaLast,
								dwDeltaLast,
								Measure,
								szDeltaInit,
								dwDeltaInit,
								Measure,
								dwPeak,
								Measure);
	xeLocal->LogMessage(szMsg, iThreadID);
	if (bEcho) AddInLogMessageText(szMsg, NOERROR);
	return;
}

//  SnapMemoryStats
//
//  This function will create a new (Current) snapshot of the memory stats, it will save Current values as
//  last values and maintain the Peak values if they have changed.
//  If the InitialSnap flag is provided then initial, last, current and peak values are populated with the
//  same value.
//
//  Parameters
//
//  1	-	BOOL		-  Switch indicating if counters are to be reset
//  2	-	int			-  ID of the calling thread
//
//  Returns
//
//  API Use Notes:
//
//
void Helper::SnapMemoryStats(BOOL bInitialSnap, int iThreadID)
{
	DWORD						dwLastError;					//  Last error code
	PROCESS_MEMORY_COUNTERS		pmc;							//  Process Memory Information structure
	_CrtMemState				HeapStats;						//  CRT Heap Stats
	char						szMsg[MAX_MSG + 1];				//  Generic message buffer

	//  If this is not the initial snap of memory then cycle the Current values to last
	if (!bInitialSnap)
	{
		dwLWSSize = dwCWSSize;
		dwLPPool = dwCPPool;
		dwLNPPool = dwCNPPool;
		dwLHeapNormalCount = dwCHeapNormalCount;
		dwLHeapNormalSize = dwCHeapNormalSize;
		dwLHeapClientCount = dwCHeapClientCount;
		dwLHeapClientSize = dwCHeapClientSize;
	}

	//  Get the Current (new Snap) values for the memory stats
	if (!GetProcessMemoryInfo( GetCurrentProcess(), &pmc, sizeof(pmc)))
	{
		dwLastError = GetLastError();
		xeLocal->LogMessage(MSG_DXR0703E, iThreadID);
		xeLocal->GetWINAPIError(dwLastError, szMsg);
		xeLocal->LogMessage(szMsg, iThreadID);
		return;
	}
	_CrtMemCheckpoint(&HeapStats);

	//  Populate the current settings
	dwCWSSize = pmc.WorkingSetSize / BYTES_PER_KB;
	dwCPPool = pmc.QuotaPagedPoolUsage / BYTES_PER_KB;
	dwCNPPool = pmc.QuotaNonPagedPoolUsage / BYTES_PER_KB;
	dwCHeapNormalCount = HeapStats.lCounts[_NORMAL_BLOCK];
	dwCHeapNormalSize = HeapStats.lSizes[_NORMAL_BLOCK] / BYTES_PER_KB;
	dwCHeapClientCount = HeapStats.lCounts[_CLIENT_BLOCK];
	dwCHeapClientSize = HeapStats.lSizes[_CLIENT_BLOCK] / BYTES_PER_KB;

	//  If this is the intial Snap (or a reset) then populate the Initial & Last values
	if (bInitialSnap)
	{
		dwLWSSize = dwCWSSize;
		dwLPPool = dwCPPool;
		dwLNPPool = dwCNPPool;
		dwLHeapNormalCount = dwCHeapNormalCount;
		dwLHeapNormalSize = dwCHeapNormalSize;
		dwLHeapClientCount = dwCHeapClientCount;
		dwLHeapClientSize = dwCHeapClientSize;
		dwIWSSize = dwCWSSize;
		dwIPPool = dwCPPool;
		dwINPPool = dwCNPPool;
		dwIHeapNormalCount = dwCHeapNormalCount;
		dwIHeapNormalSize = dwCHeapNormalSize;
		dwIHeapClientCount = dwCHeapClientCount;
		dwIHeapClientSize = dwCHeapClientSize;
		dwPWSSize = 0;
		dwPPPool = 0;
		dwPNPPool = 0;
		dwPHeapNormalCount = 0;
		dwPHeapNormalSize = 0;
		dwPHeapClientCount = 0;
		dwPHeapClientSize = 0;
		dwMonitorCount = 0;
	}
	else dwMonitorCount++;

	//  Update the peak values
	if (dwCWSSize > dwPWSSize) dwPWSSize = dwCWSSize;
	if (dwCPPool > dwPPPool) dwPPPool = dwCPPool;
	if (dwCNPPool > dwPNPPool) dwPNPPool = dwCNPPool;
	if (dwCHeapNormalCount > dwPHeapNormalCount) dwPHeapNormalCount = dwCHeapNormalCount;
	if (dwCHeapNormalSize > dwPHeapNormalSize) dwPHeapNormalSize = dwCHeapNormalSize;
	if (dwCHeapClientCount > dwPHeapClientCount) dwPHeapClientCount = dwCHeapClientCount;
	if (dwCHeapClientSize > dwPHeapClientSize) dwPHeapClientSize = dwCHeapClientSize;

	return;
}

//  GetDumpFileName
//
//  This function will generate the full file path for the file that a memory dump is to be written to.
//	1)  The file will be placed in the IBM_TECHNICAL_SUPPORT subdirectory.
//  2)  The file name will be generated from the application name and the date and time.
//
//  Parameters
//
//	1	-	char *		- Pointer to the buffer that will contain the generated name
//	2	-	int			- Size of the buffer that will contain the generated file name
//	3	-	int			- ID of the invoking thread.
//
//  Returns
//
//  API Use Notes:
//
//
void Helper::GetDumpFileName(char *szName, int iSize, int iThreadID)
{
	WORD			wRetLen;								//  Returned length
	TIME			tNow;									//  Current time/date and components
	char			szDumpDir[MAXPATH + 1];					//  Name of the dump file
	char			szDumpFile[MAXPATH + 1];				//  Name of the dump file
	char			szMsg[MAX_MSG + 1];						//  Generic message buffer

	//  Clear the returned file name
	szName[0] = '\0';

	//  Start off by forming the name of the directory
	wRetLen = OSGetDataDirectory(szDumpFile);
	szDumpFile[wRetLen] = '\0';
	sprintf_s(szDumpDir, MAXPATH, "%s/%s/", szDumpFile, DUMP_DIR);

	//  Get the current date/time to format the file name
	OSCurrentTIMEDATE(&tNow.GM);
	TimeGMToLocal(&tNow);

	//  Format the completed file path
	sprintf_s(szDumpFile, MAXPATH, "%s%s%s%04i%02i%02i-%02i%02i%02i%s",
						szDumpDir,
						DUMP_PREFIX,
						xeLocal->rsLocal->APPName,
						tNow.year,
						tNow.month,
						tNow.day,
						tNow.hour,
						tNow.minute,
						tNow.second,
						DUMP_TYPE);
	//  Check on the length
	if ((int) strlen(szDumpFile) > iSize)
	{
		xeLocal->LogMessage(szMsg, iThreadID);
		return;
	}

	//  Copy the file name back to the caller provided buffer
	
	strcpy_s(szName, iSize, szDumpFile);
	return;
}
#endif
