#pragma once
//  Helper
//
//  This class contains a number of helper routines and functions that are useful for debugging.
//  In debug (_DEBUG) builds of the application then
//
//  Message Identifier:      DXR
//  Message Range:           0701 - 0799
//  Trace area assignment:   N/A
//  DX Version:				 3.11.1
//  Version:                 3.11.1  Build: 99
//
//  Version History:
//
//  3.11.1		22/20/2011			VS 2011 Updates
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2010 - 2011
//

//  Note: The helper class is only instantiated in Debug Builds of the kernel.
#ifdef _DEBUG

//  Platform Includes
#include "../Platform/PlatBase.h"
#include <Dbghelp.h>
#include <Psapi.h>	

//  Runtime Includes
#include "../ExecEnvironment.h"

//  Local Definitions
#define		BYTES_PER_KB		1024
#define		DUMP_DIR			"IBM_TECHNICAL_SUPPORT"
#define		DUMP_PREFIX			"DXDump-"
#define		DUMP_TYPE			".dmp"

//  Trace Messages
#define		MSG_DXR0701T		"DXR0701T: A new Debug Helper instance has been created @%p."
#define		MSG_DXR0702T		"DXR0702T: A Debug Helper instance  @%p has been destroyed."

//  Information Messages
#define		MSG_DXR0704I		"DXR0704I: Current(%i) %s: %i %s, %s%i %s since last measured, %s%i %s since first measured, Peak: %i %s."
#define		MSG_DXR0707I		"DXR0707I: A full memory dump has been written to file '%s'."

//  Error Messages
#define		MSG_DXR0703E		"DXR0703E: Unable to get the current process memory information."
#define		MSG_DXR0705E		"DXR0705E: The debug helper was not able to generate the name for the dump file, dump skipped."
#define		MSG_DXR0706E		"DXR0706E: Unable to create the dump file '%s', dump skipped."
#define		MSG_DXR0708E		"DXR0708E: Generated dump file name has exceeded the maximum allowed length, dump skipped."

class Helper
{
public:
	//  Default Constructor
	Helper(void);

	//  Normal Constructor
	Helper(ExecEnvironment *xeParent, int iThreadID);

	//  Destructor
	~Helper(void);

	//  Public Members

	//  Memory Monitor
	DWORD					dwMonitorCount;						//  Monitor cycle count
	SIZE_T					dwIWSSize;							//  Initial Working Set Size
	SIZE_T					dwLWSSize;							//  Last Working Set Size
	SIZE_T					dwCWSSize;							//  Current Working Set Size
	SIZE_T					dwPWSSize;							//  Peak Working Set Size
	SIZE_T					dwIPPool;							//  Initial Paged Pool Size
	SIZE_T					dwLPPool;							//  Last Paged Pool Size
	SIZE_T					dwCPPool;							//  Current Paged Pool Size
	SIZE_T					dwPPPool;							//  Peak Paged Pool Size
	SIZE_T					dwINPPool;							//  Initial Non-Paged Pool Size
	SIZE_T					dwLNPPool;							//  Last Non-Paged Pool Size
	SIZE_T					dwCNPPool;							//  Current Non-Paged Pool Size
	SIZE_T					dwPNPPool;							//  Peak Non-Paged Pool Size
	SIZE_T					dwIHeapNormalCount;					//  Initial Count of Normal Heap Blocks
	SIZE_T					dwLHeapNormalCount;					//  Last Count of Normal Heap Blocks
	SIZE_T					dwCHeapNormalCount;					//  Current Count of Normal Heap Blocks
	SIZE_T					dwPHeapNormalCount;					//  Peak Count of Normal Heap Blocks
	SIZE_T					dwIHeapNormalSize;					//  Initial Size of Normal Heap Blocks
	SIZE_T					dwLHeapNormalSize;					//  Last Size of Normal Heap Blocks
	SIZE_T					dwCHeapNormalSize;					//  Current Size of Normal Heap Blocks
	SIZE_T					dwPHeapNormalSize;					//  Peak Size of Normal Heap Blocks
	SIZE_T					dwIHeapClientCount;					//  Initial Count of Client Heap Blocks
	SIZE_T					dwLHeapClientCount;					//  Last Count of Client Heap Blocks
	SIZE_T					dwCHeapClientCount;					//  Current Count of Client Heap Blocks
	SIZE_T					dwPHeapClientCount;					//  Peak Count of Client Heap Blocks
	SIZE_T					dwIHeapClientSize;					//  Initial Size of Client Heap Blocks
	SIZE_T					dwLHeapClientSize;					//  Last Size of Client Heap Blocks
	SIZE_T					dwCHeapClientSize;					//  Current Size of Client Heap Blocks
	SIZE_T					dwPHeapClientSize;					//  Peak Size of Client Heap Blocks

	//  Public Functions
	void ReportMemoryUsage(BOOL bReset, int iThreadID);					//  Report on Memory Usage
	void ReportMemoryUsage(BOOL bReset, BOOL bEcho, int iThreadID);		//  Report on Memory Usage
	void CreateMemoryDump(int iThreadID);								//  Create a minidump memory dump

private:
	//  Private Members
	ExecEnvironment			*xeLocal;									//  Runtime environment
	int						iOwningThread;								//  Owning thread

	//  Private Functions
	void SnapMemoryStats(BOOL bInitialSnap, int iThreadID);				//  Record current memory stats
	void ReportOnStat(char *StatName, char *Measure, SIZE_T dwCurrent, SIZE_T dwLast, SIZE_T dwInitial, SIZE_T dwPeak, BOOL bEcho, int iThreadID);
	void GetDumpFileName(char *szName, int iSize, int iThreadID);		//  Get the name of the memory dump file

};
#endif
