#pragma once
//  ExecEnvironment
//
//  This class provides all of the common runtime support for Domino eXplorer (DX) applications.
//
//  Message Identifier:      DXR
//  Message Range:           0001 - 0499
//  Trace area assignment:   TRACE_CORERT (01)
//  DX Version:				 3.14.1
//  Version:                 3.14.1  build: 109
//
//  Version History:
//
//  1.0.0		10/02/2001		Initial version
//  3.5.3		21/08/2009		Re-based on DX 3.x.x
//  3.6.0		18/09/2009		New log handler for multithreaded applications
//  3.6.1		10/10/2009		Additional controls added
//  3.6.2		20/11/2009		Unix compat changes plus minor fixes/enhancements
//  3.6.3		19/12/2009		Enhancements to DbMover
//  3.6.4		23/12/2009		Additional fixes to DbMover
//  3.7.0		23/12/2009		Added race auto-correction to MT support
//  3.7.1		29/12/2009		AppWait() updated to eliminate overhead
//  3.7.2		12/01/2010		DbCopier improvements for corrupt folder handling
//								Remote Fixup on DbOpen
//								Logger improvements
//  3.7.3		06/02/2010		DbMover & DbCopier improvements
//  3.7.4		19/03/2010		Fixed various issues with log/repository configuration
//  3.7.5		30/03/2010		Improved Error Handling in DbCopier
//  3.7.6		08/04/2010		Fixed a problem with error handling
//  3.7.7		10/04/2010		Additional recovery options in DbCopier
//  3.8.0		15/05/2010		DCF stub calls added to core
//								Load & Execute functions added
//  3.8.3       25/08/2010		Prevent escape % characters being passed to console logger
//  3.8.4		01/07/2010		(spl) Enhancements to OSLoadProg functions
//								Changed return code from simple BOOL to int
//								Added code to get RC of program if we are not running in detached mode
//  3.9.0		24/07/2010		Linkage changed for logging & request posting
//								Additional functions added GenUniqueID(), populateUNID() & xtoi()
//  3.9.1		02/09/2010		Changes to the ACL Processors
//								Fix for false RJR_NONE_EXIST signals
//  3.9.2		20/09/2010		UNID handling routines added
//								Fixes for ACL Processor
//								Avoided a recursion in DX processing
//  3.10.0		01/10/2010		Wildcard & multi-thread execution added to DX processing
//								DXNoteScanner incorporated in kernel
//  3.10.1		15/10/2010		Asynch Transaction Handler added
//  3.10.2		18/11/2010		MTIO support added to DbMover
//  3.10.3		13/12/2010		Db Property Modifier added
//								Debug Helper added
//  3.10.4		14/01/2011		Command handler supports automated command injection
//  3.10.5		18/02/2011		DX changes to support reporting on D/Bs that cannot be opened
//  3.10.6		19/03/2011		Upload capability added to DXResourceLoader
//  3.10.7		17/05/2011		Fixed an issue with ACL processing
//  3.10.8		30/05/2011		Transaction scheduling added to transaction handler
//  3.10.9		04/06/2011		Some tidying up for Linux
//  3.10.10		14/07/2011		Performance Improvements for/to DbCopier
//  3.10.11		21/07/2011		ObjectCache added to Kernel
//  3.10.12		25/08/2011		Fix timing reporting in OpenDatabase
//  3.11.0		29/09/2011		NamedEntity support added
//  3.11.1		22/10/2011		VS 2011 Readiness
//  3.11.2		04/11/2011		Migration of D/B conditioning from kernel to Design Manager
//  3.11.3		12/11/2011		Fixes to Explorer and ACL Processor
//  3.11.4		27/11/2011		Adopt TimeBasedArray, fixes for ACL processor
//  3.11.5		30/11/2011		Explorer recursion improvements
//								Document count changes for R5 servers
//								Transaction scheduler last run time caclulation changed
//								ObjectCache coherence improvements
//								Additional duplicate match messages in NEResolver
//  3.11.6		10/12/2011		OpenDatabase handles Replica IDs
//								Log writer improvements
//  3.12.0		21/12/2011		DXIP introduced
//	3.13.0		20/12/2013		Additional D/B Create options
//	3.14.0		19/01/2015		x64 compatability fixes
//	3.14.1		24/02/2015		Avoid a panic in DbCopier
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2001 - 2015
//

//  Platform Includes
#include	"Platform/PlatBase.h"
#include	"Platform/NotesBase.h"

//  Runtime Includes
#include "RunSettings.h"
#include "APIPackages.h"
#include "ElapsedTimer.h"

//  Macros
#define		TODClock(x)				x->RunningTime->getElapsed()

//  Local Constants
#define		RT_BUILD				109
#define		RT_IDENTITY				"DXCommon ST Runtime 3.14.1 (build: %i)"
#define		START_DUMP				"+---- Start of Dump: %s -------------- %i (0x%.4x) bytes @%p --------------"
#define		END_DUMP				"+---- End of Dump: %s -------------- %i (0x%.4x) bytes @%p --------------"
#define		DUMP_LINE				"%04X: %s  %s :%s %s"
#define		DEFAULT_ACL_USER		"-Default-"
#define		ANONYMOUS_ACL_USER		"Anonymous"
#define		LDS_ACL_GROUP			"LocalDomainServers"
#define		LDA_ACL_GROUP			"LocalDomainAdmins"
#define		DEFAULT_REPOSITORY		"log.nsf"					//  Default repository for persistent logger
#define		PSEUDO_WAIT_CHUNK		5							//  Application Waits are broken down to 5 Ms chunks
#define		SPECIAL_FTI_ID			0x0016						//  Special Note Class for FTI definitions UNDOCUMENTED
#define		FTI_DEF_OBJ_ID			0x00000016					//  Special Note ID for FTI definitions UNDOCUMENTED
#define		DBINFO_FLAG_VALUE		0x03FF						//  Flags for DB Info Call UNDOCUMENTED
#ifdef WIN32
#define		PLATFORM_APILIB_NAME	"nnotes.dll"				//  DLL containing Notes API entry points
#endif
#ifdef UNIX
#ifdef LINUX
#define		PLATFORM_APILIB_NAME	"libnotes.so"				//  Shared object containing Notes API entry points
#endif
#ifdef AIX
#define		PLATFORM_APILIB_NAME	"libnotes_r.a"				//  Shared object containing Notes API entry points
#endif
#endif
#define		NSF_SN_EPNAME			"NSFDbSetSpecialNoteID"		//  Undocumented API Call
#define		NSF_GDI_EPNAME			"NSFGetDbInfo"				//  Undocumented API Call
#define		NSF_GFS_EPNAME			"NSFDbFileSummary"			//  Undocumented API Call
#define		NSF_GETDBO_EPNAME		"NSFDbGetOptionsExt"		//  Undocumented API Call
#define		TRACE_FORMATTER			"%s (%i)"					//  Formatter for Trace Messages
#define		INIT_STATUS_LINE		"%s (%s) is initialising."	//  Initial default text
#define		MAX_PENDING_LOGLINES	10							//  Max 10 log lines pending
#define		MAX_STALE_LOG_MINS		5							//  Max 5 minutes since last log write
#define		SERVER_TIMEOUT			0							//  Use port timeout
#define		SINGLE_WAIT_QUANTUM		50							//  Wait of less than 50 Ms will not be chnked
#define		OPEN_FIXUP_FLAGS		DBOPEN_FORCE_FIXUP | DBOPEN_FIXUP_FULL_NOTE_SCAN  //  DbOPen Fixup Flag Bits
#define		FORMAT_FIXUP_CMD		"load Fixup %s -J -F"		//  Format for the fixup command
#define		FIXUP_WAIT_INTERVAL		5000						//  5 seconds wait interval
#define		FIXUP_MINIWAIT_INTERVAL	3000						//  3 seconds wait interval
#define		FIXUP_WAIT_LIMIT		60							//  5 minutes wait limit
#define		FIXUP_PROGRESS_LIMIT	120							//  10 minutes wait limit
#define		DEFAULT_OFFLINE_WAIT	10000						//  Wait for 10s for database to go offline
#define		FORMULA_GENUNIQUEID		"@@171@@"					//  Escaped value for "@UNIQUE()"
#define		LENGTH_UNIQUEID			20							//  Length of unique ID
#define		FORMAT_UNID_STRING		"%008lX%008lX%008lX%008lX"	//  UNID Formatter
#define		FORMAT_UNID_STRING_EXT	"OF: %008lX:%008lX - ON: %008lX:%008lX"		//  Decorated UNID Formatter
#define		FORMAT_REPID			"%008lX:%008lX"				//  Decorated Replica ID Formatter
#define		FORMAT_DVERSION			"R%i.%i.%i FP:%i HF:%i Build:%i"			//  Decorated Domino version
#define		MAX_PKGID				40							//  Max size of Pkg ID string
#define		BYTES_PER_KB			1024						//  Bytes per Kb.
#define		CATALOG_LU_REPID		"($ReplicaID)"				//  Lookup by Replica ID view in the database catalog
#define		CATALOG_REPID_COL		"$6"						//  Column name for the Replica ID value
#define		CATALOG_SERVER_COL		"$5"						//  Column name for the Server Name value

//  Posted Log Message Mutex - special values
#define		LOG_MUTEX_FREE		-1								//  Mutex is available

//  OSLoadProg Return Codes
#define		RC_OSPROG_OK		0
#define		RC_OSPROG_CP		1
#define		RC_OSPROG_GECP		2
#define		RC_OSPROG_SA		259

//  Trace Messages
#define		MSG_DXR0051T		"DXR0051T: Updating entry '%s' in pattern ACL."
#define		MSG_DXR0062T		"DXR0062T: The Notes/Domino runtime environment has been initialised."
#define		MSG_DXR0063T		"DXR0063T: The default Add-In status line has been set."
#define		MSG_DXR0064T		"DXR0064T: The Add-In MQ Queue Name has been allocated."
#define		MSG_DXR0065T		"DXR0065T: The default Add-In status line has been reset."
#define		MSG_DXR0066T		"DXR0066T: The Add-In MQ Queue Name has been opened."
#define		MSG_DXR0067T		"DXR0067T: The runtime user name has been determined as '%s'."
#define		MSG_DXR0068T		"DXR0068T: The repository database has been opened/created."
#define		MSG_DXR0069T		"DXR0069T: The runtime logger has been initialised."
#define		MSG_DXR0070T		"DXR0070T: The DX runtime has been fully initialised."
#define		MSG_DXR0071T		"DXR0071T: Commencing shutdown of the runtime environment."
#define		MSG_DXR0072T		"DXR0072T: The logging interface has been terminated."
#define		MSG_DXR0095T		"DXR0095T: About to execute program '%s' in working directory '%s'."
#define		MSG_DXR0096T		"DXR0096T: Program will use command '%s' and Flags: (0x%.8lX)."
#define		MSG_DXR0098T		"DXR0098T: Program '%s' has been successfully spawned in a detached process."
#define		MSG_DXR0100T		"DXR0100T: Program '%s' with command '%s' completed with a return code of '%d'."

//  Information Messages
#define		MSG_DXR0001I		"DXR0001I: Notes runtime has been initialised."
#define		MSG_DXR0002I		"DXR0002I: Notes runtime has been terminated."
#define		MSG_DXR0004I		"DXR0004I: The repository database has been closed."
#define		MSG_DXR0006I		"DXR0006I: The logging interface has been initialised for %s %s(%s)."
#define		MSG_DXR0015I		"DXR0015I: Database/Directory '%s' has been successfully opened."
#define		MSG_DXR0017I		"DXR0017I: The execution environment has been fully initialised."
#define		MSG_DXR0018I		"DXR0018I: The logging interface is being closed."
#define		MSG_DXR0023I		"DXR0023I: Dump of %s (@%p) has been supressed due to the logging level."
#define		MSG_DXR0024I		"DXR0024I: AddIn %s (%s) is running, version: %s"
#define		MSG_DXR0027I		"DXR0027I: AddIn has opened the Message Queue '%s'."
#define		MSG_DXR0035I		"DXR0035I: Database '%s' has been created on '%s'."
#define     MSG_DXR0062I		"DXR0062I: There is a quit pending on the message queue - attempting to drain the queue."
#define     MSG_DXR0063I		"DXR0063I: A quit message has been consumed from the Message Queue."
#define		MSG_DXR0074I		"DXR0074I: Remote open on server '%s' %s took %i ms (Latency: %i = %i + %i ms.)."
#define		MSG_DXR0075I		"DXR0075I: Local open on '%s'  %s took %i ms." 
#define		MSG_DXR0079I		"DXR0079I: This database does not require fixup as the timestamp is later than the modification times."
#define		MSG_DXR0082I		"DXR0082I: %s>> %s."
#define		MSG_DXR0085I		"DXR0085I: Fixup has completed for database '%s' on '%s'."
#define		MSG_DXR0091I		"DXR0091I: Latest Data Modification Timestamp: %s (%.8lX:%.8lX)."
#define		MSG_DXR0092I		"DXR0092I: Latest Non-Data Modification Timestamp: %s (%.8lX:%.8lX)."
#define		MSG_DXR0093I		"DXR0093I: Latest Database Fixup Timestamp: %s (%.8lX:%.8lX)."
#define		MSG_DXR0106I		"DXR0106I: The runtime was started at: %s."
#define		MSG_DXR0107I		"DXR0107I: The remote database will be fixed up using the current time."
#define		MSG_DXR0150I		"DXR0150I: Replica ID: %s resolved to '%s' on '%s'."

//  Warning Messages
#define		MSG_DXR0016W		"DXR0016W: The logging interface has been closed because of destruction of the runtime."
#define		MSG_DXR0019W		"DXR0019W: The logging interface was not initialised, continuing with console logging only."
#define		MSG_DXR0039W		"DXR0039W: Unable to set the default ACL entry to No Access."
#define		MSG_DXR0040W		"DXR0040W: Unable to add the anonymous ACL entry with No Access."
#define		MSG_DXR0041W		"DXR0041W: Unable to add the server ACL entry with Manager Access."
#define		MSG_DXR0042W		"DXR0042W: Unable to add the LocalDomainServers ACL entry with Manager Access."
#define		MSG_DXR0043W		"DXR0043W: Unable to add the LocalDomainAdmins ACL entry with Manager Access."
#define		MSG_DXR0050W		"DXR0050W: Database '%s' on '%s' already exists, using that database."
#define		MSG_DXR0094W		"DXR0094W: Persistent logging will be directed to the local notes log."
#define		MSG_DXR0127W		"DXR0127W: Unable to locate the default view design element after it was created."
#define		MSG_DXR0128W		"DXR0128W: Failed to set the default view as the default view class entry."

//  Error Messages
#define		MSG_DXR0003S		"DXR0003S: Unable to open the repository database."
#define		MSG_DXR0005E		"DXR0005E: Unable to close the repository database."
#define		MSG_DXR0007E		"DXR0007E: Logging has been terminated due to errors creating a new log document."
#define		MSG_DXR0008E		"DXR0008E: Failed to create a new Event log document."
#define		MSG_DXR0009E		"DXR0009E: Unable to write item '%s' to a new Event log document."
#define		MSG_DXR0010E		"DXR0010E: Unable to construct a timestamp for a log entry."
#define		MSG_DXR0011E		"DXR0011E: Unable to write a log entry to a log document."
#define		MSG_DXR0012S		"DXR0012S: Unable to canonicalize server name '%s'."
#define		MSG_DXR0013S		"DXR0013S: Unable to construct a net path from '%s' and '%s'."
#define		MSG_DXR0014E		"DXR0014E: Unable to open database '%s'."
#define		MSG_DXR0020E		"DXR0020E: Unable to determine the current user name."
#define		MSG_DXR0021E		"DXR0021E: Unable to determine the Mode of the current database."
#define		MSG_DXR0022E		"DXR0022E: Platform API Error: 0x%04X - %s."
#define		MSG_DXR0025E		"DXR0025E: Unable to create the Message Queue (MQ) for '%s'."
#define		MSG_DXR0026E		"DXR0026E: Unable to open the Message Queue (MQ) for '%s'."
#define		MSG_DXR0028E		"DXR0028E: Unable to open the design collection."
#define		MSG_DXR0029E		"DXR0029E: Unable to read the design collection."
#define		MSG_DXR0030E		"DXR0030E: A null collection was returned while reading the design collection."
#define		MSG_DXR0031E		"DXR0031E: Unable to determine the class of a design note."
#define		MSG_DXR0032E		"DXR0032E: Unable to build a view index."
#define		MSG_DXR0033E		"DXR0033E: Unable to create the requested database, it already exists."
#define		MSG_DXR0034E		"DXR0034E: Unable to create the requested database."
#define		MSG_DXR0036E		"DXR0036E: Unable to set the default ACL in the new database."
#define		MSG_DXR0037E		"DXR0037E: Unable to create a new ACL note."
#define		MSG_DXR0038E		"DXR0038E: Unable to add the current user to the ACL as Manager."
#define		MSG_DXR0044E		"DXR0044E: Unable to write the updated ACL to the new database."
#define		MSG_DXR0045E		"DXR0045E: Unable to get the current database information buffer."
#define		MSG_DXR0046E		"DXR0046E: Unable to set the current database information buffer."
#define		MSG_DXR0047E		"DXR0047E: Unable to open the Db Icon note to modify the title."
#define		MSG_DXR0048E		"DXR0048E: Unable to update the Db title in the Icon note."
#define		MSG_DXR0049E		"DXR0049E: Unable to update the Db Icon note to modify the title."
#define		MSG_DXR0052E		"DXR0052E: Unable to get the DBID of the current database."
#define		MSG_DXR0053E		"DXR0053E: Unable to get the replication information from the source database."
#define		MSG_DXR0054E		"DXR0054E: Unable to set the replication information in the target database."
#define		MSG_DXR0055E		"DXR0055E: Unable to copy the ACL from the source to the target database."
#define		MSG_DXR0056E		"DXR0056E: Unable to get the DB Class of the source database."
#define		MSG_DXR0057E		"DXR0057E: Unable to get the note Class of a design note."
#define		MSG_DXR0058E		"DXR0058E: Unable to open a design note."
#define		MSG_DXR0059E		"DXR0059E: Unable to load the Notes API library %s."
#define		MSG_DXR0060E		"DXR0060E: Unable to find '%s' in library '%s'."
#define		MSG_DXR0061S		"DXR0061S: A NULL object reference was passed to an initialisation routine."
#define		MSG_DXR0064E		"DXR0064E: Unable to read the Database Icon flags."
#define		MSG_DXR0073E		"DXR0073E: Unable to obtain the Infoblk structure from a database."
#define		MSG_DXR0076E		"DXR0076E: Unable to fixup and re-open database '%s'."
#define		MSG_DXR0077E		"DXR0077E: Unable to get the path to the currently open database."
#define		MSG_DXR0078E		"DXR0078E: Unable to obtain the modification timestamps for '%s'."
#define		MSG_DXR0080E		"DXR0080E: Unable to parse database path '%s'."
#define		MSG_DXR0081E		"DXR0081E: An error was detected issuing the '%s' command to '%s'."
#define		MSG_DXR0083E		"DXR0083E: Fixup progress timeout limit has been exceeded."
#define		MSG_DXR0084E		"DXR0084E: Unable to re-open remote database database '%s'."
#define		MSG_DXR0086E		"DXR0086E: Unable to get the latest fixup timestamp."
#define		MSG_DXR0087E		"DXR0087E: Fixup timeout limit has been exceeded."
#define		MSG_DXR0088E		"DXR0088E: Unable to fixup database '%s'."
#define		MSG_DXR0089E		"DXR0089E: Unable to get the DB Summary Buffer for the database."
#define		MSG_DXR0090E		"DXR0090E: Unable to locate the last fixup timestamp in the DB Summary Buffer."
#define		MSG_DXR0097E		"DXR0097E: CreateProcess failed while loading program '%s' - (%d)."
#define		MSG_DXR0099E		"DXR0099E: GetExitCodeProcess failed while getting rc from program '%s' - (%d)."
#define		MSG_DXR0101E		"DXR0101E: Failed to compile formula to generate unique ID."
#define		MSG_DXR0102E		"DXR0102E: Failed to create pointer to compiled formula to generate unique ID."
#define		MSG_DXR0103E		"DXR0103E: Failed to create compute handle to generate unique ID."
#define		MSG_DXR0104E		"DXR0104E: Failed to evaluate formula to generate unique ID."
#define		MSG_DXR0105E		"DXR0105E: Failed to stop evaluation of formula to generate unique ID."
#define		MSG_DXR0108E		"DXR0108E: Failed to get the Role name associated with privilege %i."
#define		MSG_DXR0109E		"DXR0109E: '%s' failed because source note handle was NULL."
#define		MSG_DXR0110E		"DXR0110E: '%s' failed because pointer to target string buffer was NULL."
#define		MSG_DXR0111S		"DXR0111S: **PANIC** in module '%s' at line %i, Reason: '%s' in Thread: %i."
#define		MSG_DXR0112E		"DXR0112E: Failed to allocate a %i byte memory buffer."
#define		MSG_DXR0113E		"DXR0113E: Failed to reallocate a %i byte memory buffer."
#define		MSG_DXR0140E		"DXR0140E: Unable to read the replication information from the specified database."
#define		MSG_DXR0141E		"DXR0141E: Failed to get the modified note table to count data documents."
#define		MSG_DXR0142E		"DXR0142E: Unable to locate the '%s' view in the database catalog."
#define		MSG_DXR0143E		"DXR0143E: Unable to open the '%s' view in the database catalog."
#define		MSG_DXR0144E		"DXR0144E: Replica ID: '%s' does not exist in the database catalog on '%s'."
#define		MSG_DXR0145E		"DXR0145E: Lookup for Replica ID: '%s' in the database catalog on '%s' failed."
#define		MSG_DXR0146E		"DXR0146E: An error was encountered while reading the Replica ID view in the database catalog."
#define		MSG_DXR0147E		"DXR0147E: Unable to locate the '%s' item in the summary buffer of a database catalog entry."
#define		MSG_DXR0148E		"DXR0148E: Item '%s' in the summary buffer of a database catalog entry is an unexpected type %i."
#define		MSG_DXR0149E		"DXR0249E: Unable to abbreviate server name '%s'."

//  Data Structures

//
//   INFOBLK Structure  -  UNDOCUMENTED Structure returning information from a GetInfoBlock() call
//
typedef struct {
	DWORD				dwDataDocs;				//  Count of data documents in the database
	TIMEDATE			tdDataMod;				//  Timestamp of last data modification
	TIMEDATE			tdNonDataMod;			//  Timestamp of last non-data modification
	DWORD				dwUnk1;					//  Unknown
	DWORD				dwUnk2;					//  Unknown
	DWORD				dwUnk3;					//  Unknown
	DWORD				dwUnk4;					//  Unknown
	DWORD				dwSizeAlloc;			//  Allocated size of the D/B in Kb
}INFOBLK;

//
//  OSPROG_DEF  Structure  -  Used for running external programs OSLoadProgram()
//
typedef struct {
     char				szAppName[MAXPATH];			//  The name of the program to execute
     char				szWorkingDir[MAXPATH];		//  The working directory to be set for execution
     char				szAppCmdLine[MAXPATH];		//  The command line to be passed to the application
     DWORD				dwFlags;					//  Flags controlling how the application is run
     DWORD				dwReturnCode;				//  The exit code from the application
     DWORD				dwProcessId;				//  The process ID of the application
}OSPROG_DEF;

//
//  Options Arrau  -  Used internally for getting the very extended Db Options
typedef struct {
	DWORD		Options1;							//  DbOptions flags
	DWORD		Options2;							//  DbOptions2 flags
	DWORD		Options3;							//  DbOptions3 flags
	DWORD		Options4;							//  DbOptions4 flags
} OptionsArray;

class ExecEnvironment
{
public:
	//  Default constructor (only used for constructing derived classes)
	ExecEnvironment(void);

	//  Constructor with Run settings
	ExecEnvironment(RunSettings* rsIn, int argc, char * argv[]);

	//  Destructor
	~ExecEnvironment(void);

	//  Public Members
	BOOL IsInitialised;									//  Execution environment is intialised or not
	BOOL RepositoryCreated;								//  Shows that the repository was created on this run
	BOOL RunningOnServer;								//  TRUE if executing on a server
	RunSettings *rsLocal;								//  Current run settings
	int LoggingState;									//  State of logging
	DBHANDLE hdbRepository;								//  Handle of the Repository database
	DBHANDLE hdbLog;									//  The asynch logger file handle
	MQHANDLE hmqProcessor;								//  Handle to the Message Queue
	NOTEHANDLE hnLogDoc;								//  Current log note handle
	NOTEID nidLogDoc;									//  NoteID of the log document
	ElapsedTimer *RunningTime;							//  Default Elapsed Timer
	char CanonicalUser[MAXUSERNAME + 1];				//  Canonical name of current user
	char AbbreviatedUser[MAXUSERNAME + 1];				//  Abbreviated name of current user
	char CommonUser[MAXUSERNAME+ 1];					//  Common name of current user
	char szQName[MAXAPPNAME + 1];						//  MQ Queue Name

	//  Public Functions
	BOOL Close(void);									//  Close the runtime environment

	//  Public Helper Functions

	//  Database and Repository manipulation functions
	DBHANDLE OpenDatabase(char *szServer, char *szDatabase);						//  Open a Domino Database
	DBHANDLE OpenDatabase(char *szServer, char *szDatabase, int iThreadID);		//  Open a Domino Database
	DBHANDLE OpenDatabase(char *szServer, char *szDatabase, USHORT *usMode);     //  Open a Domino Database
	DBHANDLE OpenDatabase(char *szServer, char *szDatabase, USHORT *usMode, int iThreadID);     //  Open a Domino Database
	DBHANDLE OpenDatabase(char *szServer, char *szDatabase, DBID *dbidDB);		//  Open a Domino Database
	DBHANDLE OpenDatabase(char *szServer, char *szDatabase, DBID *dbidDB, int iThreadID);		//  Open a Domino Database
	DBHANDLE OpenDatabase(char *szServer, char *szDatabase, WORD wOpenFlags, DBID * dbidDB, int iThreadID);		//  Open a Domino Database
	DBHANDLE CreateDatabase(char *szServer, char *szDatabase);					//  Create a new Database
	DBHANDLE CreateDatabase(char *szServer, char *szDatabase, int iThreadID);		//  Create a new Database
	DBHANDLE CreateDatabase(char *szServer, char *szDatabase, BOOL bTXLog, int iThreadID);		//  Create a new Database
	DBHANDLE CreateDatabase(char *szServer, char *szDatabase, BOOL bTXLog, DBID *dbidDB, int iThreadID);		//  Create a new Database
	DBHANDLE CreateDatabase(char *szServer, 
							char *szDatabase, 
							BOOL bTXLog, 
							DBID *dbidDB,
							WORD wDbClass,
							BYTE Encrypt,
							int iThreadID);											//  Create a new Database
	DBHANDLE CreateReplica(char *szServer, char *szDatabase, DBHANDLE hdbSrc, DBID *dbidDB);    //  Create a replica database
	DBHANDLE CreateReplica(char *szServer, char *szDatabase, DBHANDLE hdbSrc, DBID *dbidDB, int iThreadID);    //  Create a replica database
	DBHANDLE CreateReplica(char *szServer, char *szDatabase, BOOL bTXLog, DBHANDLE hdbSrc, DBID *dbidDB, int iThreadID);    //  Create a replica database
	DBHANDLE CreateReplica(char *szServer, 
						   char *szDatabase, 
						   BOOL bTXLog, 
						   DBHANDLE hdbSrc, 
						   DBID *dbidDB,
						   WORD wDbClass,
						   BYTE Encrypt,
						   int iThreadID);											//  Create a replica database
	BOOL BuildIndexes(void);														//  Build Repository Indexes
	BOOL BuildIndexes(DBHANDLE hdbIX);												//  Build Indexes in Database
	BOOL BuildIndexes(DBHANDLE hdbIX, int iThreadID);								//  Build Indexes in Database
	void BuildIndex(DBHANDLE hdbIX, NOTEID nidView, int iThreadID);					//  Build the index for the specified view
	void GetDbTitle(DBHANDLE hDB, char *szTitle, int iThreadID);					//  Get a Database Title
	BOOL SetDbTitle(DBHANDLE hDB, char *szTitle);									//  Set a Database Title
	BOOL SetRepositoryTitle(char *szTitle);											//  Set the Repository Title
	STATUS GetIconFlags(DBHANDLE hDB, char *szFlags, int iMaxLen);					//  Get the Database Icon Flags
	STATUS GetIconFlags(DBHANDLE hDB, char *szFlags, int iMaxLen, int iThreadID);	//  Get the Database Icon Flags
	BOOL IsIconFlagSet(DBHANDLE hDB, char cFlag);									//  Test for a particular Database Icon Flag
	BOOL IsIconFlagSet(DBHANDLE hDB, char cFlag, int iThreadID);					//  Test for a particular Database Icon Flag
	DWORD GetDataDocumentCount(DBHANDLE hDB, int iThreadID);						//  Get the Data Document Count
	STATUS DeleteDatabase(char *szServer, char *szDatabase, BOOL bMakeOffline, int iThreadID);  //  Delete a database
	DWORD GetDbAllocatedSize(DBHANDLE hDB, int  iThreadID);                           //  Get Db allocated size in KB
	DWORD GetDbFreeSpace(DBHANDLE hDB, int  iThreadID);								//  Get Db free space in KB
	STATUS GetLastOpenError(void);													//  Get last D/B Open Error
	STATUS virtual GetLastOpenError(int iThreadID);									//  Get last D/B Open Error
	BOOL GetACLRoles(DHANDLE hACL, ACL_PRIVILEGES *pAllPrivs, char *szRoles, int iThreadID);		//  Get the Role descriptors
	void GetReplicaID(DBHANDLE hDB, char *szRepID, int iLen, int iThreadID);		//  Get Decorated Replica ID

	//  Thread Local DBHANDLE Support Functions
	DBHANDLE virtual GetMappedDBH(DBHANDLE hdbNative, int iThreadID);				//  Get a locally mapped Database Handle
	BOOL virtual InvalidateNativeDBH(DBHANDLE hdbNative, int iThreadID);			//  Invalidate a Native Database Handle

	//  Error Handling Functions
	void GetAPIMessage(STATUS stAPICode, char *szMsg);								//  Get an API Error Message

#ifdef WIN32
	void GetWINAPIError(DWORD dwError, char *szMsg);								//  Format & Return a WIN API Error
#endif

#ifdef UNIX
	void GetUNIXAPIError(int iError, char *szMsg);									//  Format & Return a UNIX API Error
#endif
	//  Logging Functions  
	BOOL LogMessage(char *szMsg);													//  Write a log message
	BOOL LogMessage(char *szMsg, int iThreadID);									//  Write a log message
	BOOL LogVerbose(char *szMsg);													//  Write a Verbose log message
	BOOL LogVerbose(char *szMsg, int iThreadID);									//  Write a Verbose log message
	BOOL LogTrace(int iTraceArea, char *szMsg);										//  Write a Trace log message
	BOOL LogTrace(int iTraceArea, char *szMsg, int iThreadID);						//  Write a Trace log message
	BOOL virtual CoreLogMessage(char *szMsg, TIMEDATE *tdEvt, int iThreadID);		//  Write a log message
	BOOL LogEvent(char *szMsg, TIMEDATE *tdEvt, int iThreadID);						//  Write a log mesage as an event
	void virtual LogFlush(void);													//  Force a flush of the log
	void DumpMemory(void *lpMem, int iLen, char *szSymbol);							//  Dump Memory
	void DumpMemory(void *lpMem, int iLen, char *szSymbol, int iThreadID);			//  Dump Memory
	void DumpMemoryExt(void *lpMem, int iLen, char *szSymbol);						//  Dump Memory
	void DumpMemoryExt(void *lpMem, int iLen, char *szSymbol, int iThreadID);		//  Dump Memory
#ifdef WIN32
	void LogWINAPIError(void);														//  Format & Log a WIN API Error
	void LogWINAPIError(int iThreadID);												//  Format & Log a WIN API Error
#endif
	void LogSetVerbose(void);														//  Set Verbose Log Level
	void LogSetNormal(void);														//  Set Normal Log Level
	void LogSetTrace(int iTraceArea);												//  Set Trace Log Level
	void LogSetDebug(int iTraceArea);												//  Set Debug Log Level
	void LogSetEchoOn(void);														//  Set Log Echoing On
	void LogSetEchoOff(void);														//  Set Log Echoing Off
	void LogCommit(void);															//  Write the current Log document
	int LogGetLevel(void);															//  Get the current log level
	BOOL LogIsEchoing(void);														//  Test if the Log is being echoed
	void LogPushSettings(void);														//  Save the current log settings
	void LogPopSettings(void);														//  Restore the current log settings
	void virtual ShowStats(int iOpts, int iThreadID);								//  Show statistics

	//  Miscellaneous Functions
#ifdef WIN32
	int OSLoadProgram(OSPROG_DEF *ospProg, int  iThreadID);							//  Load & execute a program
	int OSLoadSrvCommand(OSPROG_DEF *ospProg, int  iThreadID);						//  Load & execute a server command
#endif
	BOOL GenUniqueID(char *szUniqueID, int iUniqueIDLen, int iThreadID);			//  API version of @Unique()
	void populateUNID(char *szUNID, UNID *uidFormatted);							//  Populate a UNID from a hex string
	BOOL GetNoteUNID(NOTEHANDLE hnSource, UNID *unidTarget, char *szTarget, int iTargetLen, BOOL bExtString, int iThreadID); // Get note's UNID
	void GetUNIDString(UNID unidSource, char *szTarget, int iTargetLen, BOOL bExtString, int iThreadID); // Get UNID as a string
	BOOL virtual SetFastDrainMode();												//  Set Fast draining mode
    void Panic(char *szModule, int iLineNo, char *szReason, void *p1, void *p2, void *p3, int iThreadID);  //  Induce a Panic NSD
	BOOL IsThisADatabase(char *szFileSpec);											//  Determine if the spec is a notes database
	BOOL IsThisAReplicaID(char *szFileName, char *szFRID);							//  Determine if the file name is a Replica ID
	MEMHANDLE getDominoMemory(DWORD dwSize, int iThreadID);							//  Allocate Domino Memory
	MEMHANDLE resizeDominoMemory(MEMHANDLE hMem, DWORD dwSize, int iThreadID);		//  Re-allocate Domino Memory
	BOOL LookupCatalog(char *szServer, char *szRepID, char *szFile, int iThreadID); //  Lookup a replica ID in the catalog

	//  Distributed Control Facility (DCF) Functions
	BOOL AssertGlobalPermit(void);													//  Assert the global permit
	BOOL RevokeGlobalPermit(void);													//  Revoke the global permit
	BOOL TestGlobalPermit(void);													//  Test the global permit 

	//  Primitive Functions
	void TrimToUpper_s(char *szDest, int iDest_Size, char *szSrc);					//  Trim and fold to uppercase
	void RemoveEscapes(char *szMsg, int iStr_Size);									//  Eliminate escape characters
	void GetBuildInfo_s(char *szBuild, int iSize);									//  Get Build Info
	int xtoi(char *szHex);															//  Convert hex string to integer value
	void AppWait(int iWaitMS);														//  Wait for n Milliseconds
	BOOL MatchesPattern(const char *szString, const char *szPattern);				//  Case sensitive wildcard match
	BOOL MatchesPatterni(const char *szString, const char *szPattern);				//  Case insensitive wildcard match

	//  Undocumented API Function Calls
	STATUS SetSpecialNoteID(DBHANDLE hdbTarget, WORD wSpecialID, NOTEID nidReal, int iThreadID);	//  Set a Special NoteID
	STATUS GetFileSummary(DBHANDLE hdbX, WORD wFlags, DHANDLE *hFS, int iThreadID);					//  Get a handle to the D/B summary buffer
	STATUS GetDbInfoBlock(DBHANDLE hDB, INFOBLK *ibInfo, int iThreadID);							//  Get the Database InfoBlock
	STATUS GetDbOptions(DBHANDLE hdbX, DWORD *dwO1, DWORD *dwO2, DWORD *dwO3, DWORD *dwO4);			//  Get Extended Db Options

private:
	//  Private members
	int LogLines;																	//  Lines in the current log document
	int LogLinesPending;															//  # log lines not flushed
	clock_t tLastLogWrite;															//  Elapsed running time of the last log write
	int SavedLogLevel;																//  Saved logging level
	int SavedTraceArea;																//  Saved Trace Area
	TIMEDATE tdStarted;																//  Runtime start timestamp
	APIPackages *pkgLocal;															//  Package Descriptors

	//  Private functions

	//  Logging Functions
	BOOL StartLogger(void);															//  Initialise the log interface
	BOOL CreateLogDoc(void);														//  Create a new log document
	void CoreDumpMemory(void * lpMem, int iLen, char *szSymbol, int iThreadID);		//  Dump Memory
	BOOL CoreLogEvent(char *szMsg, TIMEDATE *tdEvt);								//  Write a log message as an event
	void CoreSetLogLevel(int iLevel, int iTraceArea);								//  Set The logging Level
	void CoreSetLogEcho(BOOL bEchoState);											//  Set Log Echo State
	void LogTimestamps(TIMEDATE *tdDMod, TIMEDATE *tdNDMod, TIMEDATE *tdLFU, int iThreadID);  //  Log timestamps

	//  Error Handling Functions
#ifdef WIN32
	void CoreLogWINAPIError(int iThreadID);											//  Format & Log a WIN API Error
#endif
	void DrainMQ(char *szQName);													//  Drain the Message Queue

	//  Database and Repository Functions
	DBHANDLE CreateRepository(char *szServer, char *szDatabase, BOOL bSetACL, int iThreadID);		//  Create the repository database
	DBHANDLE CreateRepository(char *szServer, char *szDatabase, WORD wDBCLass, BOOL bSetACL, int iThreadID);		//  Create the repository database
	DBHANDLE CreateRepository(char *szServer, char *szDatabase, WORD wDBCLass, BOOL bTXLog, BOOL bSetACL, int iThreadID);		//  Create the repository database
	DBHANDLE CreateRepository(char *szServer, 
							  char *szDatabase, 
							  WORD wDBCLass, 
							  BOOL bTXLog, 
							  BOOL bSetACL,
							  BYTE Encrypt,
							  int iThreadID);								//  Create the repository database
	BOOL SetDefaultACL(DBHANDLE hdbNew, char *szServer, int iThreadID);		//  Set Default ACL in new database
	DBHANDLE CoreOpenDatabase(char *szServer, char *szDatabase, WORD wOpenFlags, int iThreadID);		//  Open a Domino Database
	DBHANDLE CoreOpenDatabase(char *szServer, char *szDatabase, WORD wOpenFlags, USHORT * usMode, int iThreadID);     //  Open a Domino Database
	DBHANDLE CoreOpenDatabase(char *szServer, char *szDatabase, WORD wOpenFlags, DBID * dbidDB, int iThreadID);		//  Open a Domino Database
	DBHANDLE CoreCreateReplica(char *szServer, 
							   char *szDatabase, 
							   BOOL bTXLog, 
							   DBHANDLE hdbSrc, 
							   DBID * dbidDB,
							   WORD wDbClass,
							   BYTE Encrypt,
							   int iThreadID);											//  Create a replica database
	BOOL CoreBuildIndexes(DBHANDLE hdbIX, int iThreadID);								//  Build Indexes in Database
	DBHANDLE ReopenWithFixup(DBHANDLE hdbX, char *szFullDbName, int iThreadID);			//  Re-open a D/B with a fixup
	BOOL GetLastFixupTime(DBHANDLE hdbX, TIMEDATE *tdLastFixup, int iThreadID);			//  Get the D/B last fixup time

	//  Primitive Functions
	BOOL GetCurrentUser(void);							//	Get the current user name

protected:
	//  Protected Members
	BOOL bNRTInitialised;								//  Notes Runtime is initialised
	DHANDLE hDefDescriptor;								//  Handle to the Addin status line
	BOOL DCFGlobalPermit;								//  DCF - Global Permit Flag
	STATUS stLastOpenError;								//  Last error from D/B Open

	//  Protected functions

	//  Initialisation/Termination Functions
	BOOL CoreClose(void);								//  Shutdown the runtime
	BOOL InitEnvironment(int argc, char *argv[]);		//  Initialise the runtime
	void CloseLogDoc(void);								//  Close the current log document
	void virtual SetLastOpenError(STATUS stOpen, int iThreadID);			//  Set last D/B open Error code
};
