#pragma once
//  DXDbScanner
//
//  This class is the eXplorer class that handles the scanning of the database level in the hierarchy according to
//  the scope and constraints from the search request.
//
//  Message Identifier:      DXR
//  Message Range:           1401 - 1499
//  Trace area assignment:   TRACE_EXPLORER (05)
//  DX Version:				 3.11.1
//  Version:                 3.11.1  Build: 99
//
//  Version History:
//
//  1.0.0		14/09/2004		Initial version
//  3.5.3		21/08/2009		Re-based ready for 4.0.0
//  3.10.0		01/10/2010		Wildcard & multi-thread execution added to DX processing
//								DXNoteScanner incorporated in kernel
//  3.10.5		18/02/2011		Report on D/Bs which cannot be opened
//  3.11.1		22/10/2011		VS 2011 Updates
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2004 - 2011
//

//  Application Includes
#include "../MTX/MTExecutive.h"					//  Runtime environment
#include "../DXException.h"
#include "DXRequest.h"
#include "DXReporter.h"

//  Trace Messages
#define		MSG_DXR1400T		"DXR1400T: A new DXDbScanner instance has been created @%p."
#define		MSG_DXR1412T		"DXR1412T: A DXDbScanner instance @%p has been destroyed."
#define		MSG_DXR1417T		"DXR1417T: Starting request @%p into scanner @%p in thread: %i."
#define		MSG_DXR1418T		"DXR1418T: Finished (RQRC: %i) for request @%p from scanner @%p in thread: %i."

//  Information Messages
#define		MSG_DXR1404I		"DXR1404I: Starting scan of database '%s' in '%s'."
#define		MSG_DXR1405I		"DXR1405I: Completed scan of database '%s' in '%s'."
#define		MSG_DXR1406I		"DXR1406I: Design scanning has completed for database '%s'."
#define		MSG_DXR1408I		"DXR1408I: Data note scanning has completed for database '%s'."

//  Error Messages
#define		MSG_DXR1401E		"DXR1401E: An unspecified exception has been trapped."
#define		MSG_DXR1402E		"DXR1402E: Unable to open database '%s' in '%s' to explode it."
#define		MSG_DXR1403E		"DXR1403E: Database scan was requested for '%s', which is a directory."
#define		MSG_DXR1407E		"DXR1407E: Design scanning failed for database '%s'."
#define		MSG_DXR1409E		"DXR1409E: Data note scanning failed for database '%s'."
#define		MSG_DXR1410E		"DXR1410E: Unable to get database replica information for '%s'."
#define		MSG_DXR1411E		"DXR1411E: Unable to get database information for '%s'."
#define		MSG_DXR1413E		"DXR1413E: Unable to get the current user name."
#define		MSG_DXR1414E		"DXR1414E: Unable to abbreviate the current user name."
#define		MSG_DXR1419E		"DXR1419E: Panic induced with Runtime: %p and Request: %p in Thread: %i."
#define		MSG_DXR1420E		"DXR1420E: The run permit has been lost, all search requests will be aborted."

class DXDbScanner :
	public Runnable
{
public:
	//  Alternate Constructors
	DXDbScanner(MTExecutive *xeParent, int iThreadID);

	//  Destructor
	~DXDbScanner(void);

	//  Public Members
	int iParentThread;												//  ID of the owning Thread

	//  Public Functions
	BOOL ProcessRequest(DXRequest *dxrRQ, int iThreadID);			//  Process a dabase scan request
	void virtual ExecuteThisRequest(void *pReqObject, int iThreadID);	//  Multi thread request entry point

private:
	//  Private Members
	MTExecutive *xeLocal;											//  Inherited execution environment

	//  Private Functions
	void DestroySubContexts(void);									//  Destroy any lower level scanners
	BOOL GetDbInfo(DBHANDLE hdbTarget, DXRequest *dxrRQ, int iThreadID);		//  Get Database Information
	BOOL CoreProcessRequest(DXRequest *dxrRQ, int iThreadID);					//  Internal Request processor
};
