#pragma once
//  DXNoteScanReq
//
//  This class contains the transaction data for a DXNoteScanner execution
//
//  Message Identifier:      DXR
//  Message Range:           1801 - 1899
//  Trace area assignment:   TRACE_DBSCAN (28)
//  DX Version:				 3.10.0
//  Version:                 3.10.0  Build: 80
//
//  Version History:
//
//  3.10.0		01/10/2010		Wildcard & multi-thread execution added to DX processing
//								DXNoteScanner incorporated in kernel
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2001-2010
//

//  Application Includes
#include	"../MTX/MTExecutive.h"					//  Run time

//  Local Constants
#define		MAX_SEARCH_STRING				256
#define		MAX_SEARCH_ENTRIES				10
#define		DEFAULT_SUBRQ_PRIORITY			10

//  Scan flags
#define		FLAG_NOTE_SCAN_OBJECT_FILE		0x0001
#define		FLAG_NOTE_SCAN_OBJECT_OTHER		0x0002
#define		FLAG_NOTE_SCAN_SIGNED			0x0004
#define		FLAG_NOTE_SCAN_ENCRYPTED		0x0008
#define		FLAG_NOTE_SCAN_DELSTUBS			0x0010
#define		FLAG_NOTE_SCAN_REPCONS			0x0020
#define		FLAG_NOTE_SCAN_NOACCESS			0x0040
#define		FLAG_NOTE_SCAN_NOT_DELSTUB		0x0080

//  Trace Messages

//  Information Messages

//  Warning Messages

//  Error Messages


//  Data Structures
typedef struct {
	DHANDLE hitMatches;										//  Handle of IDTable containing matches for this search
	WORD	wNoteClass;										//  Note classes to scan for
	DWORD	dwSearchFlags;									//  Search Flags... see FLAG_NOTE_SCAN_xxx
	char	szSearchString[MAX_SEARCH_STRING];				//  Search string to be used for this search
	DWORD	dwCount;										//  Number of noteids added to IDTable
} DXNoteScanEntry;

typedef struct {
	DXNoteScanEntry		dxnsEntry[MAX_SEARCH_ENTRIES];		//  Array of DXNoteScanEntry objects
} DXNoteScanTable;

typedef struct {
	char			szSearchString[MAX_SEARCH_STRING];				//  Search string to be used for this search
	FORMULAHANDLE	hfSearch;										//  Handle allocated to formula via NSFFormulaCompile
	char			*phfSearch;										//  Pointer used to lock handle when in use
	HCOMPUTE		hcSearch;										//  HCOMPUTE allocated by NSFComputeStart when being used
} DXNoteScanIntEntry;

typedef struct {
	int			iNIDArrayIndex;
	NOTEID		nidRetNID;
	NOTEHANDLE	hnToScan;
	WORD		wNoteclass;
	DWORD		dwNoteFlags;
} DXNoteScanNoteProps;

//  Class
class DXNoteScanReq
{
public:
	//  Constructor
	DXNoteScanReq(DXNoteScanTable *nstToUse, int iThreadID);

	//  Destructor
	~DXNoteScanReq(void);

	//  Public Members
	int				OwningThread;						//  Creating Thread ID
	BOOL			bUseMTScanner;						//  Indicate whether multi-threaded scanner should be used
	DWORD			dwMaxErrors;						//  Max number of note scan errors before exiting with error
	char			szServer[MAXUSERNAME];				//  Server that hosts database to be scanned
	char			szDbPath[MAXPATH];					//  Path of database to be scanned
	DBHANDLE		hdbToScan;							//  Handle of database to be scanned
	int				BaseRQPriority;						//  Priority to use for sub-requests
	int				iReturnCode;						//  Return code from DXNoteScanner
	DWORD			dwNotesToScan;						//  Number of notes found by base search
	DWORD			dwNotesScanned;						//  Number of notes scanned during processing
	DWORD			dwErrors;							//  Number of errors during processing
	DWORD			dwElapsed;							//  Time in seconds it took to perform the scan
	DXNoteScanTable	*nsObject;							//  Pointer to DXNoteScanTable data structure

	//  The following members are used to store transient (per request) needed for execution of the scanner

	DHANDLE hitNotesToScan;								//  IDTable of notes to scan
	void	*rqScanner;									//  Address of the scanner for this equest
	DWORD	dwNotesInserted;							//  Number of notes inserted into IDTable
	BOOL	bDbLocallyOpened;							//  Whether DXNoteScanner opened the Db or not
	DWORD	dwGlobalScanFlags;							//  Global value covering all search entry flags
	WORD	wGlobalNoteClasses;							//  Global notes document classes to scan
	DXNoteScanIntEntry	nsieTable[MAX_SEARCH_ENTRIES];	//  Array of DXNoteScanIntEntry

	//  Public Functions

private:
	//  Private Members

	//  Private Functions

protected:
	//  Protected Members

	//  Protected Functions

};
