#pragma once
//  DXReporter
//
//  This is the base class that is extended to build any Domino eXplorer functionality that operates at
//  the database level. The class defines the interface that used to invoke the functional code as well
//  as providing a number of helper functions.
//
//  Message Identifier:      DXR
//  Message Range:           1501 - 1599
//  Trace area assignment:   TRACE_EXPLORER (05)
//  DX Version:				 3.10.5
//  Version:                 3.10.5  Build: 86
//
//  Version History:
//
//  1.0.0		14/09/2004		Initial version
//  3.5.3		21/08/2209		Re-based ready for 4.0.0
//  3.10.0		01/10/2010		Wildcard & multi-thread execution added to DX processing
//								DXNoteScanner incorporated in kernel
//  3.10.5		18/02/2011		Report on D/Bs which cannot be opened
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2004-2011
//

//  Application Includes
#include "../ExecEnvironment.h"
#include "../DXException.h"
#include "DXRequest.h"

//  Local Definitions

#define		FIELD_SERVER_NAME	"ServerName"
#define		FIELD_DB_NAME		"DatabaseName"
#define		FIELD_DB_REPID		"ReplicaID"
#define		FIELD_TEMPLATE		"TemplateName"
#define		FIELD_FROM_TEMPLATE "FromTemplateName"
#define		FIELD_REPORTER		"Reporter"
#define		FIELD_NOTEID		"NoteID"
#define		FIELD_UNID			"UniqueID"
#define		FIELD_MOD_TIME		"LastModified"
#define		FIELD_ALIAS			"AliasNames"
#define		FIELD_ELEMENT		"ElementName"
#define		FIELD_LASTMOD		"LastModified"
#define		FIELD_SIGNEDBY		"SignedBy"
#define		FIELD_SIGNEDAT		"SignedAt"
#define		FIELD_SIGNEDVER		"SignatureVerifier"
#define		FIELD_DESIGNFLAGS	"DesignFlags"
#define		FIELD_DESIGNINHIBIT "DesignInheritanceInhibited"
#define		FIELD_COMMENT		"Comment"
#define		FIELD_DESVER		"DesignerVersion"
#define		UNK_DESIGN_ITEM		"-Unnamed Design Element-"
#define		SIGNER_NOSIG		"-No Signature-"
#define		SIGNER_NOVERIFY		"Signature is unverifiable"

//  Error Messages

#define		MSG_DXR1501E		"DXR1051E: Unable to create a new document in the repository."
#define		MSG_DXR1502E		"DXR1502E: Unable to write an item to a document in the repository."
#define		MSG_DXR1503E		"DXR1503E: Unable to abbreviate a name."

class DXReporter
{
public:
	//  Default Constructor
	DXReporter(void);

	//  Constructor with Execution Environment
	DXReporter(ExecEnvironment *xeParent, int iThreadID);

	//  Virtual destructor - this must be implemented in all inheriting classes
	virtual ~DXReporter(void);

	//  Public virtual functions - implemented in the inheriting classes
	virtual BOOL ReportOnThisDb(DXRequest *dxrRQ, DBHANDLE hdbEntity, int iThreadID);		//  Report on the selected Db
	virtual BOOL ReportNoAccess(DXRequest *drRQ, int iThreadID);			//  Report on an entity whic cannot be opened
	virtual void GetDescription(char *szDescription);						//  Get the reporter description

	//  Public Members

	//  Public Functions

private:
	//  Private Functions

protected:
	//  Protected Members
	ExecEnvironment *xeLocal;							//  Execution Environment
};
