#pragma once
//  DXSStash
//
//  This class stores and delivers Server scan requests for the DX Spider
//
//  Message Identifier:      DXR
//  Message Range:           1101 - 1199
//  Trace area assignment:   TRACE_EXPLORER (05)
//  DX Version:				 3.11.1
//  Version:                 3.11.1  Build: 99
//
//  Version History:
//
//  1.0.0		14/09/2004		Initial version
//  3.5.3		21/08/2209		Re-based ready for 4.0.0
//  3.9.2		22/09/2010		Avoided a recursion in DX processing
//  3.10.0		01/10/2010		Wildcard & multi-thread execution added to DX processing
//								DXNoteScanner incorporated in kernel
//  3.11.1		22/10/2011		VS 2011 Updates
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2004 - 2011
//

//  Application Includes
#include "../ExecEnvironment.h"
#include "../DXException.h"
#include "DXRequest.h"

//  Local Definitions

//  Trace Messages
#define		MSG_DXR1190T		"DXR1190T: A new DXSStash instance has been created @%p."
#define		MSG_DXR1191T		"DXR1191T: A DXSStash instance @%p has been destroyed."

class DXSStash
{
public:
	//  Default constructor
	DXSStash(void);

	//  Alternate constructor
	DXSStash(ExecEnvironment *xeParent, int iThreadID);

	//  Destructor
	~DXSStash(void);

	//  Public Functions
	BOOL HasNext(void);									//  Iterator - see if there is another request available
	DXRequest * Next(void);								//  Iterator - return the next request
	void Add(DXRequest *dxrRQ);							//  Add another request to the stash
	void Empty(void);									//  Empty the existing stash content

private:
	//  Private members
	ExecEnvironment *xeBase;							//  Inherited execution environment
	int iParentThread;									//  ID of the owning thread
	int iStashed;										//  Count of request objects stashed
	DXRequest *dxrStash[MAX_STASH];						//  Stash store
	BOOL IsInStash(DXRequest *dxrRQ);					//  Test if entry is already in the stash
};
