#pragma once
//  DominoExplorer
//
//  This class provides the root scanning capabilities for Domino eXplorer (DX) applications.
//  This is the multithreaded version of the explorer
//
//  Message Identifier:      DXR
//  Message Range:           1001 - 1099
//  Trace area assignment:   TRACE_EXPLORER (05)
//  DX Version:				 3.11.5
//  Version:                 3.11.5  Build: 103
//
//  Version History:
//
//  1.0.0		14/09/2004		Initial version
//  3.5.3		21/08/2209		Re-based ready for 4.0.0
//  3.10.0		01/10/2010		Wildcard & multi-thread execution added to DX processing
//								DXNoteScanner incorporated in kernel
//  3.11.1		22/10/2011		VS 2011 Updates
//  3.11.5		30/11/2011		Recursion avoidance improvements
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2004 - 2011
//

//  Platform includes
#include "../Platform/PlatBase.h"							//  OS and Language includes

//  Application Includes
#include "../MTX/MTExecutive.h"								//  Runtime environment
#include "../DXException.h"									//  DX Exceptions
#include "DXRequest.h"										//  Explorer Request
#include "DXSpider.h"										//  DX Spider module

//  Local Definitions
#define		DX_BUILD			103
#define		DX_IDENTITY			"Domino (MT) Explorer 3.11.5 (build: %i)"

//  Trace Messages
#define		MSG_DXR1001T		"DXR1001T: The Explorer has accepted request '0x%p' with scope: %i and wildcards: %i."
#define		MSG_DXR1017T		"DXR1017T: A new DominoExplorer instance has been created @%p."
#define		MSG_DXR1020T		"DXR1020T: Scanners: 0x%X Std Reporters: 0x%.4X Filters: 0x%.4X."
#define		MSG_DXR1022T		"DXR1022T: A DominoExplorer instance @%p has been destroyed."
#define		MSG_DXR1030T		"DXR1030T: %s '%s' %s '%s'."

//  Information Messages
#define		MSG_DXR1013I		"DXR1013I: The current request was processed successfully."
#define		MSG_DXR1023I		"DXR1023I: %i Servers were processed and %i were dropped for this request."
#define		MSG_DXR1024I		"DXR1024I: %i Directories were processed and %i were dropped for this request."
#define		MSG_DXR1025I		"DXR1025I: %i Databases were processed and %i were dropped for this request."

//  Warning Messages
#define		MSG_DXR1014W		"DXR1014W: The current request was processed with warnings."

//  Error Messages
#define		MSG_DXR1010S		"DXR1010S: Generic Error: %s."
#define		MSG_DXR1011S		"DXR1011S: Explorer exception type: %i, code: %i has been detected in thread: %i."
#define		MSG_DXR1012E		"DXR1012E: The current request could not be processed."
#define		MSG_DXR1015E		"DXR1015E: The current request was processed with errors."
#define		MSG_DXR1016E		"DXR1015E: The current request could not be completed due to errors."
#define		MSG_DXR1018S		"DXR1018S: The exception was percolated from a lower level scanner."
#define		MSG_DXR1019S		"DXR1019S: The execption originated in: %s at: %d."
#define		MSG_DXR1021S		"DXR1021S: The exception occurred at line: %i in module: %s."
#define		MSG_DXR1031E		"DXR1031E: The current explorer request has not been parsed, request rejected."
#define		MSG_DXR1032E		"DXR1032E: The current explorer request has an invalid scope (%i), request rejected."

class DominoExplorer :
	public Runnable
{
public:
	//  Constructors
	DominoExplorer(MTExecutive *xeParent, int iInstance, int iThreadID);

	//  Destructor
	~DominoExplorer(void);

	//  Public Members
	int  iParentThread;											//  Parent Thread
	int  InstanceID;											//  Instance ID of this explorer
	BOOL IsStarted;												//  Indicates if the Explorer has started successfully
	BOOL IsAvailable;											//  Indicates if the Explorer is ready to accept requests

	//  Public Functions
	void virtual ExecuteThisRequest(void *pReqObject, int iThreadID);	//  Multi thread request entry point
	BOOL ProcessRequest(DXRequest *dxrRQ, int iThreadID);		//  Process a request
	void GetBuildInfo_s(char *szBuild, int iSize);				//  Get Build Info

private:
	//  Private Members
	MTExecutive *xeLocal;										//  Parent execution environments
	DXSpider *dxsRoot;											//  The root scanner

	//  Private Functions
	BOOL CoreProcessRequest(DXRequest *dxrRQ, int iThreadID);	//  Process a request	
};
