#pragma once
//  TransactionQueue
//
//  This class provides the information that describes a Transaction Queue. It is used by the
//  standard transaction handler to access, control and monitor a particular queue.
//  the standard command set.
//
//  Message Identifier:      N/A
//  Message Range:           0000 - 0000
//  Trace area assignment:   None
//  DX Version:				 3.11.1
//  Version:                 3.11.1  Build: 99
//
//  Version History:
//
//  3.10.1		13/10/2010		Initial Version
//  3.10.8		30/05/2011		Transaction scheduling added to transaction handler
//  3.11.1		22/10/2011		VS 2011 Updates
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2010 - 2011
//

//  Application Includes
#include "../ExecEnvironment.h"

//  Local Definitions
//  Queue Protocol Flags
#define			QPFLAG_REQUEIP_STARTUP		0x00000001			//  Requeue In Progress transactions on startup
#define			QPFLAG_REQUEIP_READY		0x00000002			//  Return In Progress transactions to the ready queue
#define			QPFLAG_REQUE_DELAY			0x00000004			//  Requeue Delayed transaction
#define			QPFLAG_DELAY_IDLE			0x00000008			//  Requeue Delayed transactions when ready queue is empty
#define			QPFLAG_DELAY_CYCLE			0x00000010			//  Requeue Delayed transactions for every n ready transactions
#define			QPFLAG_MONITOR_SCHED		0x00000020			//  Monitor the schedule queue
#define			QPFLAG_DEFAULT				(QPFLAG_REQUEIP_STARTUP | QPFLAG_REQUE_DELAY | QPFLAG_DELAY_IDLE)

//  Miscellaneous settings
#define			DEFAULT_DELAY				15 * 60;			//  15 mins
#define			DEFAULT_NEW_TRANS			"NewTransactions"
#define			DEFAULT_DELAYED_TRANS		"DelayedTransactions"
#define			DEFAULT_IP_TRANS			"InProgressTransactions"
#define			DEFAULT_SCHED_TRANS			"ScheduledTransactions"

class TransactionQueue
{
public:
	//  Default Constructor
	TransactionQueue(void);

	//  Destructor
	~TransactionQueue(void);

	//  Public Members
	//  Queue Setup & Descriptors
	WORD				wQueueProtocols;					//  Queue Protocols
	int					MaxConcurrent;						//  Max Concurrent transactions
	DWORD				MaxRunLimit;						//  Max number of transactions (run limit)
	DBHANDLE			hdbQueue;							//  Native Handle to the Queue D/B
	int					MinDelay;							//  Minimum time on the delay queue (seconds)
	int					ReQTXCycle;							//  Number of ready transactions before requeue
	int					MaxReqCount;						//  Max transactions to requeue at once
	int					DelayCycleSecs;						//  Delay Cycle Time in Seconds
	volatile BOOL		LocalPermit;						//  Permit to run this queue
	BOOL				QueueIsSuspended;					//  Queue is (temp) suspended
	char				szQName[MAX_ELEMENT + 1];			//  Logical Queue Name	
	char				szQServer[MAX_SERVER + 1];			//  Queue Database Server Name
	char				szQDbPath[MAX_DATABASE + 1];		//  Queue Database Path
	char				szReadyQName[MAX_ELEMENT + 1];		//  View name of the 'Ready' Queue
	char				szInProgressQName[MAX_ELEMENT + 1];		//  View name of the 'In Progress' Queue
	char				szDelayedQName[MAX_ELEMENT + 1];	//  View name of the 'Delayed' Queue
	char				szSchedQName[MAX_ELEMENT + 1];		//  View name of the 'Scheduled" Queue

	//  Operational values - set and used by the TransactionHandler
	int					ReturnCode;							//  Return Code from executing this Queue

	//  Statistics Section
	DWORD				dwStarted;							//  Transactions started from this queue
	DWORD				dwCompleted;						//  Transactions completed from this queue
	DWORD				dwSched;							//  Transactions scheduled

	//  Public Functions

private:
	//  Private Members

	//  Private Functions

protected:
	//  Protected Members

	//  Protected Functions
};
