#pragma once
//  AutoScaleFrequencyAnalyser Class
//
//  This class provides a frequency analyser that automatically scales to maintain the maximum granularity of the data
//
//  Message Identifier:      N/A
//  Message Range:           N/A
//  Trace area assignment:   N/A
//  Version:                 3.11.5  Build: 103
//
//  Version History:
//
//  3.11.1		22/10/2011		Incorporated into kernel
//  3.11.5		30/11/2011		Cache coherence improvements
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2011
//

//  Platform Includes
#include	"../Platform/PlatBase.h"									//  Basic platform includes
#ifndef WIN32
#include	"../DXCommon/Platform/WCompat.h"								//  Windows compatability
#endif

//  Local Definitions
#define		ASFA_NUM_BUCKETS				100							//  Default is 100 buckets
#define		ASFA_NUM_OUTLIERS				10							//  Number of outliers allowed
#define		ASFA_RENORM_BUCKETS				5							//  Number of buckets for re-normalisation

//  Class Definition
class AutoScaleFrequencyAnalyser
{
public:
	//  Default Constructor
	AutoScaleFrequencyAnalyser(void);

	//  Normal Constructor
	AutoScaleFrequencyAnalyser(int Buckets, int Outliers, int RenormBkts);

	//  Destructor
	~AutoScaleFrequencyAnalyser(void);

	//  Public Members

	//  Public Functions
	void RecordDataPoint(DWORD dwDP);									//  Record a new data point
	void DisallowOutliers(void);										//  Prevent recording outliers
	DWORD GetFrequency(int Ordinate, DWORD *lo, DWORD *hi);				//  Get the frequency of a particular bucket
	int GetLowOutlierCount(void);										//  Get the count of low outliers
	int GetHighOutlierCount(void);										//  Get the count of high outliers
	DWORD GetLowOutlier(int Ordinate);									//  Get a low outlier value
	DWORD GetHighOutlier(int Ordinate);									//  Get a high outlier value

private:
	//  Private Members
	int				ASFANumBuckets;										//  Number of Buckets
	int				ASFANumOutliers;									//  Number of Outliers
	int				ASFARenormBuckets;									//  Number of Buckets permitted for re-normalisation
	UCHAR			*pPool;												//  Pointer to the storage pool
	DWORD			*Bucket;											//  Array of buckets
	DWORD			*TempBucket;										//  Temporary Array of buckets for reorganising
	DWORD			BucketSize;											//  Current bucket size
	DWORD			B0LowValue;											//  Low value of bucket 0
	DWORD			BnHighValue;										//  High value of last bucket
	DWORD			DataInBuckets;										//  Count of data points in the buckets
	BOOL			OutliersPermitted;									//  Outliers are permitted
	BOOL			ReNormalised;										//  Array has been re-normalised
	int				LXCount;											//  Low outlier count
	DWORD			*LowOutliers;										//  Array of low outliers
	int				HXCount;											//  High outlier count
	DWORD			*HighOutliers;										//  Array of high outliers

	//  Private Functions
	void InitPool(void);												//  Initialise the storage pool
	BOOL FilterOutliers(DWORD dwDP);									//  Filter out oultlier data values
	void AdjustBounds(DWORD dwDP);										//  Adjust the bounds of the recording array
	DWORD SwapForHighestLowOutlier(DWORD dwDP);							//  Swap the current value for the highest low outlier
	DWORD SwapForLowestHighOutlier(DWORD dwDP);							//  Swap the current value for the lowest high outlier
	void Increment(DWORD dwDP);											//  Increment the bucket counter for the specified data point
	void GetSuggestedBounds(DWORD lo, DWORD hi, DWORD *B0, DWORD *Bn, DWORD *bSize);       //  Get the suggested bounds
	DWORD GetBucketValue(DWORD lo, DWORD bSize);						//  Compute the value of a bucket
	DWORD GetNextBucketSize(DWORD dwOldSize);							//  Get the next bucket size
	void GetSampleBounds(DWORD *lo, DWORD *hi);							//  Get the sample bounds
};
