#pragma once
//  TimeBasedArray
//
//  This class provides the Time Based Array
//
//  Message Identifier:      N/A
//  Message Range:           None
//  Trace area assignment:   None
//  DX Version:				 3.14.0
//  Version:                 3.14.0  build: 108
// 
//  Version History:
//
//	1.0.0 -		18/11/2011		-  Initial Build
//  3.11.4		27/11/2011		-  Incorporated into kernel
//	3.14.0		21/01/2015		-  x64 compatability fixes
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2011 - 2015
//

//  Platform Includes
#include "../Platform/PlatBase.h"
#include "../Platform/NotesBase.h"

#ifdef _DEBUG
#include	"../Debug/Helper.h"			//  Debugging Helper
#endif

//  Local Definitions

//  The following entries may be pre-defined to determine the bounds of the time arrays
#ifndef		TBA_HOURS_RECORDED
#define		TBA_HOURS_RECORDED				24 * 7						//  7 days of hourly
#endif
#ifndef		TBA_DAYS_RECORDED
#define		TBA_DAYS_RECORDED				64							//  64 days of recording
#endif
#ifndef		TBA_WEEKS_RECORDED
#define		TBA_WEEKS_RECORDED				24							//  24 weeks of recording
#endif
#ifndef		TBA_MONTHS_RECORDED
#define		TBA_MONTHS_RECORDED				24							//  24 months of recording
#endif
#ifndef		TBA_YEARS_RECORDED
#define		TBA_YEARS_RECORDED				10							//  10 years of recording
#endif
//  End of the bound definitions

#define		TBA_HOURLY_INDEX				0							//  Hourly Index entry
#define		TBA_DAILY_INDEX					1							//  Daily Index entry
#define		TBA_WEEKLY_INDEX				2							//  Weekly Index entry
#define		TBA_MONTHLY_INDEX				3							//  Monthly Index entry
#define		TBA_YEARLY_INDEX				4							//  Yearly Index entry
#define		TBA_TOTAL_INDEX					5							//  Total Index entry
#define		TBA_MAX_BASENAME				30							//  Max size of a base name
#define		TBA_MAX_ITEMNAME				40							//  Max size of an item name
#define		TBA_DEF_BASENAME				"Counter"					//  Default Item Name
#define		TBA_SUFF_HOURLY					"Hour"						//  Suffix for the hourly array
#define		TBA_SUFF_DAILY					"Day"						//  Suffix for the daily array
#define		TBA_SUFF_WEEKLY					"Week"						//  Suffix for the weekly array
#define		TBA_SUFF_MONTHLY				"Month"						//  Suffix for the monthly array
#define		TBA_SUFF_YEARLY					"Year"						//  Suffix for the yearly array
#define		TBA_SUFF_TOTAL					"Total"						//  Suffix for the total array

//  Class Definition
class TimeBasedArray
{
public:
	//  Default Constructor
	TimeBasedArray();

	//  Normal Constructor
	TimeBasedArray(int iVars, BOOL bRecordHourly);

	//  Destructor
	~TimeBasedArray();

	//  Public Functions
	BOOL isArrayInitialised(void);										//  Test if the Array is initialised
	BOOL marshall(char *szLatestTDFieldName, NOTEHANDLE hnSNote);		//  Load the arrays from a note
	BOOL serialize(char *szLatestTDFieldName, NOTEHANDLE hnSNote);		//  Store the arrays in a note
	void increment(int iVarIndex, TIMEDATE *tdEvent);					//  Increment a counter
	void decrement(int iVarIndex, TIMEDATE *tdEvent);					//  Decrement a counter
	void add(int iVarIndex, TIMEDATE *tdEvent, DWORD dwValue);			//  Add a value to a counter
	void subtract(int iVarIndex, TIMEDATE *tdEvent, DWORD dwValue);		//  Subtract a value from a counter
	void virtual getFieldName(int iVarNum, char *Bfr);					//  Get a pointer to a field name
	int getArraySize(int iTimeIndex);									//  Get the size of a time slice array
	DWORD getCounterValue(int iVarIndex, int iTimeIndex, int iElement); //  Get the value of an individual counter

private:
	//  Private members
	DWORD			dwOffHourly;										//  Offset to the Hourly array
	DWORD			dwOffDaily;											//  Offset to the Daily array
	DWORD			dwOffWeekly;										//  Offset to the Weekly array
	DWORD			dwOffMonthly;										//  Offset to the Monthly array
	DWORD			dwOffYearly;										//  Offset to the Yearly array
	DWORD			dwOffTotal;											//  Offset to the Total array
	DWORD			dwVarSize;											//  Size of a single variable array collection

	//  Private functions
	void adjustAnchor(TIMEDATE *tdNewAnchor);							//  Adjust the anchor timestamp
	void getIndexPoints(TIMEDATE *tdEvent, int *piIPA);					//  Get the index point array for the supplied timestamp
	void shuffleArray(DWORD *dwArray, int iSize, int iShift);			//  Shuffle an array
	void loadArray(DWORD *dwArray, int iSize, char *szItem, NOTEHANDLE hnSNote);			//  Load an individual array from an item
	void storeArray(DWORD *dwArray, int iSize, char *szItem, NOTEHANDLE hnSNote);			//  Store an individual array in an item

protected:
	//  Protected members
	int				VarCount;											//  Count of variables in the Array
	BOOL			Hourly;												//  Hourly recording is active
	UCHAR			*pAStorage;											//  Pointer to Array Storage
	TIMEDATE		tdAnchor;											//  Anchor event timedate
	char			szFieldName[TBA_MAX_ITEMNAME];						//  Temporary buffer for item names

	//  Protected functions
	void initArray(void);												//  Initialise the array storage
};

