//  NECache
//
//  This class extends the DX ObjectCache class to provide a cache for Named Entities.
//
//  Message Identifier:      DXR
//  Message Range:           8501 - 8599
//  Trace area assignment:   TRACE_NER (30)
//  DX Version:				 3.11.1
//  Version:                 3.11.1  Build: 99
//
//  Version History:
//
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2011
//

//  Application Includes
#include "NECache.h"

//  Constructor
NECache::NECache(ExecEnvironment *xeParent, int iMaxEntries, int iStrategy, int iThreadID)
{
	char			szMsg[MAX_MSG + 1];							//  Generic message buffer

	xeLocal = xeParent;
	OwningThread = iThreadID;

	//  Trace the construction
	sprintf_s(szMsg, MAX_MSG, MSG_DXR8501T, this);
	xeLocal->LogTrace(TRACE_NER, szMsg, iThreadID);

	//  Initialise the cache
	initCache(iMaxEntries, iStrategy, iThreadID);

	//  Trace initialisation
	sprintf_s(szMsg, MAX_MSG, MSG_DXR8502I, this, Cache, CacheSize, CacheStrategy);
	xeLocal->LogTrace(TRACE_NER, szMsg, iThreadID);
}

//  Destructor
NECache::~NECache()
{
	char			szMsg[MAX_MSG + 1];							//  Generic message buffer

	//  Trace the destruction
	sprintf_s(szMsg, MAX_MSG, MSG_DXR8503T, this);
	xeLocal->LogTrace(TRACE_NER, szMsg, OwningThread);
}

//  selectObject
//
//  This function will determine if the passed object matches the passed key string.
//
//  Parameters
//
//  1  -  void *			-  Pointer to the object to be selected or not
//  2  -  char *			-  Pointer to a key string to be mateched against objects 
//  3  -  int				-  ID of the thread requesting the object
//
//  Returns
//
//	BOOL	-	TRUE is the passed key matches the key of the object and FALSE if it does not.
//
//  API Use Notes:
//
//
BOOL NECache::selectObject(void *pObject, char *szKey, int iThreadID)
{
	NamedEntity					*neCurrent;							//  Current Named Entity Object

	//  Cast the passed object pointer to a Named Entity
	neCurrent = (NamedEntity *) pObject;

	//  Compare the supplied key against the arbitrary Identity in the object
	if (strcmpi(szKey, neCurrent->AID) == 0) return TRUE;
	return FALSE;
}

//  createObject
//
//  This function will create a new NamedEntity.
//
//  Parameters
//
//  1  -  char *			-  Pointer to a key string to be mateched against objects 
//  2  -  int				-  ID of the thread requesting the object
//
//  Returns
//
//  void *		-		A pointer to the new object
//
//  API Use Notes:
//
//  The contract for this interface asserts that an object is ALWAYS returned. In the case wher an
//  object cannot be created from the specified key then an empty or dummy object should be returned.
//
void *NECache::createObject(char *szKey, int iThreadID)
{
	NamedEntity					*neNew;							//  New Named Entity Object
	
	//  Create the new Named Entity
	neNew = new NamedEntity();

	//  Set the key in the object
	strcpy_s(neNew->AID, MAXUSERNAME, szKey);

	//  Mark the Arbitrary Identity as unresolved
	neNew->AIDAff = NE_ID_UNRESOLVED;

	//  Return the object
	return neNew;
}

//  cacheOut
//
//  This function will dispose of a Named Entity object from the cache.
//
//  Parameters
//
//  1  -  void *			-  Pointer to the object to be selected or not
//  2  -  BOOL				-  If TRUE then write each cache entry out
//  3  -  int				-  ID of the thread requesting the object
//
//  Returns
//
//
//  API Use Notes:
//
//
void NECache::cacheOut(void *pObject, BOOL bWrite, int iThreadID)
{
	NamedEntity					*neCurrent;							//  Current Named Entity Object

	//  Cast the passed object pointer to a Named Entity
	neCurrent = (NamedEntity *) pObject;

	//  Dispose of the object
	delete neCurrent;
}
