#pragma once
//  NamedEntity
//
//  This class provides a container for cached information stored and used by the NEResolver (Named Entity Resolver).
//
//  Message Identifier:      N/A
//  Message Range:           0000 - 0000
//  Trace area assignment:   N/A
//  DX Version:				 3.11.0
//  Version:                 3.11.0  Build: 98
//
//  Version History:
//
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2011
//

//  Platform Includes
#include "../Platform/PlatBase.h"
#include "../Platform/NotesBase.h"

//  Local Definitions

//  Arbitraty Identity Affinity
#define			NE_ID_EMPTY				1						//  The ID is empty
#define			NE_ID_UNRESOLVED		2						//  ID Affinity has not yet been resolved
#define			NE_ID_DOMINO			4						//  ID Affinity is to Domino
#define			NE_ID_AD				8						//  ID Affinity is to Active Directory
#define			NE_ID_EMAIL				16						//  ID Affinity is to e-Mail
#define			NE_ID_UNRESOLVEABLE		32						//  No affinity can be resolved for the entity

//  Distinguished Name levels of qualification
#define			NE_DN_EMPTY				1						//  The Distinguished Name is empty
#define			NE_DN_UNRESOLVED		2						//  The current level of the name has not been resolved
#define			NE_DN_NOTFOUND			4						//  The Name stored could not be found in it's respective directory
#define			NE_DN_UNQUALIFIED		8						//  The Name is not qualified
#define			NE_DN_PARTQUALIFIED		16						//  The Name is partially qualified
#define			NE_DN_FULLYQUALIFIED	32						//  The Name is fully qualified

//  Class definition for the NamedEntity class
class NamedEntity
{
public:
	//  Default Constructor
	NamedEntity(void);

	//  Destructor
	~NamedEntity(void);

	//  Public members

	//  Qualification States
	WORD		AIDAff;											//  Affinity of the Arbitrary ID
	WORD		DDNQ;											//  Qualification level of the Domino Distinguished Name
	WORD		ADDNQ;											//  Qualification level of the AD Distinguished Name
	WORD		EMDNQ;											//  Qualification level of the Email Address

	//  Item Caches
	char		ADUID[MAXUSERNAME + 1];							//  UID of the entity in the Active Directory
	char		ADLogin[MAXUSERNAME + 1];						//  Login name of the entity in Active Directory

	//  Entity IDs
	char		AID[MAXUSERNAME + 1];							//  Arbitrary ID
	char		DDN[MAXUSERNAME + 1];							//  Domino Distinguished Name
	char		ADDN[MAXUSERNAME + 1];							//  AD Distinguished Name
	char		EMAIL[MAXUSERNAME + 1];							//  Email Address

};

