#pragma once
//  Define base Language and Platform includes and definitions

//  Build Configuration - coerce instrumentation for DEBUG builds
#ifdef _DEBUG
#define		DXIP					1
#endif

//  Alignment Enforcement (all platforms)
#ifdef W64
#define		ALIGNMENT_BOUNDARY		8					//  Force 64 bit alignment
#else
#define		ALIGNMENT_BOUNDARY		4					//  Force 32 bit alignment
#endif

//  Windows WIN32/x64
#ifdef WIN32
#define		WIN32_LEAN_AND_MEAN							//  Exclude rarely-used parts of windows headers
#define		FORCE_ALIGN				__declspec(align(ALIGNMENT_BOUNDARY))		//  Enforced aliginment

#ifdef _DEBUG
#define _CRTDBG_MAP_ALLOC								//  Have file & line recorded in memory allocations
#include <stdlib.h>										//  Standard functions
#include <crtdbg.h>										//  Include the runtime debugging macros
#endif

//  Platform Includes
#include	<cstdlib>									//  Standard Namespaces
#include	<iostream>									//  Std I/O
#include	<time.h>									//  Time functions
#include	<process.h>									//  Native Windows MP
#include	<exception>									//  Exception handling
#include	<errno.h>									//  Error symbols
#include	<windows.h>									//  Windows Definitions

#ifdef _DEBUG
#define DEBUG_NEW new(_NORMAL_BLOCK, __FILE__, __LINE__)
#define new DEBUG_NEW
#endif

//  Macros
#define		SLEEP(x)	Sleep(x)						//  Sleep
#define		SLEEPSECONDS(x) Sleep(x * 1000)				//  Sleep fot x seconds
#ifdef _DEBUG
#define		CheckMemoryLeaks() _CrtDumpMemoryLeaks()	//  Runtime checks for memory leaks
#endif

//  Safer wrapper macros - this set of macros converts the safer (_s) string variants back to the less safe versions for pre VS2005 environments.
//  For VS2005 and up only also resurface some deprecated functions.
#if ((defined(_MSC_VER)) && (_MSC_VER >= 1400))
#define		strcmpi(t, s) _strcmpi(t, s)				//  Case insensitive string compare
#define		memcmpi(t, s, l) _memicmp(t, s, l)			//  Case insensitive buffer compare
#define		memicmp(t, s, l) _memicmp(t, s, l)			//  Case insensitive buffer compare
#else
//  Safer string/file handling
#define		strcpy_s(t, l, s) strcpy(t, s)
#define		sprintf_s(t, l, f, ...) sprintf(t, f,  __VA_ARGS__)
#define		strcat_s(t, l, s) strcat(t, s)
#define		strncpy_s(t, l, s, n) strncpy(t, s, n)
#define		strtok_s(s, t, c) strtok(s, t)
#define		memcpy_s(t, m, s, l) memcpy(t, s, l)
int fopen_s(FILE *, char *, char *);
inline int fopen_s(FILE **f, char *n, char *m)
{
	*f = fopen(n, m);
	return errno;
}
int ctime_s(char *, size_t, time_t *);
inline int ctime_s(char *s, size_t l, time_t *t)
{
	strcpy(s, ctime(t));
	return 0;
}
//  Compatability Types
#define		rsize_t		int
#define		errno_t		DWORD
#define		memcmpi(t, s, l) memicmp(t, s, l)
#endif
#endif

//  Compatability Symbolic Values
#ifndef EOK
#define		EOK		0
#endif

//  UNIX Generic
#ifdef UNIX
//  Global Settings
#define		_FILE_OFFSET_BITS		64				//  Use LFS always
#define		FORCE_ALIGN				__attribute__((aligned(ALIGNMENT_BOUNDARY)))	//  Enforced aliginment

//  Platform Includes
#include	<cstdlib>								//  Standard Namespaces
#include	<iostream>								//  Std I/O
#include	<stdint.h>								//  Integer definitions
#include	<stddef.h>								//  Standard definitions
#include	<sys/unistd.h>							//  Unix standard definitions
#include	<time.h>								//  Time functions
#include	<sys/time.h>							//  Time functions & structures
#include	<pthread.h>								//  POSIX MP
#include	<dlfcn.h>								//  DLL Management
#include	<errno.h>								//  Error Codes

//  Compatability Types
#define		rsize_t		int

//  Macros
#define		strcmpi(t, s) strcasecmp(t, s)			//  Case insensitive string compare
#define		memcmpi(t, s, l) strncasecmp(t, s, l)	//  Case insensitive characters compare
#define		memicmp(t, s, l) strncasecmp(t, s, l)	//  Case insensitive characters compare
#define		SLEEP(x)	usleep(x * 100)				//  Sleep
#define		SLEEPSECONDS(x) usleep(x * 1000000)		//  Sleep fot x seconds
#define		MAKEOFFSET(o, h, l) o = (h << 32) | l	//  Make a 64 bit off_t from two DWORDs

//  Compatability Macros
#ifndef		EOK
#define		EOK		0
#endif
#define		strcpy_s(t, l, s) strcpy(t, s)
#define		sprintf_s(t, l, f, ...) sprintf(t, f,  __VA_ARGS__)
#define		strcat_s(t, l, s) strcat(t, s)
#define		strncpy_s(t, l, s, n) strncpy(t, s, n)
#define		strtok_s(s, t, c) strtok(s, t)
#define		strcat_s(t, l, s) strcat(t, s)
#define		memcpy_s(t, m, s, l) memcpy(t, s, l)
int fopen_s(FILE *, char *, char *);
inline int fopen_s(FILE **f, char *n, char *m)
{
	*f = fopen(n, m);
	return errno;
}
int ctime_s(char *, size_t, time_t *);
inline int ctime_s(char *s, size_t l, time_t *t)
{
	strcpy(s, ctime(t));
	return 0;
}
//  Inline definition for DXAtomic_cmpxchg
unsigned long DXAtomic_cmpxchg(volatile void *pTarget, unsigned long ulOld, unsigned long ulNew);
inline unsigned long DXAtomic_cmpxchg(volatile void *pTarget, unsigned long ulOld, unsigned long ulNew)
{
	unsigned long ulOriginal;						//  Original value

    asm volatile("lock;cmpxchg %1,%2" 
                 : "=a"(ulOriginal) 
                 : "r"(ulNew), "m"(*(volatile long *)pTarget), "0"(ulOld) 
                 : "memory"); 

	return ulOriginal;
}
#endif

//  Frame Marker
//
//  r = literal string the name of the function being entered
//  t = address of the first vaiable in the frame stack
//  b = address of the message buffer (normally the last variable in the frame stack)
//
#define		MARK_FRAME(r, t, b)		sprintf_s(b, MAX_MSG, "*** %s *** TOF: @0x%p, BOF: @0x%p. ***", r, t, b)
