#pragma once
//  RunSettings
//
//  This class provides the base class for setting and providing the run settings for a DX application
//
//  Message Identifier:      N/A
//  Message Range:           0000 - 0000
//  Trace area assignment:   None
//  DX Version:				 3.11.1
//  Version:                 3.11.1  Build: 99
//
//  Version History:
//
//  3.10.1		15/10/2010		Asynch Transaction Handler added
//  3.10.9		04/06/2011		Some tidying up for Linux
//  3.11.1		22/10/2011		VS 2011 readiness
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2010 - 2011
//

//  Platform Includes
#include "Platform/PlatBase.h"
#include "Platform/NotesBase.h"

//  Application Includes
#include "DXGlobals.h"											//  DX Global Settings

class RunSettings
{
public:
	//  Constructor
	RunSettings(void);
	RunSettings(int argc, char * argv[]);

	//  Destructor
	~RunSettings(void);

	//  Public Members
	BOOL	AllowExecution;										//  Allow execution to continue
	BOOL	IsValid;											//  Run paramaters are valid or not
	BOOL	EchoLog;											//  Echo Log messages to the console or not
	BOOL	NoRepository;										//  Do not use the repository
	BOOL	NoAppLog;											//  Inhibit Application Event Logging (only Echo)
	BOOL	CreateRepository;									//  Create the Repository
	BOOL	UpdateDesign;										//  Update the design rather than create
	BOOL	SignDesign;											//  Sign the design elements
	BOOL	CreateOnDemand;										//  Create repository if it doesn't exist
	BOOL	RunningAsAddin;										//  Program is running as a server addin task
	BOOL	NeedsMQ;											//  Program needs a Message Queue
	BOOL	AllowMultipleAddins;								//  Allow multiple instances of an addin task
	int		LogLevel;											//  Current Logging Level
	int		TraceArea;											//  Area for tracing
	char	szRepServer[MAX_SERVER + 1];						//  Repository Server name
	char	szRepDb[MAX_DATABASE + 1];							//  Repository Database name
	char	APPName[MAXAPPNAME + 1];							//  Application Name
	char	APPTitle[MAXAPPTITLE + 1];							//  Application Title
	char	APPVer[MAXAPPVERSION + 1];							//  Application Version

private:
	//  Private Virtual Functions - implemented in inheriting classes
	BOOL virtual SetDefaults(void) = 0;								//  Set runtime defaults
	BOOL virtual ValidateParameters(int argc, char* argv[]) = 0;    //  Validate the parameters
	void virtual ShowUsage(void) = 0;								//  Show the program usage
	void virtual ShowSettings(void) = 0;							//  Show the current run settings
};
