//  ThreadManagerPolicy
//
//  This class provides storage for the policy settings that determine many of the running parameters
//  of the threading environment.
//
//  Message Identifier:      DXR
//  Message Range:           2001 - 2199
//  Trace area assignment:   TRACE_MT (02)
//  DX Version:				 3.9.10
//  Version:                 3.9.10  Build: 94
//
//  Version History:
//
//  1.0.0		07/06/2003		Initial version
//  3.9.0		24/07/2010		Linkage changed for logging & request posting
//								A CommandHandler can be hooked into the monitor processing
//  3.10.0		01/10/2010		Wildcard & multi-thread execution added to DX processing
//								DXNoteScanner incorporated in kernel
//  3.10.10		14/07/2011		Performance Improvements for/to DbCopier
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2003 - 2011
//

//  Application Includes
#include "ThreadManagerPolicy.h"

//  Default Constructor
ThreadManagerPolicy::ThreadManagerPolicy(void)
: TPoolPolicy(0)
, MaxThreads(0)
, MinThreads(0)
, TargetThreads(0)
{
	//  Set the default operating modes
	TPSchedMode = DEFAULT_SCHEDULE_MODE;								//  Set Scheduling Mode

	//  Set default timing values
	DispatcherWaitCycle = DEFAULT_DISPATCH_WAITINTERVAL;				//  Dispatcher cycle wait interval
	DispatcherWindowLimit = DEFAULT_DISPATCH_WINDOWLIMIT;				//  Dispatch window limit
	SchedulerPrioReviewLimit = DEFAULT_SCHED_PRIOREVIEWLIMIT;			//  Scheduler priority review limit
	SchedulerHealthCheckLimit = DEFAULT_SCHED_HEALTHCHECKLIMIT;			//  Scheduler health check limit
	SchedulerWaitCycle = DEFAULT_SCHED_WAITINTERVAL;					//  Scheduler cycle wait interval
	SchedulerProrityBoost = DEFAULT_SCHED_PROIBOOST;					//  Scheduler pririty boost increment
	RJPMutexWaitInterval = DEFAULT_RJPMUTEX_WAIT;						//  RJP Murex Wait
	RJPMutexWaitLimit = DEFAULT_RJPMUTEX_WAITLIMIT;						//  RJP Mutex timeout limit
	MonitorWaitCycle = DEFAULT_MON_WAITINTERVAL;						//  Monitor cycle wait interval
	MonitorAvailableInterval = DEFAULT_MON_AVAILEVALUATE;				//  Monitor Available Threads cycle
	MonitorPrefReviewCycle = DEFAULT_MON_PREFREVIEW;					//  Monitor preferencess cycle
	MonitorStratReviewCycle = DEFAULT_MON_STRATREVIEW;					//  Monitor strategy cycle
	RejoinSemFreeWait = DEFAULT_REJOIN_SEMWAIT;							//  Rejoin semaphore clear wait
	WorkerWaitCycle = DEFAULT_WORKER_WAITINTERVAL;						//  Worker cycle wait interval
	DispatchSemFreeWait = DEFAULT_DISPATCH_SEMWAIT;						//  Request dispatch semaphore clear wait
	LogSemFreeWait = DEFAULT_LOG_SEMWAIT;								//  Log semaphore clear wait
	APPMutexWaitInterval = DEFAULT_APPMUTEX_WAIT;						//  APP Murex Wait
	APPMutexWaitLimit = DEFAULT_APPMUTEX_WAITLIMIT;						//  APP Mutex timeout limit
	DBHMutexWaitInterval = DEFAULT_DBHMUTEX_WAIT;						//  DBH Murex Wait
	DBHMutexWaitLimit = DEFAULT_DBHMUTEX_WAITLIMIT;						//  DBH Mutex timeout limit
	FHTMutexWaitInterval = DEFAULT_FHTMUTEX_WAIT;						//  FHT Murex Wait
	FHTMutexWaitLimit = DEFAULT_FHTMUTEX_WAITLIMIT;						//  FHT Mutex timeout limit
	MaxPctL0Threads = 0;												//  CMLS Thread % quota
	MaxPctL1Threads = 0;												//  CMLS Thread % quota
	MaxPctL2Threads = 0;												//  CMLS Thread % quota
	MinPctPPFullRR = DEFAULT_RR_PPFULLPCT;								//  Rejoin recommendation trip level
	MinPctRPFullRR = DEFAULT_RR_RPFULLPCT;								//  Rejoin recommendation trip level
}

//  Destructor
ThreadManagerPolicy::~ThreadManagerPolicy(void)
{
}
