#pragma once
//  ThreadManagerPolicy
//
//  This class provides storage for the policy settings that determine many of the running parameters
//  of the threading environment.
//
//  Message Identifier:      DXR
//  Message Range:           2001 - 2199
//  Trace area assignment:   TRACE_MT (02)
//  DX Version:				 3.10.10
//  Version:                 3.10.10  Build: 94
//
//  Version History:
//
//  1.0.0		07/06/2003		Initial version
//  3.9.0		24/07/2010		Linkage changed for logging & request posting
//								A CommandHandler can be hooked into the monitor processing
//  3.10.0		01/10/2010		Wildcard & multi-thread execution added to DX processing
//								DXNoteScanner incorporated in kernel
//  3.10.10		14/07/2011		Performance Improvements for/to DbCopier
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2003 - 2011
//

//  Application Includes
#include "../ExecEnvironment.h"
#include "ThreadStructs.h"

//  Local definitions
#define		DEFAULT_SCHEDULE_MODE			TPOOL_MODE_USLS			//  Unconstrained Single Lane Scheduling
#define		DEFAULT_DISPATCH_WAITINTERVAL	10						//  Wait 10 Ms while in the running state
#define		DEFAULT_DISPATCH_WINDOWLIMIT	4						//  Dispatch window limit
#define		DEFAULT_SCHED_PRIOREVIEWLIMIT	15						//  Interval for performing priority reviews
#define		DEFAULT_SCHED_HEALTHCHECKLIMIT	50						//  Interval for performing health checks
#define		DEFAULT_SCHED_WAITINTERVAL		20						//  Wait 20 Ms while in the running state
#define		DEFAULT_SCHED_PROIBOOST			2						//  Prority boost +2
#define		DEFAULT_RJPMUTEX_WAIT			10						//  Wait 10 Ms for the RJP mutex
#define		DEFAULT_RJPMUTEX_WAITLIMIT		50						//  Timeout after 50 waits for the mutex
#define		DEFAULT_MON_AVAILEVALUATE		5						//  Monitor available threads every 5 cycles
#define		DEFAULT_MON_PREFREVIEW			125						//  Monitor prefs review every 125 cycles
#define		DEFAULT_MON_STRATREVIEW			500						//  Monitor strategy every 500 cycles
#define		DEFAULT_MON_WAITINTERVAL		20						//  Wait 20 Ms while in the running state
#define		DEFAULT_REJOIN_SEMWAIT			10						//  Wait 10 Ms for the Rejoin semaphore to clear
#define		DEFAULT_WORKER_WAITINTERVAL		20						//  Wait 20 Ms while in the running state
#define		DEFAULT_DISPATCH_SEMWAIT		10						//  Wait 10 Ms for the Request dispatch semaphore to clear
#define		DEFAULT_LOG_SEMWAIT				20						//  Wait 20 Ms for the Log semaphore to clear
#define		DEFAULT_RR_PPFULLPCT			70						//  Do not reccomend rejoin unti Pending pool is 70% full
#define		DEFAULT_RR_RPFULLPCT			70						//  Do not reccomend rejoin unti Rejoin pool is 70% full
#define		DEFAULT_APPMUTEX_WAIT			10						//  Wait 10 ms for an application mutex
#define		DEFAULT_APPMUTEX_WAITLIMIT		50						//  Timeout after 50 waits for the mutex
#define		DEFAULT_DBHMUTEX_WAIT			10						//  Wait 10 ms for an DBH mutex
#define		DEFAULT_DBHMUTEX_WAITLIMIT		50						//  Timeout after 50 waits for the mutex
#define		DEFAULT_FHTMUTEX_WAIT			10						//  Wait 10 ms for an FHT mutex
#define		DEFAULT_FHTMUTEX_WAITLIMIT		50						//  Timeout after 50 waits for the mutex

class ThreadManagerPolicy
{
public:
	//  Constructor
	ThreadManagerPolicy(void);

	//  Destructor
	~ThreadManagerPolicy(void);

	//  Public Members

	//  Operating modes & options
	UINT TPSchedMode;								//  Scheduling Mode
	UINT TPoolPolicy;								//  Thread Pool Policy
	UINT PriorityPolicy;							//  Prioritisation Policy

	//  Size and setup settings and constraints 
	int MaxThreads;									//  Maximum number of threads allowed
	int MinThreads;									//  Minimum number of threads allowed
	int TargetThreads;								//  Current Target number of threads
	int PendingRQECapacity;							//  Pending RQE Pool Size
	int RejoinRQECapacity;							//  Rejoin RQE Pool Size
	int AsyLogPoolEntries;							//  Asynch Log Pool Size
	int MaxPctL0Threads;							//  Max % threads alocated to L0 requests
	int MaxPctL1Threads;							//  Max % threads alocated to L1 requests
	int MaxPctL2Threads;							//  Max % threads alocated to L2 requests
	int MinPctPPFullRR;								//  Min % pending pool full before recommend rejoin
	int MinPctRPFullRR;								//  Min % rejoin pool full before recommend rejoin
	//  Timings
	int DispatcherWaitCycle;						//  Wait cycle for the dispatcher
	int SchedulerWaitCycle;							//  Wait cycle for the scheduler
	int MonitorWaitCycle;							//  Wait cycle for the monitor
	int WorkerWaitCycle;							//  Wait cycle for a worker thread
	int RJPMutexWaitInterval;						//  Wait time for the RJP Mutex
	int RJPMutexWaitLimit;							//  Wait limit for RJP Mutex
	int MonitorAvailableInterval;					//  Interval to re-evaluate the available thread count
	int MonitorPrefReviewCycle;						//  Interval to perform prefs review
	int MonitorStratReviewCycle;					//  Interval to perform strategy review
	int RejoinSemFreeWait;							//  Wait time for the Rejoin semaphore to clear
	int DispatchSemFreeWait;						//  Wait time for the Request dispatch semaphore to clear
	int LogSemFreeWait;								//  Wait time for the Log semaphore to clear
	int APPMutexWaitInterval;						//  Wait time for the APP Mutex
	int APPMutexWaitLimit;							//  Wait limit for APP Mutex
	int DBHMutexWaitInterval;						//  Wait time for the DBH Mutex
	int DBHMutexWaitLimit;							//  Wait limit for DBH Mutex
	int FHTMutexWaitInterval;						//  Wait time for the FHT Mutex
	int FHTMutexWaitLimit;							//  Wait limit for FHT Mutex

	//  Miscellaneous limits & Settings
	int DispatcherWindowLimit;						//  Dispatch window limit
	int SchedulerPrioReviewLimit;					//  Scheduler priority review limit
	int SchedulerHealthCheckLimit;					//  Scheduler health check limit
	int SchedulerProrityBoost;						//  Scheduler priority boost increment
};
