#pragma once
//  ThreadScheduler
//
//  This class provides the scheduling function for requests for execution that are submitteD to the
//  the Thread Pool manager queues.
//
//  Message Identifier:      DXR
//  Message Range:           2401 - 2499
//  Trace area assignment:   TRACE_MT (02)
//  DX Version:				 3.12.0
//  Version:                 3.12.0  Build: 106
//
//  Version History:
//
//  1.0.0		07/06/2003		Initial version
//  3.5.3		21/08/2009		Re-based ready for 4.0.0
//  3.6.1		30/09/2009		Fixed stats & counter integrity for dispatcher
//  3.6.2		20/11/2009		Unix compat changes plus minor fixes/enhancements
//  3.7.0		23/12/2009		Added race auto-correction to Scheduler
//  3.9.0		24/07/2010		Linkage changed for logging & request posting
//  3.11.1		22/10/2011		VS 2011 Updates
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2003 - 2012
//

//  Platform includes
#include "../Platform/PlatBase.h"

//  Application Includes
#include "../ExecEnvironment.h"								//  Runtime
#include "ThreadStructs.h"									//  Thread Structures
#include "ThreadManagerPolicy.h"							//  Thread Manager Policy

//  Local Definitions
#define		IDLE_WAITINTERVAL		100						//  Wait 100 Ms while in the idle state

//  Trace Messages
#define		MSG_DXR2400T			"DXR2400T: The scheduler thread has entered the not started state."
#define		MSG_DXR2401T			"DXR2401T: The scheduler thread has entered the running state."
#define		MSG_DXR2402T			"DXR2402T: The scheduler thread has entered the quitting state."
#define		MSG_DXR2404T			"DXR2404T: A new ThreadScheduler instance has been created @%p."
#define		MSG_DXR2405T			"DXR2405T: A ThreadScheduler instance @%p has been destroyed."
#define		MSG_DXR2407T			"DXR2407T: The scheduler thread has been drained."

//  Information Messages
#define		MSG_DXR2414I			"DXR2414I: Reclaiming RQE @%p as it has been dropped in the pending pool."

//  Error Messages
#define		MSG_DXR2403E			"DXR2403E: The scheduler thread has detected an invalid state change and is quitting."
#define		MSG_DXR2406E			"DXR2406E: The thread was unable to initialise the Notes runtime."
#define		MSG_DXR2409E			"DXR2409E: RQE Rejoin pool count has been adjusted from %i to %i, with %i RQEs Allocated."
#define		MSG_DXR2411E			"DXR2411E: The Rejoin Pool Mutex has been pre-empted by thread: %i."

class ThreadScheduler
{
public:
	//  Constructor
	ThreadScheduler(ExecEnvironment *xeParent, UCHAR *pTP, ThreadManagerPolicy *tpmParent);

	//  Destructor
	~ThreadScheduler(void);

	//  Public Members
	UCHAR				*pThreadPool;						//  Pointer to the Thread Pool

	//  Public Functions
#ifdef WIN32
	static UINT LNPUBLIC StartWrapper(void *pObject);		//  Wrapper for the Start Scheduler function
#endif

#ifdef UNIX
	static void * LNPUBLIC StartWrapper(void *pObject);		//  Wrapper for the Start Scheduler function
#endif

	UINT Start(void);										//  Start the Monitor

private:
	//  Private Members
	ExecEnvironment				*xeLocal;					//  Runtime execution environment
	ThreadManagerPolicy			*tmpLocal;					//  Thread Manager policy

	//  Private Functions
	void PerformPriorityReview(UCHAR *pTP);					//  Perform a review of priority
	void PerformSelfHealing(UCHAR *pTP);					//  Perform self healing functions
};
