#pragma once
//  Thread Manager Common Structures and definitions
//
//  This Header file contains the common structures and settings used by the Thread Manager components.
//
//  Message Identifier:      N/A
//  Message Range:           0000 - 0000
//  Trace area assignment:   TRACE_MT (02)
//  DX Version:				 3.12.0
//  Version:                 3.12.0  Build: 105
//
//  Version History:
//
//  1.0.0		10/02/2001		Initial version
//  3.5.3		21/08/2009		Re-based ready for 4.0.0
//  3.6.0		18/09/2009		New log handler for multithreaded applications
//  3.6.1		30/09/2009		Fixed stats & counter integrity for dispatcher
//  3.7.0		23/12/2009		Added race auto-correction to Scheduler
//  3.9.0		24/07/2010		Linkage changed for logging & request posting and rejoining
//  3.10.0		01/10/2010		Wildcard & multi-thread execution added to DX processing
//								DXNoteScanner incorporated in kernel
//  3.10.2		18/11/2010		MTIO support added to DbMover
//  3.10.10		14/07/2011		Performance Improvements for/to DbCopier
//  3.12.0		21/12/2011		DXIP introduced
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2001 - 2011
//

//  Local Definitions
#define			MAX_DBH_CACHE		50					//  Maximum number of native DBHANDLES
#define			MAX_FH_CACHE		50					//  Maximum number of native File Handles
#define			FREEHANDLE			-1					//  Special value set to cause local handles to be released

//  Return Codes
#define			THREAD_RC_OK				0					//  Ok
#define			THREAD_RC_ERROR				8					//  Error

//  Thread Manager states
#define			TMSTATE_INIT1				0x00000001			//  Stage 1 initialisation
#define			TMSTATE_INIT2				0x00000002			//  Stage 2 initialisation
#define			TMSTATE_INIT3				0x00000004			//  Stage 3 initialisation
#define			TMSTATE_ADJUST				0x00000008			//  Adjusting the number of available worker threads

//  Thread Monitor states
#define			TXSTATE_INIT				0x00000001			//  Initialisation
#define			TXSTATE_NOTSTARTED			0x00000002			//  Not Started (but initialisation completed)
#define			TXSTATE_START				0x00000004			//  Start up
#define			TXSTATE_RUNNING				0x00000008			//  Running
#define			TXSTATE_QUIT				0x00000010			//  Quit Requested
#define			TXSTATE_QUITTING			0x00000020			//  Quit in Progress
#define			TXSTATE_DRAINED				0x00000040			//  Drained

//  Thread Scheduler states
#define			TSSTATE_INIT				0x00000001			//  Initialisation
#define			TSSTATE_NOTSTARTED			0x00000002			//  Not Started (but initialisation completed)
#define			TSSTATE_START				0x00000004			//  Start up
#define			TSSTATE_RUNNING				0x00000008			//  Running
#define			TSSTATE_QUIT				0x00000010			//  Quit Requested
#define			TSSTATE_QUITTING			0x00000020			//  Quit in Progress
#define			TSSTATE_DRAINED				0x00000040			//  Drained

//  Thread Dispatcher states
#define			TDSTATE_INIT				0x00000001			//  Initialisation
#define			TDSTATE_NOTSTARTED			0x00000002			//  Not Started (but initialisation completed)
#define			TDSTATE_START				0x00000004			//  Start up
#define			TDSTATE_RUNNING				0x00000008			//  Running
#define			TDSTATE_QUIT				0x00000010			//  Quit Requested
#define			TDSTATE_QUITTING			0x00000020			//  Quit in Progress
#define			TDSTATE_DRAINED				0x00000040			//  Drained
#define			TDSTATE_BLOCKED				0x00000080			//  Blocked

//  Worker Thread states
#define			WTSTATE_UNALLOCATED			0x00000000			//  Not Allocated
#define			WTSTATE_INIT				0x00000001			//  Initialisation
#define			WTSTATE_NOTSTARTED			0x00000002			//  Not Started (but initialisation completed)
#define			WTSTATE_START				0x00000004			//  Start up
#define			WTSTATE_RUNNING				0x00000008			//  Running
#define			WTSTATE_QUIT				0x00000010			//  Quit Requested
#define			WTSTATE_QUITTING			0x00000020			//  Quit in Progress
#define			WTSTATE_DRAINED				0x00000040			//  Drained
#define			WTSTATE_POSTED				0x00000080			//  Work has been posted to this thread
#define			WTSTATE_BUSY				0x00000100			//  Busy
#define			WTSTATE_REJOIN				0x00000200			//  Wait for space in the Rejoin RQE Pool

//  Request Element States
#define			RQESTATE_UNALLOCATED		0x00000000			//  Not Allocated
#define			RQESTATE_ALLOCATED			0x00000001			//  Request is Allocated
#define			RQESTATE_READY				0x00000002			//  Request is Ready

//  Request Element Attributes
#define			RQATTR_FIREANDFORGET		0x00000001			//  Fire & Forget style request
#define			RQATTR_REJOIN				0x00000002			//  Rejoin is required for this request
#define			RQATTR_PRODUCER				0x00000004			//  Request will produce more requests
#define			RQATTR_MEGAPRODUCER			0x00000008			//  Request will produce lots more requests
#define			RQATTR_SERVICE				0x00000010			//  Request will initate a service
#define			RQATTR_MEGACONSUMER			0x00000020			//  Request will consume multiple requests
#define			RQATTR_CMLSL0				0x00000100			//  CMLS Level 0 (Service) Request
#define			RQATTR_CMLSL1				0x00000200			//  CMLS Level 1 (Scavenger/Feeder) Request
#define			RQATTR_CMLSL2				0x00000400			//  CMLS Level 2 (UOW) Request
#define			RQATTR_CMLSL3				0x00000800			//  CMLS Level 3(+) (Sub-Task) Request
#define			RQATTR_CMLSLCONSTRAINED		(RQATTR_CMLSL0 | RQATTR_CMLSL1 | RQATTR_CMLSL2)

//  Thread Pool Dispatch Modes
#define			TPOOL_MODE_USLS				0x00000001			//  Unconstrained Single Lane Scheduling
#define			TPOOL_MODE_CMLS				0x00000002			//  Constrained Multi Lane Scheduling

//  Thread Pool Policies
#define			TPOOL_POLICY_SLOWSTART		0x00000001			//  Slow startup of threads
#define			TPOOL_POLICY_ONDEMAND		0x00000002			//  Thread numbers are controlled on-demand
#define			TPOOL_POLICY_OBEYMAXMIN		0x00000004			//  Thread numbers are within Max & Min settings
#define			TPOOL_POLICY_MONITORUOW		0x00000008			//  UOW throughput is monitored
#define			TPOOL_POLICY_PRESTARTMIN	0x00000010			//  Prestart Min threads
#define			TPOOL_POLICY_PRESTARTTARGET	0x00000020			//  Prestart the initial target thread count

//  Request Prioritisation Policies
#define			PRIO_POLICY_AGERQS			0x00000001			//  Request Ageing is active
#define			PRIO_POLICY_PREFBOOST		0x00000002			//  Preference Boosting is active

//  Logger Controls
#define			LOG_CTL_FLUSH				0x00000001			//  Signal a flush for the logger

//  Structures

//  Threadpool Header
typedef struct {
	//  States of major components
	UINT			TManState;							//  Thread Manager State
	UINT			TSchedState;						//  Thread Scheduler State
	UINT			TMonState;							//  Thread Monitor State
	UINT			TDispState;							//  Thread Dispatcher State
	//  Thread management settings
	UINT			SchedMode;							//  Scheduler Mode
	UINT			PrioControl;						//  Priority control settings
	UINT			PrefSchedAttrs;						//  Preferred scheduler attributes
	int				MaxThreads;							//  Maximum Thread count
	int				MinThreads;							//  Minimum Thread count
	int				PendRQEPoolSize;					//  Size of the Pending Requests Pool
	int				RejoinRQEPoolSize;					//  Size of the Rejoin Requests Pool
	int				LogPoolSize;						//  Size of the Log Pool
	//  CMLS Constraints
	int				MaxL0Threads;						//  Max threads allocated to L0 Requests
	int				MaxL1Threads;						//  Max threads allocated to L1 Requests
	int				MaxL2Threads;						//  Max threads allocated to L2 Requests
	int				MaxL0PPRQEs;						//  Max Pend Pool entries for L0
	int				MaxL1PPRQEs;						//  Max Pend Pool entries for L1
	int				MaxL2PPRQEs;						//  Max Pend Pool entries for L2
	//  Lightweight mutexes
	FORCE_ALIGN volatile UINT	DBHTableMutex;			//  DBHandle Table Mutex
	FORCE_ALIGN volatile UINT	FHTableMutex;			//  File Handle Table Mutex
	FORCE_ALIGN volatile UINT	RejoinPoolMutex;		//  Rejoin Pool Mutex
	FORCE_ALIGN volatile UINT	AppMutex[10];			//  Array of application Mutexes
	//  Operational counters and settings
	int				CurrThreads;						//  Current number of active threads
	int				TargetThreads;						//  Target thread count
	int				AvailableThreads;					//  Count of available threads
	volatile int	PendRQECount;						//  Count of requests in the Pend Pool
	volatile int	RejoinRQECount;						//  Count of requests in the rejoin Pool
	int				CurrL0Threads;						//  Current threads allocated to L0 Requests
	int				CurrL1Threads;						//  Current threads allocated to L1 Requests
	int				CurrL2Threads;						//  Current threads allocated to L2 Requests
	int				CurrL0PPRQEs;						//  Current Pend Pool entries for L0
	int				CurrL1PPRQEs;						//  Current Pend Pool entries for L1
	int				CurrL2PPRQEs;						//  Current Pend Pool entries for L2
	//  Major storage pool pointers
	UCHAR			*pPendRQEPool;						//  Pointer to the Pending Requests Pool
	UCHAR			*pRejoinRQEPool;					//  Pointer to the Rejoin Requests Pool
	UCHAR			*pLogPool;							//  Pointer to the Log Pool
	UCHAR			*pDBHMappingPool;					//  Pointer to the Database Handle Mapping Pool
	UCHAR			*pFHMappingPool;					//  Pointer to the File Handle Mapping Pool
	//  Statistics Section
	TIMEDATE		tdRunStart;							//  Timestamp when the Thread Manager started
	DWORD			LogMsgDropped;						//  Count of Log Messages Dropped
	DWORD			LogMsgDiscarded;					//  Count of Log Messages Discarded
	DWORD			LogMsgOverflowed;					//  Count of Log Messages that overflowed the pool
	DWORD			LogMsgWritten;						//  Count of Log Messages Written (Posted)
	DWORD			LogMsgQueued;						//  Count of Log Messages Queued
	DWORD			BFDispatches;						//  Count of Brute Force dispatches
	DWORD			BFDispThreadsBusy;					//  Count of Brute Force Dispatches that failed bacause all threads busy
	DWORD			PDispatches;						//  Count of Priority dispatches
	DWORD			NoDispNoWork;						//  Count of no dispatched because there is no work
	DWORD			NoDispThreadsBusy;					//  Count of no dispatches because all threads are busy
	DWORD			DispBruteForce;						//  Count of requests dispatched by Brute Force
	DWORD			DispPriority;						//  Count of requests dispatched by priority
	DWORD			PDispQEmpty;						//  Count of priority dispatch windows closed because Queue empty
	DWORD			PDispThreadsBusy;					//  Count of priority dispatch windows closed because all threads busy
	DWORD			RJRDequeued;						//  Count of Rejoin RQEs dequeued
	DWORD			RJRBusy;							//  Count of Rejoin Request Busy returns
	DWORD			PPFullWaits;						//  Count of waits on the Pending Pool being full
	DWORD			PPFullBackoffs;						//  Count of backoffs because the pending pool is full
	DWORD			PPLaneFullBackoffs;					//  Count of backoffs because the pending pool lane is full
	DWORD			CMLSDenials;						//  Count of CMLS dispatch denials
	DWORD			RejoinRecs;							//  Count of requests for rejoin recommendations
	DWORD			RejoinRecd;							//  Count of positive rejoin recommendations
//  Mutex activity counters
	DWORD			LogMutexAcqs;						//  Count of acquire calls
	DWORD			LogMutexWaits;						//  Count of waits on the log mutex
	DWORD			LogMutexFails;						//  Count of failures (timeout)
	DWORD			DBHTMutexAcqs;						//  Count of acquire calls
	DWORD			DBHTMutexWaits;						//  Count of waits on the Db Handle Table mutex
	DWORD			DBHTMutexFails;						//  Count of failures (timeout)
	DWORD			FHTMutexAcqs;						//  Count of acquire calls
	DWORD			FHTMutexWaits;						//  Count of waits on the File Handle Table mutex
	DWORD			FHTMutexFails;						//  Count of failures (timeout)
	DWORD			RJPMutexAcqs;						//  Count of acquire calls
	DWORD			RJPMutexWaits;						//  Count of waits on the Rejoin Pool mutex
	DWORD			RJPMutexFails;						//  Count of failures (timeout)
	DWORD			HeapMutexAcqs;						//  Count of acquire calls
	DWORD			HeapMutexWaits;						//  Count of waits on the System Heap mutex
	DWORD			HeapMutexFails;						//  Count of failures (timeout)
	DWORD			AppMutexAcqs[10];					//  Count of acquire calls
	DWORD			AppMutexWaits[10];					//  Count of waits on the Application mutexes
	DWORD			AppMutexFails[10];					//  Count of failures (timeout)	
//  Exception guard counters
	DWORD			Guard01;							//  Count of guard 1 exceptions
	DWORD			Guard02;							//  Count of guard 2 exceptions
	DWORD			Guard03;							//  Count of guard 3 exceptions
	DWORD			Guard04;							//  Count of guard 4 exceptions
	DWORD			Guard05;							//  Count of guard 5 exceptions
	DWORD			Guard06;							//  Count of guard 6 exceptions
	DWORD			Guard07;							//  Count of guard 7 exceptions
	DWORD			Guard08;							//  Count of guard 8 exceptions
	DWORD			Guard09;							//  Count of guard 9 exceptions
	DWORD			Guard10;							//  Count of guard 10 exceptions
} Threadpool_Header;

//  Request Queue Element (RQE)
typedef struct {
	volatile UINT	RQEState;							//  State of the request
	UINT			RQEAttrs;							//  Attributes of this RQE
	int				RQEPriority;						//  Priority
	int				RQEInspectCount;					//  Inspection count
	void far		*pOObject;							//  Pointer to the Owner Object
	void far		*pXObject;							//  Pointer to the Executable Object
	void far		*pPObject;							//  Pointer to the Parameter Object
} RQElement;

#ifdef DXIP
#define		IPC_STACK_SIZE				10				//  Size of the instrumentation package record stack

//  Instrumentation Package - Record types
#define		DXIPR_RQE_COMPLETED			1				//  RQE Completed

//  Instrumentation Package - Phase Change Record
typedef struct {
	int				Type;								//  Phase Change Type
	int				Size;								//  Size of this record
	int				TID;								//  Thread ID
	DWORD			EventNo;							//  Event Number
	DWORD			EventTime;							//  Relative time of this event
} PCRec;
//  Instrumentation Package - Request Completed Record
typedef struct {
	PCRec			PCBase;								//  Phase Change base record
	int				DPriority;							//  Dispatch Priority
	DWORD			Dwell;								//  Dwell time prior to dispatch
	DWORD			Mill;								//  Execution Time
} PCRecRC;
#endif

//  Threadpool Member
typedef struct {
	UINT			TMemState;							//  Thread State
	void far		*pTMemObject;						//  Pointer to the thread member object
	HANDLE			hThread;							//  Thread Handle
	UINT			uiPThreadID;						//  O/S Thread ID
	RQElement		rqePosted;							//  The request posted to this thread
	STATUS			stLastOpenError;					//  Last D/B Open error code
	//  Statistics section
	DWORD			dwReqsProc;							//  Count of requests processed
	//  Stats & semaphore section for request dispatch interface
	DWORD			dwReqDisp;							//  Count of requests to dispatch
	DWORD			dwReqDispWaits;						//  Count of requests to dispatch semaphore waits
	DWORD			dwReqDispWaitQuanta;				//  Count of requests to dispatch semaphore wait intervals
	volatile UINT	semReqDisp;							//  Request to dispatch semaphore
	RQElement		rqeDisp;							//  The request to be dispatched
	//  Stats & semaphore section for request rejoin interface
	DWORD			dwRejoin;							//  Count of requests to rejoin
	DWORD			dwRejoinWaits;						//  Count of requests to rejoin semaphore waits
	DWORD			dwRejoinWaitQuanta;					//  Count of requests to rejoin semaphore wait intervals
	volatile UINT	semRejoin;							//  Request to rejoin semaphore
	RQElement		rqeRejoin;							//  The request to be dispatched
	//  Stats & semaphore section for request rejoin polling interface
	DWORD			dwRejoinPoll;						//  Count of requests to rejoin
	DWORD			dwRejoinReturned;					//  Count of rejoins returned
	DWORD			dwRejoinNotReady;					//  Count of rejoins Not Ready
	DWORD			dwRejoinNone;						//  Count of rejoins None Exist
	DWORD			dwRejoinPollWaits;					//  Count of requests to rejoin semaphore waits
	DWORD			dwRejoinPollWaitQuanta;				//  Count of requests to rejoin semaphore wait intervals
	volatile UINT	semRejoinPoll;						//  Request to rejoin semaphore
	RQElement		rqeRejoinPoll;						//  The request to be dispatched
	int				rcRejoinPoll;						//  Return code state from rejoin poll operation
	//  Stats & semaphore section for logging interface
	DWORD			dwLogMsgsPosted;					//  Count of log messages posted
	DWORD			dwLogSemWaits;						//  Count of log message semaphore waits
	DWORD			dwLogSemWaitQuanta;					//  Count of log message semaphore wait intervals
	volatile UINT	semLogMsg;							//  Log message semaphore
	TIMEDATE		tdLogMsg;							//  Log message timestamp
	char			szLogMsg[MAX_MSG + 2];				//  Posted log message
#ifdef DXIP
	//  Instrumentation Package Section
	DWORD			tToIdle;							//  To Idle last transition time
	DWORD			tToActive;							//  To Active last ransition time
	DWORD			tRetToIdle;							//  Return to Idle time
	DWORD			EventCount;							//  Event Counter
	DWORD			LastCollection;						//  Last collection number
	PCRecRC			CStack[IPC_STACK_SIZE];				//  Collection Stack
#endif
} Threadpool_Member;

//  Log Pool Header
typedef struct {
	UINT			LogQueueControl;					//  Control Signals for the log queue
	int				LogPoolEntries;						//  Current count of entries
	int				LogQueueHead;						//  Index of the oldest entry in the pool
	int				LogQueueTail;						//  Index of the newest entry in the pool
} Logpool_Header;

//  Log Entry
typedef struct {
	int				LogThread;							//  Thread ID of the posting thread
	TIMEDATE		tdEventTime;						//  Timestamp of the log event
	char			szLogMsg[MAX_MSG + 2];				//  Message Text
} LogEntry;
